@@ -492,7 +492,7 @@ bouboule bounce braid bubble bug \
 cage cartoon clock coral crystal \
 daisy dclock deco demon dilemma drift eyes \
 fadeplot flag flame forest \
-galaxy gears geometry grav \
+galaxy gears grav \
 helix hop hyper \
 ico ifs image julia kaleid \
 laser life life1d life3d \
