@@ -1,4 +1,4 @@
-#!/usr/bin/perl -T -w
+#!/usr/bin/perl
 # This is used  to add life to life
 # This is a QUICK hack to convert life files to xlock's life format.
 # Patterns MUST have <= 64 pts at start for life.c to use the data generated
@@ -14,8 +14,6 @@
 #..........****........
 #..........*...........
 
-local($PTS, $X, $Y);
-
 print "
 Drop these points in life.c, within the 'patterns' array.
 Note if the number of points > 64, one must increase points NUMPTS;
@@ -28,7 +26,7 @@ sub search {
     local ($row, $col, $firstrow, $firstcol);
     local ($i, $j, $found, $c, $tempx, $tempy);
     local (@array);
-
+  
 
     $row = $col = 0;
     $firstrow = -1;
@@ -63,7 +61,7 @@ sub search {
     }
     $col = $X - $firstcol + 1;
     $row = $Y;
-    print "	{\n		";
+    print "    {\n        ";
     for ($j = 0; $j <= $Y; $j++) {
         $found = 0;
         for ($i = 0; $i <= $X; $i++) {
@@ -75,9 +73,9 @@ sub search {
             }
         }
         if ($found) {
-            print "\n		";
+            print "\n        ";
         }
   }
-  print "127\n	},\n";
+  print "127\n    },\n";
   $X = $col;
 }
