@@ -225,10 +225,9 @@ XCOMM    *** BEGIN MODULES CONFIG SECTION **
 XCOMM This is a HIGHLY experimental section.
 XCOMM To use modules, you must have an ELF setup installed, and
 XCOMM libdl.  This was originally written for Linux.
-XCOMM /usr/include/dlfcn.h is no longer found there at least on 2.0.0
-XCOMM Anybody have any ideas?  So I will try to get this running on
-XCOMM Solaris-2.6 with gcc and gmake first.  I don't know if
-XCOMM other Unices support this kind of dynamic loading.
+XCOMM /usr/include/dlfcn.h is not present on all Linux machines.
+XCOMM Also tried to get this running on XCOMM Solaris-2.6 with gcc and gmake.
+XCOMM Do not know if other Unixes support this kind of dynamic loading.
 XCOMM Un-commenting Modules will compile xlock so that all of the modes
 XCOMM are in separate modules which are loaded at runtime.  This means
 XCOMM you can add new modes simply by putting them in your modules
@@ -302,7 +301,11 @@ PASSWDDEF = -DHAVE_SHADOW
 
 #ifdef SunArchitecture
 #if SystemV4
+#if OSMajorVersion == 2 && OSMinorVersion < 5
+SYSTEMDEF = -DLESS_THAN_SOLARIS2_5 -DSOLARIS2
+#else
 SYSTEMDEF = -DSOLARIS2
+#endif
 XCOMM imake is usually not set up right here.
 XCOMM Assume shadowing... usually more correct.
 XCOMM  #if HasShadowPasswd
@@ -417,6 +420,7 @@ XCOMM                          window managers have problems)
 XCOMM -DALWAYS_ALLOW_ROOT      Users can not turn off allowroot
 XCOMM -DUSE_SYSLOG             Paranoid administrator option (a check is also
 XCOMM                          done to see if you have it)
+XCOMM -DUSE_MULTIPLE_USER      Multiple users ... security?
 XCOMM -DUSE_MULTIPLE_ROOT      Multiple root users ... security?
 XCOMM -DUSE_WINDOW_VISIBILITY  Allow a window to be placed over xlock
 XCOMM -DUSE_OLD_EVENT_LOOP     Some machines may still need this (fd_set
@@ -444,6 +448,7 @@ OPTDEF = -DUSE_VROOT -DALWAYS_ALLOW_ROOT -DUSE_BOMB
 XCOMM  OPTDEF += -DUSE_SYSLOG -DSYSLOG_FACILITY=LOG_AUTH
 XCOMM  OPTDEF += -DSYSLOG_WARNING=LOG_WARNING
 XCOMM  OPTDEF += -DSYSLOG_NOTICE=LOG_NOTICE -DSYSLOG_INFO=LOG_INFO
+XCOMM  OPTDEF += -DUSE_MULTIPLE_USER
 XCOMM  OPTDEF += -DUSE_MULTIPLE_ROOT
 XCOMM  OPTDEF += -DUSE_OLD_EVENT_LOOP
 XCOMM  OPTDEF += -DUSE_XLOCKRC
