@@ -5,7 +5,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ant.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)ant.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -65,12 +65,11 @@ static const char sccsid[] = "@(#)ant.c	4.04 97/07/28 xlockmore";
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-
 #endif /* STANDALONE */
 
-#define DEF_TRUCHET  "False"
-
 extern int  neighbors;
+
+#define DEF_TRUCHET  "False"
 static Bool truchet;
 
 static XrmOptionDescRec opts[] =
@@ -88,8 +87,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt ant_opts =
-{2, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
 
+#ifdef USE_MODULES
+ModStruct   ant_description =
+{"ant", "init_ant", "draw_ant", "release_ant",
+ "refresh_ant", "init_ant", NULL, &ant_opts,
+ 1000, -3, 40000, -7, 64, 1.0, "",
+ "Shows Langton's and Turk's generalized ants", 0, NULL};
+
+#endif
 
 #define ANTBITS(n,w,h)\
   ap->pixmaps[ap->init_bits++]=\
@@ -97,29 +104,13 @@ ModeSpecOpt ant_opts =
 
 /* If you change the table you may have to change the following 2 constants */
 #define STATES 2
-#define COLORS 11
 #define MINANTS 1
-#define PATTERNSIZE 8
 #define REDRAWSTEP 2000		/* How much tape to draw per cycle */
 #define MINGRIDSIZE 24
 #define MINSIZE 1
 #define ANGLES 360
 #define NEIGHBORKINDS 3
 
-static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
-{
-	{0x11, 0x22, 0x11, 0x22, 0x11, 0x22, 0x11, 0x22},	/* grey+white | stripe */
-	{0x00, 0x66, 0x66, 0x00, 0x00, 0x66, 0x66, 0x00},	/* spots */
-	{0x89, 0x44, 0x22, 0x11, 0x88, 0x44, 0x22, 0x11},	/* lt. / stripe */
-	{0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66},	/* | bars */
-	{0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa},	/* 50% grey */
-	{0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00},	/* - bars */
-	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x76},	/* dark \ stripe */
-	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
-	{0xaa, 0xff, 0xff, 0x55, 0xaa, 0xff, 0xff, 0x55},	/* black+grey - stripe */
-	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}	/* black */
-};
-
 typedef struct {
 	unsigned char color;
 	short       direction;
@@ -133,7 +124,6 @@ typedef struct {
 } antstruct;
 
 typedef struct {
-	int         init_bits;
 	int         neighbors;
 	int         generation;
 	int         xs, ys;
@@ -144,18 +134,21 @@ typedef struct {
 	int         n;
 	int         redrawing, redrawpos;
 	int         truchet;	/* Only for Turk modes */
-	statestruct machine[COLORS * STATES];
+	statestruct machine[NUMSTIPPLES * STATES];
 	unsigned char *tape;
 	unsigned char *truchet_state;
 	antstruct  *ants;
-	unsigned char colors[COLORS - 1];
+	int         init_bits;
+	unsigned char colors[NUMSTIPPLES - 1];
 	GC          stippledGC;
-	Pixmap      pixmaps[COLORS - 1];
-	XPoint      hexagonList[7];
-	XPoint      triangleList[2][4];
+	Pixmap      pixmaps[NUMSTIPPLES - 1];
+	union {
+		XPoint      hexagon[7];		/* Need more than 6 for truchet */
+		XPoint      triangle[2][4];	/* Need more than 3 for truchet */
+	} shape;
 } antfarmstruct;
 
-static int  initVal[NEIGHBORKINDS] =
+static char plots[NEIGHBORKINDS] =
 {3, 4, 6};			/* Neighborhoods, 8 just makes a mess */
 
 
@@ -190,7 +183,7 @@ static antfarmstruct *antfarms = NULL;
 /* Even runs of 0's and 1's are also symmetric */
 /* I have seen Hexagonal builders but they are more rare. */
 
-static unsigned char tables[][3 * COLORS * STATES + 2] =
+static unsigned char tables[][3 * NUMSTIPPLES * STATES + 2] =
 {
 #if 0
   /* Here just so you can figure out notation */
@@ -449,33 +442,33 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 	if (ap->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		ap->hexagonList[0].x = ap->xb + ccol * ap->xs;
-		ap->hexagonList[0].y = ap->yb + crow * ap->ys;
+		ap->shape.hexagon[0].x = ap->xb + ccol * ap->xs;
+		ap->shape.hexagon[0].y = ap->yb + crow * ap->ys;
 		if (ap->xs == 1 && ap->ys == 1)
 			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			   ap->hexagonList[0].x, ap->hexagonList[0].y, 1, 1);
+			ap->shape.hexagon[0].x, ap->shape.hexagon[0].y, 1, 1);
 		else
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			      ap->hexagonList, 6, Convex, CoordModePrevious);
+			    ap->shape.hexagon, 6, Convex, CoordModePrevious);
 	} else if (ap->neighbors == 4 || ap->neighbors == 8) {
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 		ap->xb + ap->xs * col, ap->yb + ap->ys * row, ap->xs, ap->ys);
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
 
-		ap->triangleList[orient][0].x = ap->xb + col * ap->xs;
-		ap->triangleList[orient][0].y = ap->yb + row * ap->ys;
+		ap->shape.triangle[orient][0].x = ap->xb + col * ap->xs;
+		ap->shape.triangle[orient][0].y = ap->yb + row * ap->ys;
 		if (ap->xs <= 3 || ap->ys <= 3)
 			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-			 ((orient) ? -1 : 1) + ap->triangleList[orient][0].x,
-				       ap->triangleList[orient][0].y, 1, 1);
+			((orient) ? -1 : 1) + ap->shape.triangle[orient][0].x,
+				       ap->shape.triangle[orient][0].y, 1, 1);
 		else {
 			if (orient)
-				ap->triangleList[orient][0].x += (ap->xs / 2 - 1);
+				ap->shape.triangle[orient][0].x += (ap->xs / 2 - 1);
 			else
-				ap->triangleList[orient][0].x -= (ap->xs / 2 - 1);
+				ap->shape.triangle[orient][0].x -= (ap->xs / 2 - 1);
 			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
-				     ap->triangleList[orient], 3, Convex, CoordModePrevious);
+				     ap->shape.triangle[orient], 3, Convex, CoordModePrevious);
 		}
 	}
 }
@@ -495,11 +488,11 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 		hex.y = ap->yb + crow * ap->ys - (int) ((double) ap->ys * 1.6 / 2.0);
 		for (side = 0; side < 6; side++) {
 			if (side > 0) {
-				hex.x += ap->hexagonList[side].x;
-				hex.y += ap->hexagonList[side].y;
+				hex.x += ap->shape.hexagon[side].x;
+				hex.y += ap->shape.hexagon[side].y;
 			}
-			hex2.x = hex.x + ap->hexagonList[side + 1].x / 2;
-			hex2.y = hex.y + ap->hexagonList[side + 1].y / 2;
+			hex2.x = hex.x + ap->shape.hexagon[side + 1].x / 2;
+			hex2.y = hex.y + ap->shape.hexagon[side + 1].y / 2;
 			if (truchetstate == side % 3)
 				/* Crude approx of 120 deg, so it will not cause drawing errors. */
 				XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
@@ -545,8 +538,8 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 		}
 		for (side = 0; side < 3; side++) {
 			if (side > 0) {
-				tri.x += ap->triangleList[orient][side].x;
-				tri.y += ap->triangleList[orient][side].y;
+				tri.x += ap->shape.triangle[orient][side].x;
+				tri.y += ap->shape.triangle[orient][side].y;
 			}
 			if (truchetstate == side % 3) {
 				if (orient)
@@ -566,10 +559,10 @@ static void
 drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 {
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
-  GC gc;
+	GC          gc;
 
 	if (!color) {
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 		gc = MI_GC(mi);
 	} else if (MI_NPIXELS(mi) > 2) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
@@ -579,8 +572,8 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 		XGCValues   gcv;
 
 		gcv.stipple = ap->pixmaps[color - 1];
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		XChangeGC(MI_DISPLAY(mi), ap->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		gc = ap->stippledGC;
@@ -595,18 +588,18 @@ drawtruchet(ModeInfo * mi, int col, int row,
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 
 	if (!color)
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
 	else if (MI_NPIXELS(mi) > 2 || color > ap->ncolors / 2)
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 	else
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
 	truchetcell(mi, col, row, truchetstate);
 }
 
 static void
 draw_anant(ModeInfo * mi, int col, int row)
 {
-	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
 	fillcell(mi, MI_GC(mi), col, row);
 #if 0				/* Can not see eyes */
 	{
@@ -616,7 +609,7 @@ draw_anant(ModeInfo * mi, int col, int row)
 
 		if (ap->xs > 2 && ap->ys > 2) {		/* Draw Eyes */
 
-			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
 			switch (direction) {
 				case 0:
 					XDrawPoint(display, window, MI_GC(mi),
@@ -747,10 +740,12 @@ getTurk(ModeInfo * mi, int i)
 		ap->machine[j].next = 0;
 		power2 >>= 1;
 	}
-	if (ap->neighbors != 3 && ap->neighbors != 4 && ap->neighbors != 6)
-		ap->truchet = False;
-	else if (truchet)
-		ap->truchet = True;
+	if (MI_WIN_IS_FULLRANDOM(mi))
+		ap->truchet = (Bool) (LRAND() & 1);
+	else
+		ap->truchet = truchet;
+	ap->truchet = (ap->truchet && ap->xs > 2 && ap->ys > 2 &&
+	   (ap->neighbors == 3 || ap->neighbors == 4 || ap->neighbors == 6));
 	if (MI_WIN_IS_VERBOSE(mi))
 		(void) fprintf(stdout, "neighbors %d, Turk's number %d, colors %d\n",
 			       ap->neighbors, number, ap->ncolors);
@@ -762,7 +757,6 @@ init_ant(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	int         size = MI_SIZE(mi);
-	XGCValues   gcv;
 	antfarmstruct *ap;
 	int         col, row, i, dir;
 
@@ -775,12 +769,14 @@ init_ant(ModeInfo * mi)
 	ap->redrawing = 0;
 	if (MI_NPIXELS(mi) <= 2) {
 		if (ap->stippledGC == None) {
+			XGCValues   gcv;
+
 			gcv.fill_style = FillOpaqueStippled;
 			ap->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
 		}
 		if (ap->init_bits == 0) {
-			for (i = 0; i < COLORS - 1; i++)
-				ANTBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+			for (i = 1; i < NUMSTIPPLES; i++)
+				ANTBITS(stipples[i], STIPPLESIZE, STIPPLESIZE);
 		}
 	}
 	ap->generation = 0;
@@ -802,12 +798,12 @@ init_ant(ModeInfo * mi)
 		ap->neighbors = neighbors;	/* Discourage but not deny use... */
 	else
 		for (i = 0; i < NEIGHBORKINDS; i++) {
-			if (neighbors == initVal[i]) {
-				ap->neighbors = initVal[i];
+			if (neighbors == plots[i]) {
+				ap->neighbors = plots[i];
 				break;
 			}
 			if (i == NEIGHBORKINDS - 1) {
-				ap->neighbors = initVal[NRAND(NEIGHBORKINDS)];
+				ap->neighbors = plots[NRAND(NEIGHBORKINDS)];
 				break;
 			}
 		}
@@ -838,8 +834,8 @@ init_ant(ModeInfo * mi)
 		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs / 2;
 		ap->yb = (ap->height - ap->ys * (ncrows / 2) * 2) / 2 + ap->ys;
 		for (i = 0; i < 7; i++) {
-			ap->hexagonList[i].x = (ap->xs - 1) * hexagonUnit[i].x;
-			ap->hexagonList[i].y = ((ap->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			ap->shape.hexagon[i].x = (ap->xs - 1) * hexagonUnit[i].x;
+			ap->shape.hexagon[i].y = ((ap->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
 		}
 	} else if (ap->neighbors == 4 && ap->neighbors == 8) {
 		if (size < -MINSIZE)
@@ -883,20 +879,21 @@ init_ant(ModeInfo * mi)
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2 + ap->ys;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 4; i++) {
-				ap->triangleList[orient][i].x =
+				ap->shape.triangle[orient][i].x =
 					(ap->xs - 2) * triangleUnit[orient][i].x;
-				ap->triangleList[orient][i].y =
+				ap->shape.triangle[orient][i].y =
 					(ap->ys - 2) * triangleUnit[orient][i].y;
 			}
 		}
 	}
-	XClearWindow(display, MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
 
 	/* Exclude odd # of neighbors, stepping forward not defined */
-	if (!NRAND(COLORS) && ((ap->neighbors + 1) % 2)) {
+	if (!NRAND(NUMSTIPPLES) && ((ap->neighbors + 1) % 2)) {
 		getTable(mi, (int) (NRAND(NTABLES)));
 	} else
-		getTurk(mi, (int) (NRAND(COLORS - 1)));
+		getTurk(mi, (int) (NRAND(NUMSTIPPLES - 1)));
 	if (MI_NPIXELS(mi) > 2)
 		for (i = 0; i < (int) ap->ncolors - 1; i++)
 			ap->colors[i] = (NRAND(MI_NPIXELS(mi)) +
