@@ -2,7 +2,7 @@
 /* blot --- Rorschach's ink blot test */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)blot.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)blot.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -53,12 +53,20 @@ static const char sccsid[] = "@(#)blot.c	4.04 97/07/28 xlockmore";
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-
 #endif /* STANDALONE */
 
 ModeSpecOpt blot_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   blot_description =
+{"blot", "init_blot", "draw_blot", "release_blot",
+ "refresh_blot", "init_blot", NULL, &blot_opts,
+ 200000, 6, 30, 1, 64, 0.4, "",
+ "Shows Rorschach's ink blot test", 0, NULL};
+
+#endif
+
 typedef struct {
 	int         width;
 	int         height;
@@ -112,8 +120,8 @@ init_blot(ModeInfo * mi)
 		bp->pointBufferSize = bp->size * sizeof (XPoint);
 		bp->pointBuffer = (XPoint *) malloc(bp->pointBufferSize);
 	}
-	XClearWindow(display, MI_WINDOW(mi));
-	XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+	MI_CLEARWINDOW(mi);
+	XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
 	bp->count = 0;
 }
 
@@ -185,5 +193,5 @@ release_blot(ModeInfo * mi)
 void
 refresh_blot(ModeInfo * mi)
 {
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
