@@ -2,7 +2,7 @@
 /* bouboule --- glob of spheres twisting and changing size */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bouboule.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)bouboule.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -72,7 +72,7 @@ static const char sccsid[] = "@(#)bouboule.c	4.04 97/07/28 xlockmore";
 #define HACK_INIT init_bouboule
 #define HACK_DRAW draw_bouboule
 #define bouboule_opts xlockmore_opts
-#define DEFAULTS "*delay: 5000 \n" \
+#define DEFAULTS "*delay: 10000 \n" \
  "*count: 100 \n" \
  "*size: 15 \n" \
  "*ncolors: 64 \n" \
@@ -92,6 +92,15 @@ static const char sccsid[] = "@(#)bouboule.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt bouboule_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   bouboule_description =
+{"bouboule", "init_bouboule", "draw_bouboule", "release_bouboule",
+ "refresh_bouboule", "init_bouboule", NULL, &bouboule_opts,
+ 10000, 100, 1, 15, 64, 1.0, "",
+ "Shows Mimi's bouboule of moving stars", 0, NULL};
+
+#endif
+
 #define USEOLDXARCS  1		/* If 1, we use old xarcs list for erasing.
 				   * else we just roughly erase the window.
 				   * This mainly depends on the number of stars,
@@ -327,11 +336,10 @@ init_bouboule(ModeInfo * mi)
 
 	/* use the right `black' pixel values: */
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi)) {
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
-		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			       0, 0, sp->width, sp->height);
-	} else
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+		MI_CLEARWINDOWCOLOR(mi, MI_NONE_COLOR(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
+	}
 
 	if (size < -MINSIZE)
 		sp->max_star_size = NRAND(-size - MINSIZE + 1) + MINSIZE;
@@ -471,15 +479,19 @@ init_bouboule(ModeInfo * mi)
 		else
 			star->size -= sp->max_star_size;
 
-		/* We set default values for the XArc lists elements */
-		arc->x = arc->y = 0;
+		/* We set default values for the XArc lists elements, but offscreen */
+		arc->x = MI_WIN_WIDTH(mi);
+		arc->y = MI_WIN_HEIGHT(mi);
 		if (MI_WIN_IS_USE3D(mi)) {
-			arcleft->x = arcleft->y = 0;
+			arcleft->x = MI_WIN_WIDTH(mi);
+			arcleft->y = MI_WIN_HEIGHT(mi);
 		}
 #if ((USEOLDXARCS == 1) || (ADAPT_ERASE == 1))
-		oarc->x = oarc->y = 0;
+		oarc->x = MI_WIN_WIDTH(mi);
+		oarc->y = MI_WIN_HEIGHT(mi);
 		if (MI_WIN_IS_USE3D(mi)) {
-			oarcleft->x = oarcleft->y = 0;
+			oarcleft->x = MI_WIN_WIDTH(mi);
+			oarcleft->y = MI_WIN_HEIGHT(mi);
 		}
 #endif
 		arc->width = 2 + star->size;
@@ -520,7 +532,7 @@ init_bouboule(ModeInfo * mi)
 	if (!MI_WIN_IS_USE3D(mi) && MI_NPIXELS(mi) > 2)
 		sp->color = MI_PIXEL(mi, sp->colorp);
 	else
-		sp->color = MI_WIN_WHITE_PIXEL(mi);
+		sp->color = MI_WHITE_PIXEL(mi);
 
 #if (ADAPT_ERASE == 1)
 	/* We initialize the adaptation code for screen erasing */
@@ -665,7 +677,7 @@ draw_bouboule(ModeInfo * mi)
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi))
 		XSetForeground(display, gc, MI_NONE_COLOR(mi));
 	else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 #if (ADAPT_ERASE == 1)
 	if (sp->hasbeenchecked == 0) {
