@@ -2,7 +2,7 @@
 /* bounce --- bouncing footballs */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bounce.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)bounce.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -82,7 +82,7 @@ static const char sccsid[] = "@(#)bounce.c	4.04 97/07/28 xlockmore";
 #define HACK_INIT init_bounce
 #define HACK_DRAW draw_bounce
 #define bounce_opts xlockmore_opts
-#define DEFAULTS "*delay: 10000 \n" \
+#define DEFAULTS "*delay: 5000 \n" \
  "*count: -10 \n" \
  "*size: 0 \n" \
  "*ncolors: 200 \n"
@@ -95,6 +95,15 @@ static const char sccsid[] = "@(#)bounce.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt bounce_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   bounce_description =
+{"bounce", "init_bounce", "draw_bounce", "release_bounce",
+ "refresh_bounce", "init_bounce", NULL, &bounce_opts,
+ 5000, -10, 1, 0, 64, 1.0, "",
+ "Shows bouncing footballs", 0, NULL};
+
+#endif
+
 #include "bitmaps/bounce-0.xbm"
 #include "bitmaps/bounce-1.xbm"
 #include "bitmaps/bounce-2.xbm"
@@ -213,7 +222,7 @@ drawball(ModeInfo * mi, ballstruct * ball)
 
 	if (bp->pixelmode) {
 		if (ball->xlast != -1) {
-			XSetForeground(display, MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+			XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
 			XFillRectangle(display, window, MI_GC(mi),
 				   ball->xlast, ball->ylast, bp->xs, bp->ys);
 		}
@@ -224,7 +233,7 @@ drawball(ModeInfo * mi, ballstruct * ball)
 		bouncestruct *bp = &bounces[MI_SCREEN(mi)];
 
 		if (ball->xlast != -1) {
-			XSetForeground(display, bp->stippledGC, MI_WIN_BLACK_PIXEL(mi));
+			XSetForeground(display, bp->stippledGC, MI_BLACK_PIXEL(mi));
 			XSetStipple(display, bp->stippledGC, bp->pixmaps[ORIENTS]);
 			XSetFillStyle(display, bp->stippledGC, FillStippled);
 			XSetTSOrigin(display, bp->stippledGC, ball->xlast, ball->ylast);
@@ -403,8 +412,8 @@ init_bounce(ModeInfo * mi)
 	bp = &bounces[MI_SCREEN(mi)];
 
 	if (!bp->stippledGC) {
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((bp->stippledGC = XCreateGC(display, window,
 				 GCForeground | GCBackground, &gcv)) == None)
 			return;
@@ -471,7 +480,7 @@ init_bounce(ModeInfo * mi)
 				bp->balls[i].color =
 					MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 			else
-				bp->balls[i].color = MI_WIN_WHITE_PIXEL(mi);
+				bp->balls[i].color = MI_WHITE_PIXEL(mi);
 			bp->balls[i].xlast = -1;
 			bp->balls[i].ylast = 0;
 			bp->balls[i].orientlast = 0;
@@ -485,7 +494,7 @@ init_bounce(ModeInfo * mi)
 		} else
 			tryagain++;
 	}
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -556,7 +565,7 @@ bounce_windows(ModeInfo * mi, bouncestruct * bs)
 	}
 	bs->nwindow = nchildren;
 	if (bs->windows != NULL) {
-		free(bs->windows);
+		(void) free((void *) bs->windows);
 	}
 	bs->windows = (ballwindow *) malloc(bs->nwindow * sizeof (ballwindow));
 	for (n = 0, i = 0; i < bs->nwindow; i++) {
