@@ -2,7 +2,7 @@
 /* bug --- Michael Palmiter's simulated evolution */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bug.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)bug.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -53,13 +53,23 @@ static const char sccsid[] = "@(#)bug.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt bug_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   bug_description =
+{"bug", "init_bug", "draw_bug", "release_bug",
+ "refresh_bug", "init_bug", NULL, &bug_opts,
+ 75000, 10, 32767, -4, 64, 1.0, "",
+ "Shows Palmiter's bug evolution and garden of Eden", 0, NULL};
+
+#endif
+
+extern int  neighbors;
+
 #define BUGBITS(n,w,h)\
   bp->pixmaps[bp->init_bits++]=\
   XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
 
 #define BACTERIA 0
 #define BUG 1
-#define ORIENTS 6
 
 #define MAXENERGY 1500		/* Max energy storage */
 #define INITENERGY 400		/* Initial energy */
@@ -72,42 +82,25 @@ ModeSpecOpt bug_opts =
 #define REDRAWSTEP 2000		/* How many bacteria to draw per cycle */
 #define MINGRIDSIZE 24
 #define MINSIZE 1
-#define PATTERNSIZE 8
-#define COLORS 11
-
-static unsigned char patterns[COLORS - 1][PATTERNSIZE] =
-{
-	{0x11, 0x22, 0x11, 0x22, 0x11, 0x22, 0x11, 0x22},	/* grey+white | stripe */
-	{0x00, 0x66, 0x66, 0x00, 0x00, 0x66, 0x66, 0x00},	/* spots */
-	{0x89, 0x44, 0x22, 0x11, 0x88, 0x44, 0x22, 0x11},	/* lt. / stripe */
-	{0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66},	/* | bars */
-	{0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa},	/* 50% grey */
-	{0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00},	/* - bars */
-	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x76},	/* dark \ stripe */
-	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
-	{0xaa, 0xff, 0xff, 0x55, 0xaa, 0xff, 0xff, 0x55},	/* black+grey - stripe */
-	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}	/* black */
-};
+#define ANGLES 360
+#define NEIGHBORKINDS 6
+#ifdef NEIGHBORS
+#define MAXNEIGHBORS 12
+#else
+#define MAXNEIGHBORS 6
+#endif
 
 /* Relative bug moves */
 /* Forward, Right, Hard Right, Reverse, Hard Left, Left */
 
-/* Compass bug moves */
-#define NE 0
-#define E 1
-#define SE 2
-#define SW 3
-#define W 4
-#define NW 5
-
 /* Bug data */
 typedef struct {
 	char        direction;
 	int         age, energy;
 	unsigned int color;
 	int         col, row;
-	int         gene[ORIENTS];
-	double      gene_prob[ORIENTS];
+	int         gene[MAXNEIGHBORS];
+	double      gene_prob[MAXNEIGHBORS];
 } bugstruct;
 
 /* Doublely linked list */
@@ -117,13 +110,11 @@ typedef struct _BugList {
 } BugList;
 
 typedef struct {
-	int         init_bits;
-	int         initialized;
+	int         neighbors;
 	int         n;		/* Number of bugs */
 	int         eden;	/* Does the garden exist? */
 	int         generation;
 	int         nhcols, nhrows;
-	int         nccols, ncrows;
 	int         width, height;
 	int         edenwidth, edenheight;
 	int         edenstartx, edenstarty;
@@ -132,13 +123,212 @@ typedef struct {
 	BugList    *currbug, *babybug, *lastbug, *firstbug, *lasttemp, *firsttemp;
 	BugList   **arr;	/* 0=empty or pts to a bug */
 	char       *bacteria;	/* 0=empty or food */
+	int         init_bits;
 	GC          stippledGC;
-	Pixmap      pixmaps[COLORS - 1];
+	Pixmap      pixmaps[NUMSTIPPLES - 1];
+	union {
+		XPoint      hexagon[6];
+		XPoint      triangle[2][3];
+	} shape;
 } bugfarmstruct;
 
 static double genexp[MAXGENE - MINGENE + 1];
+
+#ifdef NEIGHBORS
+static char plots[NEIGHBORKINDS] =
+{3, 4, 6, 8, 9, 12};		/* Neighborhoods */
+
+#endif
+
 static bugfarmstruct *bugfarms = NULL;
 
+#ifdef NEIGHBORS
+static void
+position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
+{
+	int         col = *pcol, row = *prow;
+
+	/* NO WRAPING */
+
+	if (bp->neighbors == 6) {
+		switch (dir) {
+			case 0:
+				col = col + 1;
+				break;
+			case 60:
+				if (!(row & 1))
+					col = col + 1;
+				row = row - 1;
+				break;
+			case 120:
+				if (row & 1)
+					col = col - 1;
+				row = row - 1;
+				break;
+			case 180:
+				col = col - 1;
+				break;
+			case 240:
+				if (row & 1)
+					col = col - 1;
+				row = row + 1;
+				break;
+			case 300:
+				if (!(row & 1))
+					col = col + 1;
+				row = row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (bp->neighbors == 4 || bp->neighbors == 8) {
+		switch (dir) {
+			case 0:
+				col = col + 1;
+				break;
+			case 45:
+				col = col + 1;
+				row = row - 1;
+				break;
+			case 90:
+				row = row - 1;
+				break;
+			case 135:
+				col = col - 1;
+				row = row - 1;
+				break;
+			case 180:
+				col = col - 1;
+				break;
+			case 225:
+				col = col - 1;
+				row = row + 1;
+				break;
+			case 270:
+				row = row + 1;
+				break;
+			case 315:
+				col = col + 1;
+				row = row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else {		/* TRI */
+		if ((col + row) % 2) {	/* right */
+			switch (dir) {
+				case 0:
+					col = col - 1;
+					break;
+				case 30:
+				case 40:
+					col = col - 1;
+					row = row - 1;
+					break;
+				case 60:
+					col = col - 1;
+					row = row - 2;
+					break;
+				case 80:
+				case 90:
+					row = row - 2;
+					break;
+				case 120:
+					row = row - 1;
+					break;
+				case 150:
+				case 160:
+					col = col + 1;
+					row = row - 1;
+					break;
+				case 180:
+					col = col + 1;
+					break;
+				case 200:
+				case 210:
+					col = col + 1;
+					row = row + 1;
+					break;
+				case 240:
+					row = row + 1;
+					break;
+				case 270:
+				case 280:
+					row = row + 2;
+					break;
+				case 300:
+					col = col - 1;
+					row = row + 2;
+					break;
+				case 320:
+				case 330:
+					col = col - 1;
+					row = row + 1;
+					break;
+				default:
+					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			}
+		} else {	/* left */
+			switch (dir) {
+				case 0:
+					col = col + 1;
+					break;
+				case 30:
+				case 40:
+					col = col + 1;
+					row = row + 1;
+					break;
+				case 60:
+					col = col + 1;
+					row = row + 2;
+					break;
+				case 80:
+				case 90:
+					row = row + 2;
+					break;
+				case 120:
+					row = row + 1;
+					break;
+				case 150:
+				case 160:
+					col = col - 1;
+					row = row + 1;
+					break;
+				case 180:
+					col = col - 1;
+					break;
+				case 200:
+				case 210:
+					col = col - 1;
+					row = row - 1;
+					break;
+				case 240:
+					row = row - 1;
+					break;
+				case 270:
+				case 280:
+					row = row - 2;
+					break;
+				case 300:
+					col = col + 1;
+					row = row - 2;
+					break;
+				case 320:
+				case 330:
+					col = col + 1;
+					row = row - 1;
+					break;
+				default:
+					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			}
+		}
+	}
+	*pcol = col;
+	*prow = row;
+}
+
+#endif
+
 #if 0
 static void
 cart2hex(cx, cy, hx, hy)
@@ -162,27 +352,27 @@ static char
 hexcmove(bugfarmstruct * bp, int hx, int hy, int dir, int *nhx, int *nhy)
 {
 	switch (dir) {
-		case NE:
+		case 0:
 			*nhy = hy - 1;
 			*nhx = hx + (int) !(hy & 1);
 			return (*nhy >= 0 && *nhx < bp->nhcols - !(*nhy & 1));
-		case E:
+		case 60:
 			*nhy = hy;
 			*nhx = hx + 1;
 			return (*nhx < bp->nhcols - !(*nhy & 1));
-		case SE:
+		case 120:
 			*nhy = hy + 1;
 			*nhx = hx + (int) !(hy & 1);
 			return (*nhy < bp->nhrows && *nhx < bp->nhcols - !(*nhy & 1));
-		case SW:
+		case 180:
 			*nhy = hy + 1;
 			*nhx = hx - (int) (hy & 1);
 			return (*nhy < bp->nhrows && *nhx >= 0);
-		case W:
+		case 240:
 			*nhy = hy;
 			*nhx = hx - 1;
 			return (*nhx >= 0);
-		case NW:
+		case 300:
 			*nhy = hy - 1;
 			*nhx = hx - (int) (hy & 1);
 			return (*nhy >= 0 && *nhx >= 0);
@@ -192,7 +382,7 @@ hexcmove(bugfarmstruct * bp, int hx, int hy, int dir, int *nhx, int *nhy)
 }
 
 static int
-neighbor(bugfarmstruct * bp, int hcol, int hrow)
+has_neighbor(bugfarmstruct * bp, int hcol, int hrow)
 {
 	int         colrow = hcol + hrow * bp->nhcols;
 
@@ -230,9 +420,9 @@ drawabacterium(ModeInfo * mi, int hcol, int hrow, unsigned char color)
 	int         ccol, crow;
 
 	if (color)
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
 	else
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 	hex2cart(hcol, hrow, &ccol, &crow);
 	XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 		       bp->xb + bp->xs * ccol, bp->yb + bp->ys * crow,
@@ -253,8 +443,8 @@ drawabug(ModeInfo * mi, int hcol, int hrow, unsigned int color)
 		XGCValues   gcv;
 
 		gcv.stipple = bp->pixmaps[color];
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		XChangeGC(MI_DISPLAY(mi), bp->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		gc = bp->stippledGC;
@@ -273,7 +463,7 @@ eraseabug(ModeInfo * mi, int hcol, int hrow)
 	bugfarmstruct *bp = &bugfarms[MI_SCREEN(mi)];
 	int         ccol, crow;
 
-	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 	hex2cart(hcol, hrow, &ccol, &crow);
 	XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 		  bp->xb + bp->xs * (ccol - 1), bp->yb + bp->ys * (crow - 1),
@@ -384,27 +574,27 @@ flush_buglist(bugfarmstruct * bp)
 }
 
 static int
-dirbug(bugstruct * info)
+dirbug(bugstruct * info, int neighbors)
 {
 	double      sum = 0.0, prob;
 	int         i;
 
 	prob = (double) LRAND() / MAXRAND;
-	for (i = 0; i < ORIENTS; i++) {
+	for (i = 0; i < neighbors; i++) {
 		sum += info->gene_prob[i];
 		if (prob < sum)
 			return i;
 	}
-	return ORIENTS - 1;	/* Could miss due to rounding */
+	return neighbors - 1;	/* Could miss due to rounding */
 }
 
 static void
-mutatebug(bugstruct * info)
+mutatebug(bugstruct * info, int neighbors)
 {
 	double      sum = 0.0;
 	int         gene;
 
-	gene = NRAND(ORIENTS);
+	gene = NRAND(neighbors);
 	if (LRAND() & 1) {
 		if (info->gene[gene] == MAXGENE)
 			return;
@@ -414,9 +604,9 @@ mutatebug(bugstruct * info)
 			return;
 		info->gene[gene]--;
 	}
-	for (gene = 0; gene < ORIENTS; gene++)
+	for (gene = 0; gene < neighbors; gene++)
 		sum += genexp[info->gene[gene] + MAXGENE];
-	for (gene = 0; gene < ORIENTS; gene++)
+	for (gene = 0; gene < neighbors; gene++)
 		info->gene_prob[gene] = genexp[info->gene[gene] + MAXGENE] / sum;
 }
 
@@ -467,6 +657,7 @@ init_bug(ModeInfo * mi)
 	XGCValues   gcv;
 	bugfarmstruct *bp;
 	int         i = 0, j = 0, hcol, hrow, gene, colrow;
+	int         nccols, ncrows;
 	double      sum;
 	bugstruct   info;
 
@@ -483,13 +674,12 @@ init_bug(ModeInfo * mi)
 			bp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
 		}
 		if (bp->init_bits == 0) {
-			for (i = 0; i < COLORS - 1; i++)
-				BUGBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+			for (i = 1; i < NUMSTIPPLES; i++)
+				BUGBITS(stipples[i], STIPPLESIZE, STIPPLESIZE);
 		}
 	}
 	bp->generation = 0;
-	if (!bp->initialized) {	/* Genesis */
-		bp->initialized = 1;
+	if (!bp->firstbug) {	/* Genesis */
 		/* Set up what will be a 'triply' linked list.
 		   doubly linked list, doubly linked to an array */
 		init_buglist(bp);
@@ -501,6 +691,21 @@ init_bug(ModeInfo * mi)
 	} else			/* Apocolypse now */
 		flush_buglist(bp);
 
+#ifndef NEIGHBORS
+	bp->neighbors = 6;
+#else
+	for (i = 0; i < NEIGHBORKINDS; i++) {
+		if (neighbors == plots[i]) {
+			dp->neighbors = plots[i];
+			break;
+		}
+		if (i == NEIGHBORKINDS - 1) {
+			dp->neighbors = plots[NRAND(NEIGHBORKINDS)];
+			break;
+		}
+	}
+#endif
+
 	bp->width = MI_WIN_WIDTH(mi);
 	bp->height = MI_WIN_HEIGHT(mi);
 	if (bp->width < 4)
@@ -520,13 +725,13 @@ init_bug(ModeInfo * mi)
 		bp->ys = MIN(size, MAX(MINSIZE, MIN(bp->width, bp->height) /
 				       MINGRIDSIZE));
 	bp->xs = bp->ys;
-	bp->nccols = MAX(bp->width / bp->xs - 1, 2);
-	bp->ncrows = MAX(bp->height / bp->ys, 2);
-	bp->nhcols = bp->nccols / 2;
-	bp->nhrows = bp->ncrows / 2;
+	nccols = MAX(bp->width / bp->xs - 2, 2);
+	ncrows = MAX(bp->height / bp->ys - 1, 2);
+	bp->nhcols = nccols / 2;
+	bp->nhrows = ncrows / 2;
 	bp->nhrows -= !(bp->nhrows & 1);	/* Must be odd */
-	bp->xb = (bp->width - bp->xs * bp->nccols) / 2;
-	bp->yb = (bp->height - bp->ys * bp->ncrows) / 2;
+	bp->xb = (bp->width - bp->xs * nccols) / 2;
+	bp->yb = (bp->height - bp->ys * ncrows) / 2;
 	if (bp->arr != NULL)
 		(void) free((void *) bp->arr);
 	bp->arr = (BugList **) calloc(bp->nhcols * bp->nhrows, sizeof (BugList *));
@@ -550,7 +755,7 @@ init_bug(ModeInfo * mi)
 	if (bp->n < 0)
 		bp->n = NRAND(-bp->n) + 1;
 
-	XClearWindow(display, window);
+	MI_CLEARWINDOW(mi);
 
 	/* Make bugs but if can't, don't loop forever */
 	i = 0, j = 0;
@@ -562,12 +767,12 @@ init_bug(ModeInfo * mi)
 			hcol = 0;
 		colrow = hcol + hrow * bp->nhcols;
 		j++;
-		if (!bp->arr[colrow] && !neighbor(bp, hcol, hrow)) {
+		if (!bp->arr[colrow] && !has_neighbor(bp, hcol, hrow)) {
 			i++;
 			info.age = 0;
 			info.energy = INITENERGY;
-			info.direction = NRAND(ORIENTS);
-			for (gene = 0; gene < ORIENTS; gene++)
+			info.direction = NRAND(bp->neighbors);
+			for (gene = 0; gene < bp->neighbors; gene++)
 #if 0				/* Some creationalism, may create gliders or twirlers */
 				do {
 					double      temp = (double) LRAND() / MAXRAND;
@@ -580,14 +785,14 @@ init_bug(ModeInfo * mi)
 				info.gene[gene] = 0;
 #endif
 			sum = 0;
-			for (gene = 0; gene < ORIENTS; gene++)
+			for (gene = 0; gene < bp->neighbors; gene++)
 				sum += genexp[info.gene[gene] + MAXGENE];
-			for (gene = 0; gene < ORIENTS; gene++)
+			for (gene = 0; gene < bp->neighbors; gene++)
 				info.gene_prob[gene] = genexp[info.gene[gene] + MAXGENE] / sum;
 			if (MI_NPIXELS(mi) > 3)
 				info.color = NRAND(MI_NPIXELS(mi));
 			else
-				info.color = NRAND(COLORS - 1);
+				info.color = NRAND(NUMSTIPPLES - 1);
 			info.col = hcol;
 			info.row = hrow;
 			addto_buglist(bp, info);
@@ -628,9 +833,9 @@ draw_bug(ModeInfo * mi)
 					break;
 				}
 				absdir = (bp->currbug->info.direction +
-				       dirbug(&bp->currbug->info)) % ORIENTS;
-			} while (!hexcmove(bp, hcol, hrow, absdir, &nhcol, &nhrow) ||
-				 neighbor(bp, nhcol, nhrow));
+					  dirbug(&bp->currbug->info, bp->neighbors)) % bp->neighbors;
+			} while (!hexcmove(bp, hcol, hrow, absdir * ANGLES / bp->neighbors,
+			  &nhcol, &nhrow) || has_neighbor(bp, nhcol, nhrow));
 			bp->currbug->info.age++;
 			bp->currbug->info.energy--;
 			if (tryit <= ENOUGH) {
@@ -652,8 +857,8 @@ draw_bug(ModeInfo * mi)
 					bp->babybug->info.age = 0;
 					bp->babybug->info.energy = INITENERGY;
 					dupin_buglist(bp);
-					mutatebug(&bp->babybug->previous->info);
-					mutatebug(&bp->babybug->info);
+					mutatebug(&bp->babybug->previous->info, bp->neighbors);
+					mutatebug(&bp->babybug->info, bp->neighbors);
 					bp->arr[colrow] = bp->babybug;
 					bp->babybug->info.col = hcol;
 					bp->babybug->info.row = hrow;
