@@ -2,7 +2,7 @@
 /* clock --- displays an analog clock */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)clock.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)clock.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -51,6 +51,15 @@ static const char sccsid[] = "@(#)clock.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt clock_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   clock_description =
+{"clock", "init_clock", "draw_clock", "release_clock",
+ "refresh_clock", "init_clock", NULL, &clock_opts,
+ 100000, -16, 200, -200, 64, 1.0, "",
+ "Shows Packard's clock", 0, NULL};
+
+#endif
+
 #ifdef TM_IN_SYS_TIME
 #include <sys/time.h>
 #else
@@ -156,7 +165,7 @@ DrawBorder(ModeInfo * mi, int clck, short int mode)
 
 
 	if (mode == 0)
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	else
 		XSetForeground(display, gc, cp->oclocks[clck].border_color);
 
@@ -178,7 +187,7 @@ DrawJewel(ModeInfo * mi, int clck, short int mode)
 
 	XSetLineAttributes(display, gc, 1, LineSolid, CapNotLast, JoinMiter);
 	if (mode == 0)
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	else
 		XSetForeground(display, gc, cp->oclocks[clck].jewel_color);
 	DrawDisk(display, MI_WINDOW(mi), gc,
@@ -198,7 +207,7 @@ DrawHand(ModeInfo * mi, int clck, hand * h, double angle, short int mode)
 
 	/* mode: 0 for erasing, anything else for drawing */
 	if (mode == 0)
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	else
 		XSetForeground(display, gc, h->color);
 	XSetLineAttributes(display, gc, h->width, LineSolid, CapNotLast, JoinMiter);
@@ -350,7 +359,7 @@ init_clock(ModeInfo * mi)
 	cp->width = MI_WIN_WIDTH(mi);
 	cp->height = MI_WIN_HEIGHT(mi);
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	cp->nclocks = MI_BATCHCOUNT(mi);
 	if (cp->nclocks < -MINCLOCKS) {
@@ -373,11 +382,11 @@ init_clock(ModeInfo * mi)
 
 	for (clck = 0; clck < cp->nclocks; clck++) {
 		/* By default, we set the clock's colors to white */
-		cp->oclocks[clck].border_color = MI_WIN_WHITE_PIXEL(mi);
-		cp->oclocks[clck].jewel_color = MI_WIN_WHITE_PIXEL(mi);
-		cp->oclocks[clck].hours.color = MI_WIN_WHITE_PIXEL(mi);
-		cp->oclocks[clck].minutes.color = MI_WIN_WHITE_PIXEL(mi);
-		cp->oclocks[clck].seconds.color = MI_WIN_WHITE_PIXEL(mi);
+		cp->oclocks[clck].border_color = MI_WHITE_PIXEL(mi);
+		cp->oclocks[clck].jewel_color = MI_WHITE_PIXEL(mi);
+		cp->oclocks[clck].hours.color = MI_WHITE_PIXEL(mi);
+		cp->oclocks[clck].minutes.color = MI_WHITE_PIXEL(mi);
+		cp->oclocks[clck].seconds.color = MI_WHITE_PIXEL(mi);
 		cp->d_time = time((time_t *) 0);
 	}
 	for (clck = 0; clck < cp->nclocks; clck++) {
