@@ -2,7 +2,7 @@
 /* coral --- a coral reef */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)coral.c 4.06 97/10/29 xlockmore";
+static const char sccsid[] = "@(#)coral.c 4.07 97/11/24 xlockmore";
 
 #endif
 /* 
@@ -55,6 +55,15 @@ static const char sccsid[] = "@(#)coral.c 4.06 97/10/29 xlockmore";
 ModeSpecOpt coral_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   coral_description =
+{"coral", "init_coral", "draw_coral", "release_coral",
+ "init_coral", "init_coral", NULL, &coral_opts,
+ 60000, -3, 1, 35, 64, 0.8, "",
+ "Shows a coral reef", 0, NULL};
+
+#endif
+
 #define MINSIZE 1
 #define MINSEEDS 1
 #define MAXSIZE 99
@@ -78,7 +87,8 @@ typedef struct {
 static coralstruct *reefs = NULL;
 
 #define getdot(x,y) (cp->reef[(y*cp->widthb)+(x>>5)] &  (1<<(x & 31)))
-#define setdot(x,y) (cp->reef[(y*cp->widthb)+(x>>5)] |= (1<<(x & 31)))
+/* Avoid array bounds reads and writes */
+#define setdot(x,y) (cp->reef[((y*cp->widthb)+(x>>5) >= 0 ? (y*cp->widthb)+(x>>5) : 0)] |= (1<<(x & 31)))
 
 /*-
  * returns 2 bits of randomness (conserving calls to LRAND()).
@@ -132,10 +142,11 @@ init_coral(ModeInfo * mi)
 
 	if (cp->reef) {
 		(void) free((void *) cp->reef);
-    cp->reef = NULL;
-  }
-	cp->reef = (unsigned int *) calloc(cp->widthb * cp->height,
+		cp->reef = NULL;
+	}
+	cp->reef = (unsigned int *) calloc((cp->widthb + 1) * cp->height,
 					   sizeof (unsigned int));
+
 	if (!cp->reef)
 		return;
 
@@ -147,10 +158,10 @@ init_coral(ModeInfo * mi)
 		cp->density = MIN(MAXSIZE, size);
 
 	cp->nwalkers = (cp->width * cp->height * cp->density) / 100;
-  if (cp->walkers) {
+	if (cp->walkers) {
 		(void) free((void *) cp->walkers);
-    cp->walkers = NULL;
-  }
+		cp->walkers = NULL;
+	}
 	cp->walkers = (XPoint *) calloc(cp->nwalkers, sizeof (XPoint));
 	if (!cp->walkers)
 		return;
@@ -161,12 +172,13 @@ init_coral(ModeInfo * mi)
 	else if (cp->seeds < MINSEEDS)
 		cp->seeds = MINSEEDS;
 
-	XClearWindow(display, window);
+	MI_CLEARWINDOW(mi);
+
 	if (MI_NPIXELS(mi) > 2) {
 		cp->colorindex = NRAND(MI_NPIXELS(mi) * COLORTHRESH);
 		XSetForeground(display, gc, MI_PIXEL(mi, cp->colorindex / COLORTHRESH));
 	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
 	for (i = 0; i < cp->seeds; i++) {
 		int         x, y;
@@ -192,10 +204,10 @@ init_coral(ModeInfo * mi)
 		cp->walkers[i].x = NRAND(cp->width - 2) + 1;
 		cp->walkers[i].y = NRAND(cp->height - 2) + 1;
 	}
-  if (cp->pointbuf) {
+	if (cp->pointbuf) {
 		(void) free((void *) cp->pointbuf);
-    cp->pointbuf = NULL;
-  }
+		cp->pointbuf = NULL;
+	}
 	cp->pointbuf = (XPoint *) calloc((MAXPOINTS + 2), sizeof (XPoint));
 	if (!cp->pointbuf)
 		return;
@@ -249,7 +261,7 @@ draw_coral(ModeInfo * mi)
 				if (++cp->colorindex >= MI_NPIXELS(mi) * COLORTHRESH)
 					cp->colorindex = 0;
 			} else
-				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+				XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
 			if (0 == cp->nwalkers) {
 				if (cp->pointbuf)
