@@ -2,7 +2,7 @@
 /* daisy --- flowers in a field */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)daisy.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)daisy.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -45,10 +45,38 @@ static const char sccsid[] = "@(#)daisy.c	4.04 97/07/28 xlockmore";
 
 #endif /* STANDALONE */
 
-ModeSpecOpt daisy_opts =
-{0, NULL, 0, NULL, NULL};
+#define DEF_GARDEN  "False"
 
-#define GREEN	21
+static Bool garden;
+
+static XrmOptionDescRec opts[] =
+{
+	{"-garden", ".daisy.garden", XrmoptionNoArg, (caddr_t) "on"},
+	{"+garden", ".daisy.garden", XrmoptionNoArg, (caddr_t) "off"}
+};
+static argtype vars[] =
+{
+	{(caddr_t *) & garden, "garden", "Garden", DEF_GARDEN, t_Bool}
+};
+static OptionStruct desc[] =
+{
+	{"-/+garden", "turn on/off garden"}
+};
+
+ModeSpecOpt daisy_opts =
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   daisy_description =
+{"daisy", "init_daisy", "draw_daisy", "release_daisy",
+ "refresh_daisy", "init_daisy", NULL, &daisy_opts,
+ 100000, 300, 350, 1, 64, 1.0, "",
+ "Shows a meadow of daisies", 0, NULL};
+
+#endif
+
+#define GREEN	(23 * MI_NPIXELS(mi) / 64)
+#define NOTGREEN	(7 * MI_NPIXELS(mi) / 64)
 #define MINDAISIES   1
 
 #define DROOP 20		/* Percentage x with relation to y */
@@ -61,6 +89,7 @@ typedef struct {
 	int         time;	/* up time */
 	int         ndaisies;
 	int         meadow_y;
+	Bool        garden;
 	float       step;
 } daisystruct;
 
@@ -75,9 +104,9 @@ drawstem(ModeInfo * mi, XPoint start, XPoint stop)
 	GC          gc = MI_GC(mi);
 
 	if (MI_NPIXELS(mi) > 2)
-		XSetForeground(display, gc, MI_PIXEL(mi, GREEN + 3));
+		XSetForeground(display, gc, MI_PIXEL(mi, GREEN));
 	else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 #if 1
 	XDrawLine(display, window, gc, start.x, start.y, stop.x, stop.y);
 #else
@@ -100,14 +129,14 @@ drawpetals(ModeInfo * mi, XPoint center,
 	int         petal, inc;
 	float       sine, cosine;
 
-	if (MI_NPIXELS(mi) > GREEN + 7) {
+	if (MI_NPIXELS(mi) > GREEN + NOTGREEN) {
 		do {
 			colour = NRAND(MI_NPIXELS(mi));
-		} while (colour >= GREEN - 7 && colour <= GREEN + 7);
+		} while (colour >= GREEN - NOTGREEN && colour <= GREEN + NOTGREEN);
 	}
 	start_angle = NRAND(360) * M_PI / 180;
 	inc_angle = 2 * M_PI / petals;
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	if (size > 2)
 		XFillArc(display, window, gc,
 			 center.x - size / 2, center.y - size / 2, size, size,
@@ -117,7 +146,7 @@ drawpetals(ModeInfo * mi, XPoint center,
 		cosine = COSF(start_angle + petal * inc_angle);
 		if (size > 2)
 			if (MI_NPIXELS(mi) <= 2) {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				for (inc = 0; inc < circles; inc++) {
 					newcenter.x = center.x + (int) (sine * (offset + delta * inc));
 					newcenter.y = center.y + (int) (cosine * (offset + delta * inc));
@@ -126,10 +155,10 @@ drawpetals(ModeInfo * mi, XPoint center,
 						 0, 23040);
 				}
 			}
-		if (MI_NPIXELS(mi) > GREEN + 7)
+		if (MI_NPIXELS(mi) > 2 && MI_NPIXELS(mi) > GREEN + NOTGREEN)
 			XSetForeground(display, gc, MI_PIXEL(mi, colour));
 		else
-			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 		for (inc = 0; inc < circles; inc++) {
 			newcenter.x = center.x + (int) (sine * (offset + delta * inc));
 			newcenter.y = center.y + (int) (cosine * (offset + delta * inc));
@@ -153,15 +182,15 @@ drawcenter(ModeInfo * mi, XPoint center, int size, unsigned long petalcolour)
 	GC          gc = MI_GC(mi);
 	unsigned long colour;
 
-	if (MI_NPIXELS(mi) > GREEN + 7) {
+	if (MI_NPIXELS(mi) > GREEN + NOTGREEN) {
 		do {
 			/* Insure good contrast */
 			colour = (NRAND(2 * MI_NPIXELS(mi) / 3) + petalcolour +
 				  MI_NPIXELS(mi) / 6) % MI_NPIXELS(mi);
-		} while (colour >= GREEN - 7 && colour <= GREEN + 7);
+		} while (colour >= GREEN - NOTGREEN && colour <= GREEN + NOTGREEN);
 		XSetForeground(display, gc, MI_PIXEL(mi, colour));
 	} else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	if (size < 2)
 		XDrawPoint(display, window, gc, center.x, center.y);
 	else
@@ -191,13 +220,13 @@ drawdaisy(ModeInfo * mi)
 		2 * (dp->time * dp->time + 1) / (dp->ndaisies * dp->ndaisies + 1);
 #endif
 	stem_start.y = stem_stop.y + height;
-#ifdef GARDEN
-	stem_stop.x = ((LRAND() & 1) ? 1 : -1) *
-		NRAND(dp->width / 4 * (dp->time + 1) / (dp->ndaisies + 20) +
-		      dp->width / 4) + dp->width / 2;
-#else /* MEADOW */
-	stem_stop.x = NRAND(dp->width);
-#endif
+	if (dp->garden) {
+		stem_stop.x = ((LRAND() & 1) ? 1 : -1) *
+			NRAND(dp->width / 4 * (dp->time + 1) / (dp->ndaisies + 20) +
+			      dp->width / 4) + dp->width / 2;
+	} else {
+		stem_stop.x = NRAND(dp->width);
+	}
 	/* Give about droop left or right with 25% randomness */
 	droop = ((LRAND() & 1) ? 1 : -1) * DROOP * (NRAND(50) + 75) / 100;
 	stem_start.x = stem_stop.x + droop * height / 100;
@@ -226,6 +255,10 @@ init_daisy(ModeInfo * mi)
 	dp->width = MI_WIN_WIDTH(mi);
 	dp->height = MI_WIN_HEIGHT(mi);
 	dp->time = 0;
+	if (MI_WIN_IS_FULLRANDOM(mi))
+		dp->garden = (Bool) (LRAND() & 1);
+	else
+		dp->garden = garden;
 
 	dp->ndaisies = MI_BATCHCOUNT(mi);
 	if (dp->ndaisies < -MINDAISIES)
@@ -234,7 +267,7 @@ init_daisy(ModeInfo * mi)
 		dp->ndaisies = MINDAISIES;
 	dp->meadow_y = dp->height / 5;
 	dp->step = (float) (dp->height - 2 * dp->meadow_y) / (dp->ndaisies + 1.0);
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -262,8 +295,9 @@ refresh_daisy(ModeInfo * mi)
 {
 	daisystruct *dp = &daisies[MI_SCREEN(mi)];
 
-	if (dp->time < dp->ndaisies)
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
-	else
+	if (dp->time < dp->ndaisies) {
+		MI_CLEARWINDOW(mi);
+	} else {
 		init_daisy(mi);
+	}
 }
