@@ -155,6 +155,14 @@ draw_deco(ModeInfo * mi)
 	if (dp->time == 0) {
 		/* This fills up holes */
 		MI_CLEARWINDOWCOLOR(mi, dp->bordercolor);
+#ifdef SOLARIS2
+		/*
+		   * if this is not done the first rectangle is sometimes messed up on
+		   * Solaris2 with 24 bit TrueColor (Ultra2)
+		 */
+		XDrawRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+			  0, 0, MI_WIN_WIDTH(mi) / 2 + 1, MI_WIN_HEIGHT(mi) / 2 + 1);
+#endif
 		deco(mi, 0, 0, MI_WIN_WIDTH(mi), MI_WIN_HEIGHT(mi), 0);
 	}
 	if (++dp->time > MI_CYCLES(mi))
