@@ -2,7 +2,7 @@
 /* demon --- David Griffeath's cellular automata */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)demon.c	4.06 97/10/28 xlockmore";
+static const char sccsid[] = "@(#)demon.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -61,10 +61,23 @@ static const char sccsid[] = "@(#)demon.c	4.06 97/10/28 xlockmore";
 
 #endif /* STANDALONE */
 
+extern int  neighbors;
+
 ModeSpecOpt demon_opts =
 {0, NULL, 0, NULL, NULL};
 
-extern int  neighbors;
+#ifdef USE_MODULES
+ModStruct   demon_description =
+{"demon", "init_demon", "draw_demon", "release_demon",
+ "refresh_demon", "init_demon", NULL, &demon_opts,
+ 50000, 0, 1000, -7, 64, 1.0, "",
+ "Shows Griffeath's cellular automata", 0, NULL};
+
+#endif
+
+#define DEMONBITS(n,w,h)\
+  dp->pixmaps[dp->init_bits++]=\
+  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
 
 #define REDRAWSTEP 2000		/* How many cells to draw per cycle */
 #define MINSTATES 2
@@ -88,14 +101,19 @@ typedef struct {
 	int         state;
 	int         redrawing, redrawpos;
 	int        *ncells;
-	CellList **cellList;
+	CellList  **cellList;
 	unsigned char *oldcell, *newcell;
 	int         neighbors;
-	XPoint      hexagonList[6];
-	XPoint      triangleList[2][3];
+	int         init_bits;
+	GC          stippledGC;
+	Pixmap      pixmaps[NUMSTIPPLES - 1];
+	union {
+		XPoint      hexagon[6];
+		XPoint      triangle[2][3];
+	} shape;
 } demonstruct;
 
-static int  initVal[2][NEIGHBORKINDS] =
+static char plots[2][NEIGHBORKINDS] =
 {
 	{3, 4, 6, 8, 9, 12},	/* Neighborhoods */
 	{12, 16, 18, 20, 22, 24}	/* Number of states */
@@ -107,46 +125,56 @@ static void
 drawcell(ModeInfo * mi, int col, int row, unsigned char state)
 {
 	demonstruct *dp = &demons[MI_SCREEN(mi)];
+	GC          gc;
 
-	if (!state)
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
-	else if (MI_NPIXELS(mi) > 2)
+	if (!state) {
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
+		gc = MI_GC(mi);
+	} else if (MI_NPIXELS(mi) >= NUMSTIPPLES) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
 			   MI_PIXEL(mi, (((int) state - 1) * MI_NPIXELS(mi) /
 					 (dp->states - 1)) % MI_NPIXELS(mi)));
-	else
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), (state % 2) ?
-			    MI_WIN_WHITE_PIXEL(mi) : MI_WIN_BLACK_PIXEL(mi));
+		gc = MI_GC(mi);
+	} else {
+		XGCValues   gcv;
+
+		gcv.stipple = dp->pixmaps[(state - 1) % (NUMSTIPPLES - 1)];
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
+		XChangeGC(MI_DISPLAY(mi), dp->stippledGC,
+			  GCStipple | GCForeground | GCBackground, &gcv);
+		gc = dp->stippledGC;
+	}
 	if (dp->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		dp->hexagonList[0].x = dp->xb + ccol * dp->xs;
-		dp->hexagonList[0].y = dp->yb + crow * dp->ys;
+		dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+		dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
 		if (dp->xs == 1 && dp->ys == 1)
 			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
-				       MI_GC(mi), dp->hexagonList[0].x, dp->hexagonList[0].y, 1, 1);
+				       gc, dp->shape.hexagon[0].x, dp->shape.hexagon[0].y, 1, 1);
 		else
-			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			      dp->hexagonList, 6, Convex, CoordModePrevious);
+			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+			    dp->shape.hexagon, 6, Convex, CoordModePrevious);
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
-		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 		dp->xb + dp->xs * col, dp->yb + dp->ys * row, dp->xs, dp->ys);
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
 
-		dp->triangleList[orient][0].x = dp->xb + col * dp->xs;
-		dp->triangleList[orient][0].y = dp->yb + row * dp->ys;
+		dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+		dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
 		if (dp->xs <= 3 || dp->ys <= 3)
-			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			 ((orient) ? -1 : 1) + dp->triangleList[orient][0].x,
-				       dp->triangleList[orient][0].y, 1, 1);
+			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+			((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
+				       dp->shape.triangle[orient][0].y, 1, 1);
 		else {
 			if (orient)
-				dp->triangleList[orient][0].x += (dp->xs / 2 - 1);
+				dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
 			else
-				dp->triangleList[orient][0].x -= (dp->xs / 2 - 1);
-			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				     dp->triangleList[orient], 3, Convex, CoordModePrevious);
+				dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
+			XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 
 		}
 	}
@@ -156,7 +184,7 @@ static void
 addtolist(ModeInfo * mi, int col, int row, unsigned char state)
 {
 	demonstruct *dp = &demons[MI_SCREEN(mi)];
-	CellList  *current;
+	CellList   *current;
 
 	current = dp->cellList[state];
 	dp->cellList[state] = (CellList *) malloc(sizeof (CellList));
@@ -171,7 +199,7 @@ static void
 print_state(ModeInfo * mi, int state)
 {
 	demonstruct *dp = &demons[MI_SCREEN(mi)];
-	CellList  *locallist;
+	CellList   *locallist;
 	int         i = 0;
 
 	locallist = dp->cellList[state];
@@ -189,7 +217,7 @@ print_state(ModeInfo * mi, int state)
 static void
 free_state(demonstruct * dp, int state)
 {
-	CellList  *current;
+	CellList   *current;
 
 	while (dp->cellList[state]) {
 		current = dp->cellList[state];
@@ -237,20 +265,29 @@ static void
 draw_state(ModeInfo * mi, int state)
 {
 	demonstruct *dp = &demons[MI_SCREEN(mi)];
+	GC          gc;
 	XRectangle *rects;
-	CellList  *current;
+	CellList   *current;
 
-	if (!state)
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
-	else if (MI_NPIXELS(mi) > 2)
+	if (!state) {
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
+		gc = MI_GC(mi);
+	} else if (MI_NPIXELS(mi) >= NUMSTIPPLES) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
 			   MI_PIXEL(mi, (((int) state - 1) * MI_NPIXELS(mi) /
 					 (dp->states - 1)) % MI_NPIXELS(mi)));
-	else
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), (state % 2) ?
-			    MI_WIN_WHITE_PIXEL(mi) : MI_WIN_BLACK_PIXEL(mi));
+		gc = MI_GC(mi);
+	} else {
+		XGCValues   gcv;
 
-	if (dp->neighbors == 6) {	/*Draw right away, slow */
+		gcv.stipple = dp->pixmaps[(state - 1) % (NUMSTIPPLES - 1)];
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
+		XChangeGC(MI_DISPLAY(mi), dp->stippledGC,
+			  GCStipple | GCForeground | GCBackground, &gcv);
+		gc = dp->stippledGC;
+	}
+	if (dp->neighbors == 6) {	/* Draw right away, slow */
 		current = dp->cellList[state];
 		while (current) {
 			int         col, row, ccol, crow;
@@ -258,23 +295,22 @@ draw_state(ModeInfo * mi, int state)
 			col = current->pt.x;
 			row = current->pt.y;
 			ccol = 2 * col + !(row & 1), crow = 2 * row;
-			dp->hexagonList[0].x = dp->xb + ccol * dp->xs;
-			dp->hexagonList[0].y = dp->yb + crow * dp->ys;
+			dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+			dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
 			if (dp->xs == 1 && dp->ys == 1)
 				XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),
-					       MI_GC(mi), dp->hexagonList[0].x, dp->hexagonList[0].y, 1, 1);
+					       gc, dp->shape.hexagon[0].x, dp->shape.hexagon[0].y, 1, 1);
 			else
-				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-					     dp->hexagonList, 6, Convex, CoordModePrevious);
+				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					     dp->shape.hexagon, 6, Convex, CoordModePrevious);
 			current = current->next;
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		/* Take advantage of XDrawRectangles */
-		int         ncells;
+		int         ncells = 0;
 
 		/* Create Rectangle list from part of the cellList */
 		rects = (XRectangle *) malloc(dp->ncells[state] * sizeof (XRectangle));
-		ncells = 0;
 		current = dp->cellList[state];
 		while (current) {
 			rects[ncells].x = dp->xb + current->pt.x * dp->xs;
@@ -285,7 +321,7 @@ draw_state(ModeInfo * mi, int state)
 			ncells++;
 		}
 		/* Finally get to draw */
-		XFillRectangles(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), rects, ncells);
+		XFillRectangles(MI_DISPLAY(mi), MI_WINDOW(mi), gc, rects, ncells);
 		/* Free up rects list and the appropriate part of the cellList */
 		(void) free((void *) rects);
 	} else {		/* TRI */
@@ -296,19 +332,19 @@ draw_state(ModeInfo * mi, int state)
 			col = current->pt.x;
 			row = current->pt.y;
 			orient = (col + row) % 2;	/* O left 1 right */
-			dp->triangleList[orient][0].x = dp->xb + col * dp->xs;
-			dp->triangleList[orient][0].y = dp->yb + row * dp->ys;
+			dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+			dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
 			if (dp->xs <= 3 || dp->ys <= 3)
-				XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-					       ((orient) ? -1 : 1) + dp->triangleList[orient][0].x,
-					dp->triangleList[orient][0].y, 1, 1);
+				XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					       ((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
+				      dp->shape.triangle[orient][0].y, 1, 1);
 			else {
 				if (orient)
-					dp->triangleList[orient][0].x += (dp->xs / 2 - 1);
+					dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
 				else
-					dp->triangleList[orient][0].x -= (dp->xs / 2 - 1);
-				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-					     dp->triangleList[orient], 3, Convex, CoordModePrevious);
+					dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
+				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+					     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 			}
 			current = current->next;
 		}
@@ -336,6 +372,8 @@ RandomSoup(ModeInfo * mi)
 void
 init_demon(ModeInfo * mi)
 {
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
 	int         size = MI_SIZE(mi), nk;
 	demonstruct *dp;
 
@@ -347,16 +385,30 @@ init_demon(ModeInfo * mi)
 	dp = &demons[MI_SCREEN(mi)];
 	dp->generation = 0;
 	dp->redrawing = 0;
+	if (MI_NPIXELS(mi) < NUMSTIPPLES) {
+		if (dp->stippledGC == None) {
+			XGCValues   gcv;
+
+			gcv.fill_style = FillOpaqueStippled;
+			dp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
+		}
+		if (dp->init_bits == 0) {
+			int         i;
+
+			for (i = 1; i < NUMSTIPPLES; i++)
+				DEMONBITS(stipples[i], STIPPLESIZE, STIPPLESIZE);
+		}
+	}
 	free_struct(dp);
 
 	for (nk = 0; nk < NEIGHBORKINDS; nk++) {
-		if (neighbors == initVal[0][nk]) {
-			dp->neighbors = initVal[0][nk];
+		if (neighbors == plots[0][nk]) {
+			dp->neighbors = plots[0][nk];
 			break;
 		}
 		if (nk == NEIGHBORKINDS - 1) {
 			nk = NRAND(NEIGHBORKINDS);
-			dp->neighbors = initVal[0][nk];
+			dp->neighbors = plots[0][nk];
 			break;
 		}
 	}
@@ -365,9 +417,7 @@ init_demon(ModeInfo * mi)
 	if (dp->states < -MINSTATES)
 		dp->states = NRAND(-dp->states - MINSTATES + 1) + MINSTATES;
 	else if (dp->states < MINSTATES)
-		dp->states = initVal[1][nk];
-	if (dp->states > NUMCOLORS)
-		dp->states = NUMCOLORS;
+		dp->states = plots[1][nk];
 	dp->cellList = (CellList **) calloc(dp->states, sizeof (CellList *));
 	dp->ncells = (int *) calloc(dp->states, sizeof (int));
 
@@ -402,8 +452,8 @@ init_demon(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs / 2;
 		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys;
 		for (i = 0; i < 6; i++) {
-			dp->hexagonList[i].x = (dp->xs - 1) * hexagonUnit[i].x;
-			dp->hexagonList[i].y = ((dp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			dp->shape.hexagon[i].x = (dp->xs - 1) * hexagonUnit[i].x;
+			dp->shape.hexagon[i].y = ((dp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		if (size < -MINSIZE)
@@ -447,15 +497,15 @@ init_demon(ModeInfo * mi)
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2 + dp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 3; i++) {
-				dp->triangleList[orient][i].x =
+				dp->shape.triangle[orient][i].x =
 					(dp->xs - 2) * triangleUnit[orient][i].x;
-				dp->triangleList[orient][i].y =
+				dp->shape.triangle[orient][i].y =
 					(dp->ys - 2) * triangleUnit[orient][i].y;
 			}
 		}
 	}
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	dp->oldcell = (unsigned char *)
 		malloc(dp->ncols * dp->nrows * sizeof (unsigned char));
@@ -814,8 +864,17 @@ release_demon(ModeInfo * mi)
 	if (demons != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
-			free_struct(&demons[screen]);
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			demonstruct *dp = &demons[screen];
+			int         shade;
+
+			if (dp->stippledGC != None) {
+				XFreeGC(MI_DISPLAY(mi), dp->stippledGC);
+			}
+			for (shade = 0; shade < dp->init_bits; shade++)
+				XFreePixmap(MI_DISPLAY(mi), dp->pixmaps[shade]);
+			free_struct(dp);
+		}
 		(void) free((void *) demons);
 		demons = NULL;
 	}
