@@ -106,7 +106,7 @@ ModeSpecOpt dilemma_opts =
 ModStruct   dilemma_description =
 {"dilemma", "init_dilemma", "draw_dilemma", "release_dilemma",
  "refresh_dilemma", "init_dilemma", NULL, &dilemma_opts,
- 200000, -2, 1000, 0, 6, 64, 1.0, "",
+ 200000, -2, 1000, 0, 64, 1.0, "",
  "Shows Lloyd's Prisoner's Dilemma simulation", 0, NULL};
 
 #endif
@@ -120,10 +120,10 @@ extern int  neighbors;
 #define DEFECTOR 0
 #define COOPERATOR 1
 #define COLORS 4
-#define BLUE (45 * MI_NPIXELS(mi) / 64)	/* COOPERATING, was cooperating */
-#define GREEN (23 * MI_NPIXELS(mi) / 64) /* COOPERATING, was defecting */
-#define YELLOW (MI_NPIXELS(mi) / 6) /* DEFECTING, was cooperating */
-#define RED	0	/* DEFECTING, was defecting */
+#define BLUE (45 * MI_NPIXELS(mi) / 64)		/* COOPERATING, was cooperating */
+#define GREEN (23 * MI_NPIXELS(mi) / 64)	/* COOPERATING, was defecting */
+#define YELLOW (MI_NPIXELS(mi) / 6)	/* DEFECTING, was cooperating */
+#define RED	0		/* DEFECTING, was defecting */
 #define MINDEFECT 1
 #define BITMAPS 2
 #define MINGRIDSIZE 16
@@ -240,9 +240,9 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 			}
 		} else
 			(void) XPutImage(display, window, gc,
-				  &logo[bitmap], 0, 0,
+					 &logo[bitmap], 0, 0,
 				dp->xb + dp->xs * col, dp->yb + dp->ys * row,
-				  icon_width, icon_height);
+					 icon_width, icon_height);
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
 
@@ -752,7 +752,7 @@ init_dilemma(ModeInfo * mi)
 	if (MI_NPIXELS(mi) >= COLORS) {
 
 		dp->colors[0][0] = MI_PIXEL(mi, BLUE);	/* COOPERATING, was cooperating */
-		dp->colors[0][1] = MI_PIXEL(mi, GREEN);	/* COOPERATING, was defecting */
+		dp->colors[0][1] = MI_PIXEL(mi, GREEN);		/* COOPERATING, was defecting */
 		dp->colors[1][0] = MI_PIXEL(mi, YELLOW);	/* DEFECTING, was cooperating */
 		dp->colors[1][1] = MI_PIXEL(mi, RED);	/* DEFECTING, was defecting */
 	} else {
