@@ -2,7 +2,7 @@
 /* drift --- drifting recursive fractal cosmic flames */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)drift.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)drift.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -74,7 +74,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt drift_opts =
-{4, opts, 2, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   drift_description =
+{"drift", "init_drift", "draw_drift", "release_drift",
+ "refresh_drift", "init_drift", NULL, &drift_opts,
+ 10000, 30, 1, 1, 64, 1.0, "",
+ "Shows cosmic drifting flame fractals", 0, NULL};
+
+#endif
 
 #define MAXBATCH1	200	/* mono */
 #define MAXBATCH2	20	/* color */
@@ -223,7 +232,7 @@ initmode(ModeInfo * mi, int mode)
 	}
 	dp->fractal_len = (dp->fractal_len * MI_BATCHCOUNT(mi)) / 20;
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
 
 static void
@@ -260,10 +269,10 @@ initfractal(ModeInfo * mi)
 	dp->total_points = 0;
 
 	if (!dp->ncpoints)
-		dp->ncpoints = (int *) malloc(NUMCOLORS * sizeof (int));
+		dp->ncpoints = (int *) malloc(MI_NCOLORS(mi) * sizeof (int));
 
 	if (!dp->cpts)
-		dp->cpts = (XPoint *) malloc(NUMCOLORS * MAXBATCH2 * sizeof (XPoint));
+		dp->cpts = (XPoint *) malloc(MI_NCOLORS(mi) * MAXBATCH2 * sizeof (XPoint));
 	if (dp->rainbow)
 		for (i = 0; i < MI_NPIXELS(mi); i++)
 			dp->ncpoints[i] = 0;
@@ -295,7 +304,7 @@ initfractal(ModeInfo * mi)
 	if (dp->color)
 		dp->pixcol = MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi)));
 	else
-		dp->pixcol = MI_WIN_WHITE_PIXEL(mi);
+		dp->pixcol = MI_WHITE_PIXEL(mi);
 
 }
 
@@ -626,5 +635,5 @@ release_drift(ModeInfo * mi)
 void
 refresh_drift(ModeInfo * mi)
 {
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
