@@ -159,8 +159,8 @@ typedef struct {		/* info about a pair of eyes */
 
 typedef struct {		/* per-screen info */
 	Pixmap      flypix;
-  int         flywidth, flyheight;
-  int         graphics_format;
+	int         flywidth, flyheight;
+	int         graphics_format;
 	GC          eyeGC;
 	GC          flyGC;
 	int         num_eyes;
@@ -557,7 +557,7 @@ move_fly(ModeInfo * mi, Fly * f)
 		unsigned int m;
 
 		(void) XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
-			      &r, &c, &rx, &ry, &cx, &cy, &m);
+				     &r, &c, &rx, &ry, &cx, &cy, &m);
 		if (cx <= 0 || cy <= 0 ||
 		    cx >= MI_WIN_WIDTH(mi) - f->width - 1 ||
 		    cy >= MI_WIN_HEIGHT(mi) - f->width - 1)
@@ -629,7 +629,6 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 	Window      window = MI_WINDOW(mi);
 	int         win_width = MI_WIN_WIDTH(mi);
 	int         win_height = MI_WIN_HEIGHT(mi);
-	int         win_depth = MI_DEPTH(mi);
 	unsigned long black_pixel = MI_BLACK_PIXEL(mi);
 	unsigned long white_pixel = MI_WHITE_PIXEL(mi);
 	Bool        iconic = MI_WIN_IS_ICONIC(mi);
@@ -707,7 +706,7 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 			XFreePixmap(display, e->pixmap);
 		}
 		e->pixmap = XCreatePixmap(display, window,
-					  e->width, e->height, win_depth);
+					  e->width, e->height, MI_DEPTH(mi));
 		if (e->pixmap == None) {
 			e->width = e->height = 0;
 			return;
@@ -884,9 +883,9 @@ init_eyes(ModeInfo * mi)
 #endif /* !STANDALONE */
 
 	if (ep->flypix == None) {
-    getPixmap(display, window, FLY_WIDTH, FLY_HEIGHT, FLY_BITS,
-    &(ep->flywidth), &(ep->flyheight), &(ep->flypix), 
-    &(ep->graphics_format));
+		getPixmap(display, window, FLY_WIDTH, FLY_HEIGHT, FLY_BITS,
+			  &(ep->flywidth), &(ep->flyheight), &(ep->flypix),
+			  &(ep->graphics_format));
 		if (ep->flypix == None) {
 			return;
 		}
