@@ -2,12 +2,13 @@
 /* flag --- a waving flag of your operating system */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)flag.c	4.06 97/10/28 xlockmore";
+static const char sccsid[] = "@(#)flag.c	4.07 97/11/24 xlockmore";
 
 #endif
 
 /*-
- * Copyright (c) 1996 by Charles Vidal <vidalc@univ-mlv.fr>
+ * Copyright (c) 1996 by Charles Vidal <vidalc@club-internet.fr>
+ *         http://www.chez.com/vidalc
  *
  * Thanks to Bas van Gaalen, Holland, PD, for his Pascal source
  *
@@ -70,7 +71,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt flag_opts =
-{2, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   flag_description =
+{"flag", "init_flag", "draw_flag", "release_flag",
+ "refresh_flag", "init_flag", NULL, &flag_opts,
+ 50000, 1, 1000, -7, 64, 1.0, "",
+ "Shows a waving flag of your operating system", 0, NULL};
+
+#endif
 
 /* aliases for vars defined in the bitmap file */
 #define FLAG_WIDTH   image_width
@@ -82,7 +92,7 @@ ModeSpecOpt flag_opts =
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 #define FLAG_NAME  image_name
 #include "flag.xpm"
-#define DEFAULT_XPM 1
+#define DEFAULT_XPM 0
 #endif
 
 #if !defined( VMS ) || ( __VMS_VER >= 70000000 )
@@ -109,6 +119,8 @@ ModeSpecOpt flag_opts =
 #define MINW(fp) (MINSCALE * (fp)->image->width + 2 * MINAMP + (fp)->pointsize)
 #define MINH(fp) (MINSCALE * (fp)->image->height+ 2 * MINAMP + (fp)->pointsize)
 #define ANGLES		360
+#define IMAGE_FLAG 0
+#define MESSAGE_FLAG 1
 
 typedef struct {
 	int         samp;
@@ -125,11 +137,11 @@ typedef struct {
 	int         startcolor;
 	int         choice;
 	XImage     *image;
-  XImage     *logo;
-  Colormap    cmap;
-  unsigned long black, white;
-  int         graphics_format;
-  GC          backGC;
+	XImage     *logo;
+	Colormap    cmap;
+	unsigned long black;
+	int         graphics_format;
+	GC          backGC;
 } flagstruct;
 
 static XFontStruct *mode_font = None;
@@ -150,8 +162,18 @@ initSintab(ModeInfo * mi)
 	flagstruct *fp = &flags[MI_SCREEN(mi)];
 	int         i;
 
+  /*-
+   * change the periodicity of the sin formula : the maximum of the
+   * periocity seem to be 16 ( 2^4 ), after the drawing isn't good looking
+   */
+	int         periodicity = random_num(4);
+	int         puissance = 1;
+
+	/* for (i=0;i<periodicity;i++) puissance*=2; */
+	puissance <<= periodicity;
 	for (i = 0; i < ANGLES; i++)
-		fp->stab[i] = (int) (SINF(i * 4 * M_PI / ANGLES) * fp->samp) + fp->sofs;
+		fp->stab[i] = (int) (SINF(i * puissance * M_PI / ANGLES) * fp->samp) +
+			fp->sofs;
 }
 
 static void
@@ -167,17 +189,18 @@ affiche(ModeInfo * mi)
 				fp->stab[(fp->sidx + x + y) % ANGLES];
 			yp = (int) (fp->size * (float) y) +
 				fp->stab[(fp->sidx + 4 * x + y + y) % ANGLES];
-      if (MI_NPIXELS(mi) <= 2 || fp->graphics_format < IS_XPM || fp->choice) {
-			if (((int) !invert) ^ XGetPixel(fp->image, x, y))
-				XSetForeground(display, fp->backGC, fp->black);
-			else if (MI_NPIXELS(mi) <= 2)
-				XSetForeground(display, fp->backGC, MI_WIN_WHITE_PIXEL(mi));
-			else
-				XSetForeground(display, fp->backGC,
-					       MI_PIXEL(mi, (y + x + fp->sidx + fp->startcolor) %
-							MI_NPIXELS(mi)));
-      } else {
-			  XSetForeground(display, fp->backGC, XGetPixel(fp->image, x, y));
+			if (MI_NPIXELS(mi) <= 2 || fp->graphics_format < IS_XPM ||
+			    fp->choice == MESSAGE_FLAG) {
+				if (((int) !invert) ^ XGetPixel(fp->image, x, y))
+					XSetForeground(display, fp->backGC, fp->black);
+				else if (MI_NPIXELS(mi) <= 2)
+					XSetForeground(display, fp->backGC, MI_WHITE_PIXEL(mi));
+				else
+					XSetForeground(display, fp->backGC,
+						       MI_PIXEL(mi, (y + x + fp->sidx + fp->startcolor) %
+							    MI_NPIXELS(mi)));
+			} else {
+				XSetForeground(display, fp->backGC, XGetPixel(fp->image, x, y));
 			}
 			if (fp->pointsize <= 1)
 				XDrawPoint(display, fp->cache, fp->backGC, xp, yp);
@@ -218,7 +241,7 @@ getText(ModeInfo * mi, XImage ** image)
 		struct utsname uts;
 
 		if (uname(&uts) < 0) {
-			text1 = strdup("uname() failed");
+			text1 = (char *) strdup("uname() failed");
 		} else {
 			char       *s;
 
@@ -227,19 +250,24 @@ getText(ModeInfo * mi, XImage ** image)
 			text1 = (char *) malloc(strlen(uts.nodename) +
 						strlen(uts.sysname) +
 						strlen(uts.release) + 10);
+#ifdef AIXV3
+			(void) sprintf(text1, "%s\n%s %s",
+				       uts.nodename, uts.sysname, "3");
+#else
 			(void) sprintf(text1, "%s\n%s %s",
 				     uts.nodename, uts.sysname, uts.release);
+#endif
 		}
 #else
-		text1 = strdup("OpenVMS");	/* It says release 0 in my utsname.h */
+		text1 = (char *) strdup("OpenVMS");	/* It says release 0 in my utsname.h */
 #endif
 	} else {
-		text1 = strdup(message);
+		text1 = (char *) strdup(message);
 	}
 	while (*text1 && (text1[strlen(text1) - 1] == '\r' ||
 			  text1[strlen(text1) - 1] == '\n'))
 		text1[strlen(text1) - 1] = 0;
-	text2 = strdup(text1);
+	text2 = (char *) strdup(text1);
 
 
 	if (mode_font == None)
@@ -253,8 +281,8 @@ getText(ModeInfo * mi, XImage ** image)
 		int         ascent, descent, direction;
 
 		token = 0;
-		XTextExtents(mode_font, line, strlen(line),
-			     &direction, &ascent, &descent, &o2);
+		(void) XTextExtents(mode_font, line, strlen(line),
+				    &direction, &ascent, &descent, &o2);
 		overall.lbearing = MAX(overall.lbearing, o2.lbearing);
 		overall.rbearing = MAX(overall.rbearing, o2.rbearing);
 		lines++;
@@ -282,16 +310,16 @@ getText(ModeInfo * mi, XImage ** image)
 
 		token = 0;
 
-		XTextExtents(mode_font, line, strlen(line),
-			     &direction, &ascent, &descent, &o2);
+		(void) XTextExtents(mode_font, line, strlen(line),
+				    &direction, &ascent, &descent, &o2);
 		xoff = ((overall.lbearing + overall.rbearing) -
 			(o2.lbearing + o2.rbearing)) / 2;
 
-		XDrawString(display, text_pixmap, gc,
-			    overall.lbearing + margin + xoff,
-			    ((mode_font->ascent * (lines + 1)) +
-			     (mode_font->descent * lines) + margin),
-			    line, strlen(line));
+		(void) XDrawString(display, text_pixmap, gc,
+				   overall.lbearing + margin + xoff,
+				   ((mode_font->ascent * (lines + 1)) +
+				    (mode_font->descent * lines) + margin),
+				   line, strlen(line));
 		lines++;
 	}
 	(void) free((void *) text1);
@@ -301,50 +329,49 @@ getText(ModeInfo * mi, XImage ** image)
 
 	*image = XGetImage(display, text_pixmap, 0, 0, width, height,
 			   1L, XYPixmap);
-	XFreePixmap(MI_DISPLAY(mi), text_pixmap);
+	XFreePixmap(display, text_pixmap);
 }
 
 static void
 init_stuff(ModeInfo * mi)
 {
-  Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
 	flagstruct *fp = &flags[MI_SCREEN(mi)];
- 
-  if (!fp->logo)
-    getImage(mi, &fp->logo, FLAG_WIDTH, FLAG_HEIGHT, FLAG_BITS,
+
+	if (!fp->logo)
+		getImage(mi, &fp->logo, FLAG_WIDTH, FLAG_HEIGHT, FLAG_BITS,
 #if defined( USE_XPM ) || defined( USE_XPMINC )
-       DEFAULT_XPM, FLAG_NAME,
+			 DEFAULT_XPM, FLAG_NAME,
 #endif
-       &fp->graphics_format, &fp->cmap,
-       &fp->black, &fp->white);
-  if (fp->cmap != None) {
-    setColormap(display, window, fp->cmap, MI_WIN_IS_INWINDOW(mi));
-    if (fp->backGC == None) {
-      XGCValues   xgcv;
- 
-      xgcv.background = fp->black;
-      fp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
-    }
-  } else {
-    fp->black = MI_WIN_BLACK_PIXEL(mi);
-    fp->backGC = MI_GC(mi);
-  }
+			 &fp->graphics_format, &fp->cmap, &fp->black);
+	if (fp->cmap != None) {
+		setColormap(display, window, fp->cmap, MI_WIN_IS_INWINDOW(mi));
+		if (fp->backGC == None) {
+			XGCValues   xgcv;
+
+			xgcv.background = fp->black;
+			fp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
+		}
+	} else {
+		fp->black = MI_BLACK_PIXEL(mi);
+		fp->backGC = MI_GC(mi);
+	}
 }
 
 static void
 free_stuff(Display * display, flagstruct * fp)
 {
-  if (fp->cmap != None) {
-    XFreeColormap(display, fp->cmap);
-    if (fp->backGC != None) {
-      XFreeGC(display, fp->backGC);
-      fp->backGC = None;
-    }
-    fp->cmap = None;
-  } else
-    fp->backGC = None;
-  destroyImage(&fp->logo, &fp->graphics_format);
+	if (fp->cmap != None) {
+		XFreeColormap(display, fp->cmap);
+		if (fp->backGC != None) {
+			XFreeGC(display, fp->backGC);
+			fp->backGC = None;
+		}
+		fp->cmap = None;
+	} else
+		fp->backGC = None;
+	destroyImage(&fp->logo, &fp->graphics_format);
 }
 
 void
@@ -361,42 +388,42 @@ init_flag(ModeInfo * mi)
 	}
 	fp = &flags[MI_SCREEN(mi)];
 
-  init_stuff(mi);
+	init_stuff(mi);
 
 	fp->width = MI_WIN_WIDTH(mi);
 	fp->height = MI_WIN_HEIGHT(mi);
 
 	if (fp->image) {
-		if (fp->choice)
-			XDestroyImage(fp->image);
+		if (fp->graphics_format == IS_XBM)
+			XFree(fp->image);
+		else
+			(void) XDestroyImage(fp->image);
+		fp->image = NULL;
 	}
 	if (MI_WIN_IS_FULLRANDOM(mi) ||
 	    (imagefile && *imagefile && message && *message) ||
 	    ((!imagefile || !*imagefile) && (!message || !*message)))
 		fp->choice = LRAND() & 1;
 	else if (imagefile && *imagefile)
-		fp->choice = 0;
+		fp->choice = IMAGE_FLAG;
 	else
-		fp->choice = 1;
-	if (fp->choice) {
+		fp->choice = MESSAGE_FLAG;
+	if (fp->choice == MESSAGE_FLAG) {
 		getText(mi, &fp->image);
 	} else {
-    if (fp->graphics_format < IS_XPM) {
-		fp->image = XCreateImage(display, MI_VISUAL(mi), 1, XYBitmap, 0,
-				fp->logo->data, fp->logo->width, fp->logo->height, 8, 0);
-		fp->image->byte_order = LSBFirst;
-		fp->image->bitmap_bit_order = LSBFirst;
-    } else {
-     int depth;
+		int         depth = MI_DEPTH(mi);
 
-		depth = MI_WIN_DEPTH(mi);
-    if (depth >= 24 && depth < 32)
-      depth = 32;
-		fp->image = XCreateImage(display, MI_VISUAL(mi), depth, ZPixmap, 0,
-				fp->logo->data, fp->logo->width, fp->logo->height, 8, 0);
+		if (depth >= 24 && depth < 32)
+			depth = 32;
+		if (fp->graphics_format < IS_XPM)
+			depth = 1;
+		fp->image = XCreateImage(display, MI_VISUAL(mi), depth,
+			 (fp->graphics_format < IS_XPM) ? XYBitmap : ZPixmap,
+					 0, fp->logo->data,
+					 (fp->graphics_format == IS_RASTERFILE && (fp->logo->width & 1)) ?
+					 fp->logo->width + 1 : fp->logo->width, fp->logo->height, 8, 0);
 		fp->image->byte_order = LSBFirst;
 		fp->image->bitmap_bit_order = LSBFirst;
-		}
 	}
 	fp->samp = MAXAMP;	/* Amplitude */
 	fp->sofs = 20;		/* ???????? */
@@ -412,10 +439,10 @@ init_flag(ModeInfo * mi)
 	fp->sidx = fp->x_flag = fp->y_flag = 0;
 
 	if (fp->cache) {
-		XFreePixmap(MI_DISPLAY(mi), fp->cache);
+		XFreePixmap(display, fp->cache);
 	}
 	if (!(fp->cache = XCreatePixmap(display, MI_WINDOW(mi),
-				    MAXW(fp), MAXH(fp), MI_WIN_DEPTH(mi)))) {
+					MAXW(fp), MAXH(fp), MI_DEPTH(mi)))) {
 		return;
 	}
 	XSetForeground(display, fp->backGC, fp->black);
@@ -440,9 +467,7 @@ init_flag(ModeInfo * mi)
 
 	initSintab(mi);
 
-  XSetForeground(display, fp->backGC, fp->black);
-  XFillRectangle(display, MI_WINDOW(mi), fp->backGC,
-           0, 0, fp->width, fp->height);
+	MI_CLEARWINDOWCOLORMAP(mi, fp->backGC, fp->black);
 }
 
 void
@@ -491,8 +516,12 @@ release_flag(ModeInfo * mi)
 
 			if (fp->cache)
 				XFreePixmap(display, fp->cache);
-			if (fp->image && fp->choice) {
-					XDestroyImage(fp->image);
+			if (fp->image) {
+				if (fp->choice == IMAGE_FLAG && (fp->graphics_format == IS_XBM ||
+					  fp->graphics_format == IS_XBMDONE))
+					XFree((caddr_t) fp->image);	/* Do not destroy data */
+				else
+					(void) XDestroyImage(fp->image);
 			}
 			free_stuff(display, fp);
 		}
