@@ -2,7 +2,7 @@
 /* gears --- 3D gear wheels */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)gears.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)gears.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -60,11 +60,21 @@ static const char sccsid[] = "@(#)gears.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt gears_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   gears_description =
+{"gears", "init_gears", "draw_gears", "release_gears",
+ "draw_gears", "init_gears", NULL, &gears_opts,
+ 1000, 1, 2, 1, 4, 1.0, "",
+ "Shows GL's gears", 0, NULL};
+
+#endif
+
 typedef struct {
 	GLfloat     view_rotx, view_roty, view_rotz;
 	GLuint      gear1, gear2, gear3;
 	GLfloat     angle;
 	GLXContext *glx_context;
+	Window      window;
 #if 0
 	Window      win;
 #endif
@@ -418,11 +428,6 @@ pinit(ModeInfo * mi)
 void
 init_gears(ModeInfo * mi)
 {
-#if 0
-	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
-
-#endif
 	int         screen = MI_SCREEN(mi);
 
 	/*Colormap    cmap; */
@@ -436,9 +441,7 @@ init_gears(ModeInfo * mi)
 	}
 	gp = &gears[screen];
 
-#if 0
-	gp->win = window;
-#endif
+	gp->window = MI_WINDOW(mi);
 	gp->view_rotx = NRAND(360);
 	gp->view_roty = NRAND(360);
 	gp->view_rotz = NRAND(360);
@@ -447,6 +450,8 @@ init_gears(ModeInfo * mi)
 	if ((gp->glx_context = init_GL(mi)) != NULL) {
 		reshape(MI_WIN_WIDTH(mi), MI_WIN_HEIGHT(mi));
 		pinit(mi);
+	} else {
+		MI_CLEARWINDOW(mi);
 	}
 }
 
@@ -488,7 +493,7 @@ release_gears(ModeInfo * mi)
 
 			if (gp->glx_context) {
 				/* Display lists MUST be freed while their glXContext is current. */
-				glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(gp->glx_context));
+				glXMakeCurrent(MI_DISPLAY(mi), gp->window, *(gp->glx_context));
 
 				if (glIsList(gp->gear1))
 					glDeleteLists(gp->gear1, 1);
@@ -502,7 +507,7 @@ release_gears(ModeInfo * mi)
 		(void) free((void *) gears);
 		gears = NULL;
 	}
-	FreeAllGL(MI_DISPLAY(mi));
+	FreeAllGL(mi);
 }
 
 
