@@ -2,7 +2,7 @@
 /* pipes --- 3D selfbuiding pipe system */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)pipes.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)pipes.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -112,7 +112,21 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt pipes_opts =
-{7, opts, 4, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   pipes_description =
+{"pipes", "init_pipes", "draw_pipes", "release_pipes",
+#if defined( MESA ) && defined( SLOW )
+ "draw_pipes",
+#else
+ "change_pipes",
+#endif
+ "change_pipes", NULL, &pipes_opts,
+ 1000, 2, 5, 500, 4, 1.0, "",
+ "Shows a selfbuilding pipe system", 0, NULL};
+
+#endif
 
 #define Scale4Window               0.1
 #define Scale4Iconic               0.07
@@ -151,6 +165,7 @@ typedef struct {
 	int         system_type;
 	int         system_length;
 	int         turncounter;
+	Window      window;
 	float      *system_color;
 	GLfloat     initial_rotation;
 	GLuint      valve, bolts, betweenbolts, elbowbolts, elbowcoins;
@@ -622,6 +637,7 @@ init_pipes(ModeInfo * mi)
 	}
 	pp = &pipes[screen];
 
+	pp->window = MI_WINDOW(mi);
 	if ((pp->glx_context = init_GL(mi)) != NULL) {
 
 		reshape(mi, MI_WIN_WIDTH(mi), MI_WIN_HEIGHT(mi));
@@ -662,6 +678,8 @@ init_pipes(ModeInfo * mi)
 		} else {
 			pp->system_length = MI_SIZE(mi);
 		}
+	} else {
+		MI_CLEARWINDOW(mi);
 	}
 }
 
@@ -985,7 +1003,7 @@ release_pipes(ModeInfo * mi)
 			if (pp->glx_context) {
 
 				/* Display lists MUST be freed while their glXContext is current. */
-				glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(pp->glx_context));
+				glXMakeCurrent(MI_DISPLAY(mi), pp->window, *(pp->glx_context));
 
 				if (pp->valve)
 					glDeleteLists(pp->valve, 1);
@@ -1013,7 +1031,7 @@ release_pipes(ModeInfo * mi)
 		(void) free((void *) pipes);
 		pipes = NULL;
 	}
-	FreeAllGL(MI_DISPLAY(mi));
+	FreeAllGL(mi);
 }
 
 #endif
