@@ -2,7 +2,7 @@
 /* sproingiewrap.c - sproingies wrapper */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)sproingiewrap.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)sproingiewrap.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -74,6 +74,15 @@ static const char sccsid[] = "@(#)sproingiewrap.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt sproingies_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   sproingies_description =
+{"sproingies", "init_sproingies", "draw_sproingies", "release_sproingies",
+ "refresh_sproingies", "init_sproingies", NULL, &sproingies_opts,
+ 1000, 5, 0, 400, 4, 1.0, "",
+ "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL};
+
+#endif
+
 #define MINSIZE 32
 
 #include <GL/glu.h>
@@ -98,6 +107,7 @@ typedef struct {
 	GLuint      count;
 	GLXContext *glx_context;
 	int         mono;
+	Window      window;
 } sproingiesstruct;
 
 static sproingiesstruct *sproingies = NULL;
@@ -135,7 +145,7 @@ init_sproingies(ModeInfo * mi)
 	sp = &sproingies[screen];
 
 	sp->mono = (MI_WIN_IS_MONO(mi) ? 1 : 0);
-
+	sp->window = window;
 	if ((sp->glx_context = init_GL(mi)) != NULL) {
 
 		if ((cycles & 1) || MI_WIN_IS_WIREFRAME(mi))
@@ -173,6 +183,8 @@ init_sproingies(ModeInfo * mi)
 		swap_display = display;
 		swap_window = window;
 		DisplaySproingies(MI_SCREEN(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
 	}
 }
 
@@ -217,7 +229,7 @@ release_sproingies(ModeInfo * mi)
 
 			if (sp->glx_context) {
 
-				glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sp->glx_context));
+				glXMakeCurrent(MI_DISPLAY(mi), sp->window, *(sp->glx_context));
 				CleanupSproingies(MI_SCREEN(mi));
 			}
 		}
@@ -225,7 +237,7 @@ release_sproingies(ModeInfo * mi)
 		(void) free((void *) sproingies);
 		sproingies = NULL;
 	}
-	FreeAllGL(MI_DISPLAY(mi));
+	FreeAllGL(mi);
 }
 
 #endif
