@@ -2,7 +2,7 @@
 /* superquadrics --- 3D mathematical shapes */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)superquadrics.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)superquadrics.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -120,7 +120,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt superquadrics_opts =
-{1, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   superquadrics_description =
+{"superquadrics", "init_superquadrics", "draw_superquadrics", "release_superquadrics",
+ "refresh_superquadrics", "init_superquadrics", NULL, &superquadrics_opts,
+ 1000, 25, 40, 1, 4, 1.0, "",
+ "Shows 3D mathematical shapes", 0, NULL};
+
+#endif
 
 #include <GL/glu.h>
 
@@ -725,6 +734,8 @@ init_superquadrics(ModeInfo * mi)
 		DisplaySuperquadrics(sp);
 		glFinish();
 		glXSwapBuffers(display, window);
+	} else {
+		MI_CLEARWINDOW(mi);
 	}
 }
 
@@ -759,7 +770,7 @@ release_superquadrics(ModeInfo * mi)
 		(void) free((void *) superquadrics);
 		superquadrics = NULL;
 	}
-	FreeAllGL(MI_DISPLAY(mi));
+	FreeAllGL(mi);
 }
 
 
