@@ -2,7 +2,7 @@
 /* hop --- real plane fractals */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)hop.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)hop.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -139,7 +139,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt hop_opts =
-{20, opts, 10, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   hop_description =
+{"hop", "init_hop", "draw_hop", "release_hop",
+ "refresh_hop", "init_hop", NULL, &hop_opts,
+ 10000, 1000, 2500, 1, 64, 1.0, "",
+ "Shows real plane iterated fractals", 0, NULL};
+
+#endif
 
 #define MARTIN 0
 #define SINE 7
@@ -357,9 +366,9 @@ init_hop(ModeInfo * mi)
 	if (!hp->pointBuffer)
 		hp->pointBuffer = (XPoint *) malloc(hp->bufsize * sizeof (XPoint));
 
-	XClearWindow(display, MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
-	XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	hp->count = 0;
 }
 
@@ -492,5 +501,5 @@ release_hop(ModeInfo * mi)
 void
 refresh_hop(ModeInfo * mi)
 {
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
