@@ -2,7 +2,7 @@
 /* laser --- spinning lasers */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)laser.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)laser.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -43,6 +43,15 @@ static const char sccsid[] = "@(#)laser.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt laser_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   laser_description =
+{"laser", "init_laser", "draw_laser", "release_laser",
+ "refresh_laser", "init_laser", NULL, &laser_opts,
+ 20000, -10, 200, 1, 64, 1.0, "",
+ "Shows spinning lasers", 0, NULL};
+
+#endif
+
 #define MINREDRAW 3		/* Number of redrawn on each frame */
 #define MAXREDRAW 8
 
@@ -128,13 +137,13 @@ init_laser(ModeInfo * mi)
 	if (lp->stippledGC == NULL) {
 		XGCValues   gcv;
 
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
-		lp->gcv_black.foreground = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
+		lp->gcv_black.foreground = MI_BLACK_PIXEL(mi);
 		lp->stippledGC = XCreateGC(MI_DISPLAY(mi), MI_WINDOW(mi),
 					   GCForeground | GCBackground, &gcv);
 	}
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	if (MINDIST < lp->width - MINDIST)
 		lp->cx = RANGE_RAND(MINDIST, lp->width - MINDIST);
@@ -181,7 +190,7 @@ init_laser(ModeInfo * mi)
 			l->gcv.foreground = MI_PIXEL(mi, c);
 			c = (c + COLORSTEP) % MI_NPIXELS(mi);
 		} else
-			l->gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
+			l->gcv.foreground = MI_WHITE_PIXEL(mi);
 	}
 }
 
