@@ -127,7 +127,7 @@ ModStruct   life3d_description =
 
 #define COLORBASE 3
 #define COLORS (COLORBASE + 2)
-#define COLORSTEP (MI_NPIXELS(mi)/COLORBASE)		/* 21 different colors per side */
+#define COLORSTEP (MI_NPIXELS(mi)/COLORBASE)	/* 21 different colors per side */
 
 #define BLACK 0
 #define RED 1
@@ -1313,7 +1313,8 @@ parseFile()
 	if (done)
 		return;
 	done = True;
-	if (!strlen(life3dfile) || ((file = my_fopen(life3dfile, "r")) == NULL)) {
+	if (!life3dfile || !*life3dfile ||
+	    ((file = my_fopen(life3dfile, "r")) == NULL)) {
 		/*(void) fprintf(stderr, "could not read file \"%s\"\n", life3dfile); */
 		return;
 	}
