@@ -2,7 +2,7 @@
 /* lisa --- animated full-loop lisajous figures */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)lisa.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)lisa.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -47,8 +47,6 @@ static const char sccsid[] = "@(#)lisa.c	4.04 97/07/28 xlockmore";
  "*ncolors: 200 \n"
 #define UNIFORM_COLORS
 #include "xlockmore.h"		/* in xscreensaver distribution */
-void        refresh_lisa(ModeInfo * mi);
-void        change_lisa(ModeInfo * mi);
 
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -59,25 +57,33 @@ void        change_lisa(ModeInfo * mi);
 
 static Bool additive;
 
-static XrmOptionDescRec lisa_xrm_opts[] =
+static XrmOptionDescRec opts[] =
 {
 	{"-additive", ".lisa.additive", XrmoptionNoArg, (caddr_t) "True"},
 	{"+additive", ".lisa.additive", XrmoptionNoArg, (caddr_t) "False"}
 };
 
-static argtype lisa_vars[] =
+static argtype vars[] =
 {
 	{(caddr_t *) & additive, "additive", "Additive", DEF_ADDITIVE, t_Bool}
 };
 
-static OptionStruct lisa_vars_desc[] =
+static OptionStruct desc[] =
 {
 	{"-/+additive", "turn on/off additive functions mode"}
 };
 
 ModeSpecOpt lisa_opts =
-{2, lisa_xrm_opts, 1, lisa_vars, lisa_vars_desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
 
+#ifdef USE_MODULES
+ModStruct   lisa_description =
+{"lisa", "init_lisa", "draw_lisa", "release_lisa",
+ "refresh_lisa", "change_lisa", NULL, &lisa_opts,
+ 25000, 1, 256, -1, 64, 1.0, "",
+ "Shows animated lisajous loops", 0, NULL};
+
+#endif
 
 #define  DRAWLINES    1
 #define  TWOLOOPS     1
@@ -91,7 +97,7 @@ ModeSpecOpt lisa_opts =
 if (MI_NPIXELS(mi) > 2) { \
   XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, loop->color)); \
   if (++(loop->color) >= MI_NPIXELS(mi)) { loop->color=0; } \
-  } else { XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi)); }
+  } else { XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi)); }
 #define getRadius(context) \
   ((context->width > context->height)?context->height:context->width) * 3 / 8
 #define checkRadius(loop, context) \
@@ -276,7 +282,7 @@ drawlisa(ModeInfo * mi, lisas * loop)
 
 #if defined DRAWLINES
 		/* erase the last cycle's point */
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 		XDrawLine(MI_DISPLAY(mi), MI_WINDOW(mi), \
 			  MI_GC(mi), lp[pctr].x, lp[pctr].y, \
 			  lp[(pctr + 1) % loop->nsteps].x, \
@@ -292,7 +298,7 @@ drawlisa(ModeInfo * mi, lisas * loop)
 			  np[(pctr + 1) % loop->nsteps].y);
 #else
 		/* erase the last cycle's point */
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 		XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), \
 			   MI_GC(mi), lp[pctr].x, lp[pctr].y);
 
@@ -320,7 +326,7 @@ initlisa(ModeInfo * mi, lisas * loop)
 	if (MI_NPIXELS(mi) > 2) {
 		loop->color = 0;
 	} else
-		loop->color = MI_WIN_WHITE_PIXEL(mi);
+		loop->color = MI_WHITE_PIXEL(mi);
 	loop->nsteps = MI_CYCLES(mi);
 	if (loop->nsteps == 0)
 		loop->nsteps = 1;
@@ -383,6 +389,35 @@ initlisa(ModeInfo * mi, lisas * loop)
 	}
 }
 
+void
+refresh_lisa(ModeInfo * mi)
+{
+	lisacons   *lc = &Lisa[MI_SCREEN(mi)];
+	int         lctr;
+
+	for (lctr = 0; lctr < lc->nlisajous; lctr++) {
+		drawlisa(mi, &lc->lisajous[lctr]);
+	}
+}
+
+
+void
+change_lisa(ModeInfo * mi)
+{
+	lisacons   *lc = &Lisa[MI_SCREEN(mi)];
+	lisas      *loop;
+	int         lctr;
+
+	lc->loopcount = 0;
+	for (lctr = 0; lctr < lc->nlisajous; lctr++) {
+		loop = &lc->lisajous[lctr];
+		loop->function[1] = &Function[(loop->function[0]->index + 1) %
+					      NUMSTDFUNCS];
+		loop->melting = loop->nsteps - 1;
+		loop->nfuncs = 2;
+	}
+}
+
 void
 init_lisa(ModeInfo * mi)
 {
@@ -406,7 +441,6 @@ init_lisa(ModeInfo * mi)
 		if ((lc->lisajous = (lisas *) calloc(lc->nlisajous, sizeof (lisas))) \
 		    == NULL)
 			return;
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
 		for (lctr = 0; lctr < lc->nlisajous; lctr++) {
 			initlisa(mi, &lc->lisajous[lctr]);
 			lc->loopcount++;
@@ -414,7 +448,8 @@ init_lisa(ModeInfo * mi)
 	} else {
 		refresh_lisa(mi);
 	}
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -428,17 +463,6 @@ draw_lisa(ModeInfo * mi)
 	refresh_lisa(mi);
 }
 
-void
-refresh_lisa(ModeInfo * mi)
-{
-	lisacons   *lc = &Lisa[MI_SCREEN(mi)];
-	int         lctr;
-
-	for (lctr = 0; lctr < lc->nlisajous; lctr++) {
-		drawlisa(mi, &lc->lisajous[lctr]);
-	}
-}
-
 void
 release_lisa(ModeInfo * mi)
 {
@@ -460,20 +484,3 @@ release_lisa(ModeInfo * mi)
 		Lisa = NULL;
 	}
 }
-
-void
-change_lisa(ModeInfo * mi)
-{
-	lisacons   *lc = &Lisa[MI_SCREEN(mi)];
-	lisas      *loop;
-	int         lctr;
-
-	lc->loopcount = 0;
-	for (lctr = 0; lctr < lc->nlisajous; lctr++) {
-		loop = &lc->lisajous[lctr];
-		loop->function[1] = &Function[(loop->function[0]->index + 1) %
-					      NUMSTDFUNCS];
-		loop->melting = loop->nsteps - 1;
-		loop->nfuncs = 2;
-	}
-}
