@@ -69,7 +69,7 @@ ModeSpecOpt loop_opts =
 ModStruct   loop_description =
 {"loop", "init_loop", "draw_loop", "release_loop",
  "refresh_loop", "init_loop", NULL, &loop_opts,
- 100000, 1, 1600, -12, 64, 1.0, 15, "",
+ 100000, 1, 1600, -12, 64, 1.0, "",
  "Shows Langton's self-producing loops", 0, NULL};
 
 #endif
