@@ -215,7 +215,7 @@ init_mandelbrot(ModeInfo * mi)
 		&(mp->ncolors), &truecolor, &redmask, &greenmask, &bluemask);
 	if (!mp->fixed_colormap) {
 		preserve = preserveColors(MI_BLACK_PIXEL(mi), MI_WHITE_PIXEL(mi),
-           MI_BG_PIXEL(mi), MI_FG_PIXEL(mi));
+					  MI_BG_PIXEL(mi), MI_FG_PIXEL(mi));
 		mp->bgcol.pixel = MI_BG_PIXEL(mi);
 		mp->fgcol.pixel = MI_FG_PIXEL(mi);
 		XQueryColor(display, MI_COLORMAP(mi), &(mp->bgcol));
@@ -286,7 +286,7 @@ draw_mandelbrot(ModeInfo * mi)
 
 	log_top = log((float) top);
 	screen_width = MI_WIN_WIDTH(mi);
-	screen_height = MI_WIN_HEIGHT(mi);
+	screen_height = ((MI_WIN_HEIGHT(mi) + 1) >> 1) << 1;	/* screen_height even */
 
 	if (mp->counter % 5 == 0) {
 		/* advance "drawing" color */
