@@ -43,7 +43,7 @@ static const char sccsid[] = "@(#)pacman.c	4.07 97/11/24 xlockmore";
 #define DEFAULTS "*delay: 100000 \n" \
  "*count: 10 \n" \
  "*size: 0 \n"
- "*ncolors: 5 \n"
+"*ncolors: 6 \n"
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -56,7 +56,7 @@ ModeSpecOpt pacman_opts =
 ModStruct   pacman_description =
 {"pacman", "init_pacman", "draw_pacman", "release_pacman",
  "refresh_pacman", "init_pacman", NULL, &pacman_opts,
- 100000, 10, 1, 0, 5, 64, 1.0, "",
+ 100000, 10, 1, 0, 64, 1.0, "",
  "Shows Pacman(tm)", 0, NULL};
 
 #endif
@@ -81,9 +81,9 @@ ModStruct   pacman_description =
 #define LB   0x0100
 #define ALL  0x1111
 
-#define YELLOW (MI_NPIXELS(mi) / 6)  
-#define GREEN (23 * MI_NPIXELS(mi) / 64)  
-#define BLUE (45 * MI_NPIXELS(mi) / 64)  
+#define YELLOW (MI_NPIXELS(mi) / 6)
+#define GREEN (23 * MI_NPIXELS(mi) / 64)
+#define BLUE (45 * MI_NPIXELS(mi) / 64)
 
 typedef struct {
 	int         col, row;
@@ -101,8 +101,8 @@ typedef struct {
 	int         incx, incy;
 	GC          stippledGC;
 	Pixmap      ghostPixmap;
-	int ghostWidth, ghostHeight;
-  int graphics_format;
+	int         ghostWidth, ghostHeight;
+	int         graphics_format;
 	beingstruct pacman;
 	beingstruct *ghosts;
 	int         nghosts;
@@ -124,9 +124,9 @@ clearcorners(ModeInfo * mi)
 	pacmangamestruct *pp = &pacmangames[MI_SCREEN(mi)];
 
 	if (MI_NPIXELS(mi) > 2)
-	  XSetForeground(display, gc, MI_PIXEL(mi, GREEN));
-		else
-			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_PIXEL(mi, GREEN));
+	else
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	if ((pp->pacman.nextrow == 0) && (pp->pacman.nextcol == 0)) {
 		XFillRectangle(display, window, gc, 0, 0,
 		((pp->width / pp->ncols) / 2), ((pp->width / pp->ncols) / 2));
@@ -470,17 +470,17 @@ init_pacman(ModeInfo * mi)
 		pp->ghostHeight = CELL_HEIGHT;
 		if (size == 0) {
 			if (pp->ghostPixmap == None) {
-    		getPixmap(display, window, CELL_WIDTH, CELL_HEIGHT, CELL_BITS,
-    			&(pp->ghostWidth), &(pp->ghostHeight), &(pp->ghostPixmap),
-    			&(pp->graphics_format));
+				getPixmap(display, window, CELL_WIDTH, CELL_HEIGHT, CELL_BITS,
+					  &(pp->ghostWidth), &(pp->ghostHeight), &(pp->ghostPixmap),
+					  &(pp->graphics_format));
 				pp->xs = pp->ghostWidth;
 				pp->ys = pp->ghostHeight;
 			}
 		}
 	}
 	if (size == 0) {
-   if (MINGRIDSIZE * pp->ghostWidth > pp->width ||
-       MINGRIDSIZE * pp->ghostHeight > pp->height) {
+		if (MINGRIDSIZE * pp->ghostWidth > pp->width ||
+		    MINGRIDSIZE * pp->ghostHeight > pp->height) {
 			pp->pixelmode = 1;
 			pp->xs = pp->ys = MAX(MINSIZE, MIN(pp->width, pp->height) / MINGRIDSIZE);
 		} else {
