@@ -2,7 +2,7 @@
 /* penrose --- quasiperiodic tilings */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)penrose.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)penrose.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -91,7 +91,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt penrose_opts =
-{2, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   penrose_description =
+{"penrose", "init_penrose", "draw_penrose", "release_penrose",
+ "init_penrose", "init_penrose", NULL, &penrose_opts,
+ 10000, 1, 1, -40, 64, 1.0, "",
+ "Shows Penrose's quasiperiodic tilings", 0, NULL};
+
+#endif
 
 /*-
  * Annoyingly the ANSI C library people have reserved all identifiers
@@ -106,8 +115,8 @@ ModeSpecOpt penrose_opts =
  * vertex rules only allow at most seven tiles to meet at a vertex.
  */
 
-#define CELEBRATE 31415927	/* This causes a pause, an error occurred. */
-#define COMPLETION 3141593	/* This causes a pause, tiles filled up screen. */
+#define CELEBRATE 31415		/* This causes a pause, an error occurred. */
+#define COMPLETION 3141		/* This causes a pause, tiles filled up screen. */
 
 #define MAX_TILES_PER_VERTEX 7
 #define N_VERTEX_RULES 8
@@ -230,6 +239,8 @@ typedef struct {
 	forced_pool_c forced;
 	int         done, failures;
 	int         thick_color, thin_color;
+	int         busyLoop;
+	Bool        ammann;
 } tiling_c;
 
 static tiling_c *tilings;	/* = {0} */
@@ -310,7 +321,7 @@ vertex_dir(ModeInfo * mi, fringe_node_c * vertex, unsigned side)
 			(void) fprintf(stderr, "v2->fived[%d]=%d, vertex->fived[%d]=%d\n",
 				       i, v2->fived[i], i, vertex->fived[i]);
 	}
-	MI_PAUSE(mi) = CELEBRATE;
+	tp->busyLoop = CELEBRATE;
 	return 0;
 }
 
@@ -404,6 +415,11 @@ init_penrose(ModeInfo * mi)
 			return;
 	}
 	tp = &tilings[MI_SCREEN(mi)];
+
+	if (MI_WIN_IS_FULLRANDOM(mi))
+		tp->ammann = (Bool) (LRAND() & 1);
+	else
+		tp->ammann = ammann;
 	tp->done = False;
 	tp->failures = 0;
 	tp->width = MI_WIN_WIDTH(mi);
@@ -624,12 +640,12 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 		else
 			XSetForeground(display, gc, MI_PIXEL(mi, tp->thin_color));
 	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	XFillPolygon(display, window, gc, pts, 4, Convex, CoordModeOrigin);
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	XDrawLines(display, window, gc, pts, 5, CoordModeOrigin);
 
-	if (ammann) {
+	if (tp->ammann) {
 		/* Draw some Ammann lines for debugging purposes.  This will probably
 		   fail miserably on a b&w display. */
 
@@ -644,7 +660,7 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 			if (MI_NPIXELS(mi) > 2)
 				XSetForeground(display, gc, MI_PIXEL(mi, tp->thin_color));
 			else {
-				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XSetLineAttributes(display, gc, 1, LineOnOffDash, CapNotLast, JoinMiter);
 			}
 			XDrawLine(display, window, gc,
@@ -658,7 +674,7 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
 			if (MI_NPIXELS(mi) > 2)
 				XSetForeground(display, gc, MI_PIXEL(mi, tp->thick_color));
 			else {
-				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XSetLineAttributes(display, gc, 1, LineOnOffDash, CapNotLast, JoinMiter);
 			}
 			XDrawLine(display, window, gc,
@@ -685,7 +701,7 @@ draw_tile(fringe_node_c * v1, fringe_node_c * v2,
  * might get called with an untileable vertex, causing ( n <= 1).
  * (This is what the tp->done checks for).
  *
- * A MI_PAUSE celebrates the dislocation.
+ * A delayLoop celebrates the dislocation.
  */
 static void
 check_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
@@ -699,7 +715,7 @@ check_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 		if (MI_WIN_IS_VERBOSE(mi)) {
 			(void) fprintf(stderr, "Dislocation occurred!\n");
 		}
-		MI_PAUSE(mi) = CELEBRATE;	/* Should be able to recover */
+		tp->busyLoop = CELEBRATE;	/* Should be able to recover */
 	}
 	if (1 == find_completions(vertex, hits, n_hits, S_LEFT, 0 /*, False */ ))
 		forced_sides |= S_LEFT;
@@ -754,7 +770,7 @@ delete_vertex(ModeInfo * mi, fringe_node_c * vertex, tiling_c * tp)
 			(void) fprintf(stderr, "Weirdness in delete_penrose()\n");
 			(void) fprintf(stderr, "tp->fringe.nodes == vertex\n");
 		}
-		MI_PAUSE(mi) = CELEBRATE;
+		tp->busyLoop = CELEBRATE;
 	}
 	if (vertex->list_ptr != 0) {
 		forced_node_c *node = *vertex->list_ptr;
@@ -896,7 +912,7 @@ alloc_vertex(ModeInfo * mi, angle_c dir, fringe_node_c * from, tiling_c * tp)
 			(void) fprintf(stderr, "Weirdness in alloc_vertex()\n");
 			(void) fprintf(stderr, "v = 0\n");
 		}
-		MI_PAUSE(mi) = CELEBRATE;
+		tp->busyLoop = CELEBRATE;
 	}
 	*v = *from;
 	add_unit_vec(dir, v->fived);
@@ -993,7 +1009,7 @@ add_tile(ModeInfo * mi,
 	} while (node != tp->fringe.nodes);
 
 	/* Rechain. */
-	if (!(fc & FC_CUT_THIS))
+	if (!(fc & FC_CUT_THIS)) {
 		if (side == S_LEFT) {
 			vertex->next = right;
 			right->prev = vertex;
@@ -1001,6 +1017,7 @@ add_tile(ModeInfo * mi,
 			vertex->prev = left;
 			left->next = vertex;
 		}
+	}
 	if (!(fc & FC_CUT_FAR)) {
 		if (!(fc & FC_CUT_LEFT)) {
 			far->next = left;
@@ -1124,7 +1141,7 @@ add_random_tile(fringe_node_c * vertex, ModeInfo * mi)
 		tp->thin_color = (NRAND(2 * MI_NPIXELS(mi) / 3) + tp->thick_color +
 				  MI_NPIXELS(mi) / 6) % MI_NPIXELS(mi);
 	} else
-		tp->thick_color = tp->thin_color = MI_WIN_WHITE_PIXEL(mi);
+		tp->thick_color = tp->thin_color = MI_WHITE_PIXEL(mi);
 	n_hits = match_rules(vertex, hits, False);
 	side = NRAND(2) ? S_LEFT : S_RIGHT;
 	n = find_completions(vertex, hits, n_hits, side, vtypes /*, False */ );
@@ -1201,6 +1218,10 @@ draw_penrose(ModeInfo * mi)
 	int         i = 0, n;
 	forced_node_c *p = tp->forced.first;
 
+	if (tp->busyLoop > 0) {
+		tp->busyLoop--;
+		return;
+	}
 	if (tp->done || tp->failures >= 100) {
 		init_penrose(mi);
 		return;
@@ -1209,14 +1230,15 @@ draw_penrose(ModeInfo * mi)
 	if (tp->fringe.nodes->prev == tp->fringe.nodes->next) {
 		vertex_type_c vtype = VT_TOTAL_MASK & LRAND();
 
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+		MI_CLEARWINDOW(mi);
+
 		(void) add_tile(mi, tp->fringe.nodes, S_LEFT, vtype);
 		return;
 	}
 	/* No visible nodes left. */
 	if (tp->fringe.n_nodes == 0) {
 		tp->done = True;
-		MI_PAUSE(mi) = COMPLETION;	/* Just finished drawing */
+		tp->busyLoop = COMPLETION;	/* Just finished drawing */
 		return;
 	}
 	if (tp->forced.n_visible > 0 && tp->failures < 10) {
