@@ -2,7 +2,7 @@
 /* puzzle --- the familiar Sam Loyd puzzle */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)puzzle.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)puzzle.c	4.09 98/03/20 xlockmore";
 
 #endif
 
@@ -22,6 +22,8 @@ static const char sccsid[] = "@(#)puzzle.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 20-Mar-98: Ideas from slip.c and eyes.c, problems with XGetImage when
+ *            image moved off screen (-inwindow or -debug).
  * 10-May-97: Compatible with xscreensaver
  * 15-Mar-96: cleaned up, NUMBERED compile-time switch is now broken.
  * Feb-96: combined with rastering.  Jouk Jansen <joukj@crys.chem.uva.nl>.
@@ -36,13 +38,6 @@ static const char sccsid[] = "@(#)puzzle.c	4.07 97/11/24 xlockmore";
  * if you have a picture on your background.
  */
 
-/*-
--visual DirectColor may yield
-X Error of failed request:  BadMatch (invalid parameter attributes)
- Major opcode of failed request:  2 (X_ChangeWindowAttributes)
-cartoon.c has the same disease.
- */
-
 #ifdef STANDALONE
 #define PROGCLASS "Puzzle"
 #define HACK_INIT init_puzzle
@@ -99,11 +94,12 @@ typedef struct {
 
 	/* Delta move stuff */
 	int         movingBox;
-	XImage     *box;
+	Pixmap      bufferBox;
 	int         cfactor, rfactor;
 	int         Lp;
 	int         cbs, cbw, rbs, rbw;
 	int         lengthOfMove;
+
 #ifdef NUMBERED
 	XImage     *image;
 	GC          gc;
@@ -180,9 +176,9 @@ NumberScreen(ModeInfo * mi)
 					    pp->boxsize.y > pp->fontHeight) {
 						(void) sprintf(buf, "%d", count);
 						(void) XDrawString(display, window, pp->gc,
-							    pos.x + letter.x - letterOffset * digitOffset +
-							    pp->randompos.x,
-							    pos.y + letter.y + pp->randompos.y, buf, digitOffset);
+								   pos.x + letter.x - letterOffset * digitOffset +
+							     pp->randompos.x,
+								   pos.y + letter.y + pp->randompos.y, buf, digitOffset);
 					}
 					XDrawRectangle(display, window, pp->gc,
 						       pos.x + 1 + pp->randompos.x, pos.y + 1 + pp->randompos.y,
@@ -271,66 +267,23 @@ static void
 setupmovedelta(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
-	int         screen = MI_SCREEN(mi);
-	puzzlestruct *pp = &puzzs[screen];
-	int         x, y, w, h;
+	puzzlestruct * pp = &puzzs[MI_SCREEN(mi)];
 
-	/* if (pp->boxsize.x > 1 && pp->boxsize.y > 1) This is already true */
-
-	/* There is a big problem here when used with -inwindow , possible solutions
-	 * seem to expose more levels of complication when implemented.
-	 */
-	/* UnmapNotify VisibilityPartiallyObscured)) */
-	if (pp->box) {
-		(void) XDestroyImage(pp->box);
-		pp->box = NULL;
-	} {
-		XWindowAttributes wa, pwa =
-		{0};
-		Window      root, parent, *children = NULL;
-		unsigned int nchildren;
-
-		if (XQueryTree(display, window, &root, &parent,
-			       &children, &nchildren)) {
-			if (children && nchildren)
-				XFree((caddr_t) children);
-			/*
-			 * X Error of failed request:  Bad Window has been reported.
-			 * It is probably on the next line or the line after. */
-			(void) XGetWindowAttributes(display, window, &wa);
-			if (parent)
-				(void) XGetWindowAttributes(display, parent, &pwa);
-			x = pp->nextcol * pp->boxsize.x + pp->randompos.x + 1;
-			y = pp->nextrow * pp->boxsize.y + pp->randompos.y + 1;
-			w = pp->boxsize.x - 2;
-			h = pp->boxsize.y - 2;
-			/* Test to if viewable and on screen else a BadMatch is generated */
-			if (wa.map_state == IsViewable &&
-			(pwa.x + wa.x + x >= 0) && (pwa.y + wa.y + y >= 0) &&
-			    (pwa.x + wa.x + x + w < DisplayWidth(display, screen)) &&
-			    (pwa.y + wa.y + y + h < DisplayHeight(display, screen))) {
-				pp->box = XGetImage(display, window, x, y, w, h, AllPlanes, ZPixmap);
-				XFlush(display);
-			}
-		}
+	if (pp->bufferBox != None) {
+		XFreePixmap(display, pp->bufferBox);
+		pp->bufferBox = None;
 	}
-#if 0
-	/* Current state of the puzzle has to be known for this to work
-	 * (pp->nextrow) and (pp->nextcol) are the correct starting values, but
-	 * as the puzzle progresses it is not correct.
-	 */
-	if (pp->graphics_format >= IS_XPM)
-		/* This can only be used with a fully initialized image where logo is
-		 * initialized directly or indirectly with XCreateImage .
-		 * It also has the disadvantage that it uses XGetPixel and XPutPixel
-		 * internally, so it could be slow.
-		 */
-		pp->box = XSubImage(pp->logo,
-				    (pp->nextrow) * pp->boxsize.x + 1,
-				    (pp->nextcol) * pp->boxsize.y + 1,
-				    pp->boxsize.x - 2, pp->boxsize.y - 2);
-#endif
+	pp->bufferBox = XCreatePixmap(display, MI_WINDOW(mi),
+				 pp->boxsize.x, pp->boxsize.y, MI_DEPTH(mi));
+	if (pp->bufferBox == None) {
+		return;
+	}
+	XCopyArea(MI_DISPLAY(mi), MI_WINDOW(mi), pp->bufferBox, pp->backGC,
+			 pp->nextcol * pp->boxsize.x + pp->randompos.x + 1,
+			 pp->nextrow * pp->boxsize.y + pp->randompos.y + 1,
+			 pp->boxsize.x - 2, pp->boxsize.y - 2, 0, 0);
+	XFlush(MI_DISPLAY(mi));
+
 	if (pp->nextcol > pp->col) {
 		pp->cfactor = -1;
 		pp->cbs = pp->boxsize.x;
@@ -386,18 +339,20 @@ wrapupmovedelta(ModeInfo * mi)
 {
 	puzzlestruct *pp = &puzzs[MI_SCREEN(mi)];
 
-	if (pp->box) {
-		(void) XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box, 0, 0,
-			  pp->col * pp->boxsize.x + pp->randompos.x + 1,
-			  pp->row * pp->boxsize.y + pp->randompos.y + 1,
-			  pp->boxsize.x - 2, pp->boxsize.y - 2);
+	if (pp->bufferBox) {
+
+		XCopyArea(MI_DISPLAY(mi), pp->bufferBox, MI_WINDOW(mi), pp->backGC,
+				 0, 0, pp->boxsize.x - 2, pp->boxsize.y - 2,
+			       pp->col * pp->boxsize.x + pp->randompos.x + 1,
+			      pp->row * pp->boxsize.y + pp->randompos.y + 1);
+
 		XFlush(MI_DISPLAY(mi));
 
 		pp->row = pp->nextrow;
 		pp->col = pp->nextcol;
 
-		(void) XDestroyImage(pp->box);
-		pp->box = NULL;
+		XFreePixmap(MI_DISPLAY(mi), pp->bufferBox);
+		pp->bufferBox = None;
 	}
 }
 
@@ -411,9 +366,10 @@ moveboxdelta(ModeInfo * mi)
 	pp->Lp * pp->rfactor + pp->randompos.y;
 
 	if (pp->Lp <= pp->lengthOfMove) {
-		if (pp->box) {
-			(void) XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box,
-				  0, 0, cf + 1, rf + 1, pp->boxsize.x - 2, pp->boxsize.y - 2);
+		if (pp->bufferBox) {
+			XCopyArea(MI_DISPLAY(mi), pp->bufferBox, MI_WINDOW(mi),
+					 pp->backGC, 0, 0, pp->boxsize.x - 2, pp->boxsize.y - 2,
+           cf + 1, rf + 1);
 			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC,
 				       cf + pp->cbs - 1, rf + pp->rbs - 1, pp->cbw + 2, pp->rbw + 2);
 		}
@@ -505,11 +461,8 @@ init_puzzle(ModeInfo * mi)
 		pp->windowsize.y = 7;
 	pp->forward = 1;
 	pp->prev = 0;
-
-	if (pp->box) {
-		(void) XDestroyImage(pp->box);
-		pp->box = NULL;
-	}
+	/* don't want any exposure events from XCopyArea */
+	XSetGraphicsExposures(display, pp->backGC, False);
 
 	MI_CLEARWINDOWCOLORMAP(mi, pp->backGC, pp->black);
 
@@ -530,6 +483,10 @@ init_puzzle(ModeInfo * mi)
 	pp->count.x = size.x / pp->boxsize.x;
 	pp->count.y = size.y / pp->boxsize.y;
 
+	if (pp->bufferBox != None) {
+		XFreePixmap(display, pp->bufferBox);
+		pp->bufferBox = None;
+	}
 	pp->usablewindow.x = pp->count.x * pp->boxsize.x;
 	pp->usablewindow.y = pp->count.y * pp->boxsize.y;
 	pp->offsetwindow.x = (pp->windowsize.x - pp->usablewindow.x) / 2;
@@ -549,9 +506,9 @@ init_puzzle(ModeInfo * mi)
 			XSetForeground(display, pp->backGC, MI_PIXEL(mi, NRAND(MI_NPIXELS(mi))));
 		(void) XPutImage(display, window, pp->backGC, pp->logo,
 		(int) (NRAND(MAX(1, (pp->logo->width - pp->usablewindow.x)))),
-			  (int) (NRAND(MAX(1, (pp->logo->height - pp->usablewindow.y)))),
-			  pp->randompos.x, pp->randompos.y,
-			  pp->usablewindow.x, pp->usablewindow.y);
+				 (int) (NRAND(MAX(1, (pp->logo->height - pp->usablewindow.y)))),
+				 pp->randompos.x, pp->randompos.y,
+				 pp->usablewindow.x, pp->usablewindow.y);
 		XSetForeground(display, pp->backGC, pp->black);
 		for (x = 0; x <= pp->count.x; x++) {
 			int         tempx = x * pp->boxsize.x;
@@ -632,8 +589,8 @@ release_puzzle(ModeInfo * mi)
 
 			if (pp->fixbuff != NULL)
 				(void) free((void *) pp->fixbuff);
-			if (pp->box)
-				(void) XDestroyImage(pp->box);
+			if (pp->bufferBox != None)
+				XFreePixmap(MI_DISPLAY(mi), pp->bufferBox);
 			free_stuff(MI_DISPLAY(mi), pp);
 		}
 		(void) free((void *) puzzs);
