@@ -2,7 +2,7 @@
 /* pyro --- fireworks */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)pyro.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)pyro.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -59,7 +59,16 @@ static const char sccsid[] = "@(#)pyro.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt pyro_opts =
 {0, NULL, 0, NULL, NULL};
 
-#define ORANGE 3
+#ifdef USE_MODULES
+ModStruct   pyro_description =
+{"pyro", "init_pyro", "draw_pyro", "release_pyro",
+ "refresh_pyro", "init_pyro", NULL, &pyro_opts,
+ 15000, 100, 1, -3, 64, 1.0, "",
+ "Shows fireworks", 0, NULL};
+
+#endif
+
+#define ORANGE (MI_NPIXELS(mi) / 16)
 
 #define MINROCKETS 1
 #define MINSIZE 1
@@ -70,11 +79,13 @@ ModeSpecOpt pyro_opts =
 
 #define CLOUD 0
 #define DOUBLECLOUD 1
+#define COLORCLOUD 2
 /* Clearly other types and other fascinating visual effects could be added... */
 
 /* P_xxx parameters represent the reciprocal of the probability... */
 #define P_IGNITE 5000		/* ...of ignition per cycle */
 #define P_DOUBLECLOUD 10	/* ...of an ignition being double */
+#define P_COLORCLOUD 5		/* ...of an ignition being full color */
 #define P_MULTI 75		/* ...of an ignition being several @ once */
 #define P_FUSILLADE 250		/* ...of an ignition starting a fusillade */
 
@@ -100,7 +111,7 @@ ModeSpecOpt pyro_opts =
 
 #define SVELFACTOR 0.1		/* Max star velocity / yvel */
 #define BOUYANCY 0.2		/* Reduction in grav deceleration for stars */
-#define MAXSTARS 75		/* Number of stars issued from a shell */
+#define MAXSTARS 150		/* Number of stars issued from a shell */
 #define MINSTARS 50
 #define MINSFUSE 50		/* Range of fuse lengths for stars */
 #define MAXSFUSE 100
@@ -112,6 +123,7 @@ ModeSpecOpt pyro_opts =
 #define TWOPI (2.0*M_PI)
 
 typedef struct {
+	unsigned long color;
 	float       sx, sy, sz;	/* Distance from notional center  */
 	float       sxvel, syvel, szvel;	/* Relative to notional center */
 } star;
@@ -145,8 +157,6 @@ typedef struct {
 	rocket     *rockq;
 } pyrostruct;
 
-static void ignite(ModeInfo * mi, pyrostruct * pp);
-static void animate(ModeInfo * mi, pyrostruct * pp, rocket * rp);
 static void shootup(ModeInfo * mi, pyrostruct * pp, rocket * rp);
 static void burst(ModeInfo * mi, pyrostruct * pp, rocket * rp);
 
@@ -177,9 +187,10 @@ ignite(ModeInfo * mi, pyrostruct * pp)
 	if (MI_NPIXELS(mi) > 2) {
 		pix = NRAND(MI_NPIXELS(mi));
 		color[0] = MI_PIXEL(mi, pix);
-		color[1] = MI_PIXEL(mi, ((pix + (MI_NPIXELS(mi) / 2)) % MI_NPIXELS(mi)));
+		color[1] = MI_PIXEL(mi,
+			    ((pix + (MI_NPIXELS(mi) / 2)) % MI_NPIXELS(mi)));
 	} else {
-		color[0] = color[1] = MI_WIN_WHITE_PIXEL(mi);
+		color[0] = color[1] = MI_WHITE_PIXEL(mi);
 	}
 
 	multi = 1;
@@ -190,6 +201,8 @@ ignite(ModeInfo * mi, pyrostruct * pp)
 		if (NRAND(P_MULTI) == 0)
 			multi = INTRAND(5, 15);
 	}
+	if (NRAND(P_COLORCLOUD) == 0)
+		shelltype |= COLORCLOUD;
 
 	rp = pp->rockq;
 	while (multi--) {
@@ -235,7 +248,7 @@ animate(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 					rp->Xpointsleft[0][starn].x = (int) rp->x - diff;
 					rp->Xpoints[0][starn].y =
 						rp->Xpointsleft[0][starn].y = (int) rp->y;
-					if (rp->shelltype == DOUBLECLOUD) {
+					if (rp->shelltype & DOUBLECLOUD) {
 						rp->Xpoints[1][starn].x = (int) rp->x + diff;
 						rp->Xpointsleft[1][starn].x = (int) rp->x - diff;
 						rp->Xpoints[1][starn].y =
@@ -252,7 +265,13 @@ animate(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 				} else {
 					rp->Xpoints[0][starn].x = (int) rp->x;
 					rp->Xpoints[0][starn].y = (int) rp->y;
-					if (rp->shelltype == DOUBLECLOUD) {
+					if ((rp->shelltype & COLORCLOUD) && (MI_NPIXELS(mi) > 2)) {
+						if (NRAND(6) < 1)
+							rp->stars[starn].color = MI_WHITE_PIXEL(mi);
+						else
+							rp->stars[starn].color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+					}
+					if (rp->shelltype & DOUBLECLOUD) {
 						rp->Xpoints[1][starn].x = (int) rp->x;
 						rp->Xpoints[1][starn].y = (int) rp->y;
 					}
@@ -282,7 +301,7 @@ shootup(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi))
 		XSetForeground(display, gc, MI_NONE_COLOR(mi));
 	else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	if (MI_WIN_IS_USE3D(mi)) {
 		diff = (int) GETZDIFF(rp->z);
 		XFillRectangle(display, window, gc, (int) (rp->x) + diff, (int) (rp->y),
@@ -338,14 +357,14 @@ burst(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi))
 		XSetForeground(display, gc, MI_NONE_COLOR(mi));
 	else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 	XFillRectangles(display, window, gc, rp->Xpoints[0], nstars);
-	if (stype == DOUBLECLOUD)
+	if (stype & DOUBLECLOUD)
 		XFillRectangles(display, window, gc, rp->Xpoints[1], nstars);
 	if (MI_WIN_IS_USE3D(mi)) {
 		XFillRectangles(display, window, gc, rp->Xpointsleft[0], nstars);
-		if (stype == DOUBLECLOUD)
+		if (stype & DOUBLECLOUD)
 			XFillRectangles(display, window, gc, rp->Xpointsleft[1], nstars);
 	}
 	if (rp->fuse-- <= 0) {
@@ -377,7 +396,7 @@ burst(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 			rp->Xpointsleft[0][starn].x = (int) (rx + sx - diff);
 			rp->Xpoints[0][starn].y =
 				rp->Xpointsleft[0][starn].y = (int) (ry + sy);
-			if (stype == DOUBLECLOUD) {
+			if (stype & DOUBLECLOUD) {
 				rp->Xpoints[1][starn].x = (int) (rx + 1.7 * sx) + diff;
 				rp->Xpointsleft[1][starn].x = (int) (rx + 1.7 * sx) - diff;
 				rp->Xpoints[1][starn].y =
@@ -387,7 +406,7 @@ burst(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 
 			rp->Xpoints[0][starn].x = (int) (rx + sx);
 			rp->Xpoints[0][starn].y = (int) (ry + sy);
-			if (stype == DOUBLECLOUD) {
+			if (stype & DOUBLECLOUD) {
 				rp->Xpoints[1][starn].x = (int) (rx + 1.7 * sx);
 				rp->Xpoints[1][starn].y = (int) (ry + 1.7 * sy);
 			}
@@ -405,21 +424,30 @@ burst(ModeInfo * mi, pyrostruct * pp, rocket * rp)
 			XSetFunction(display, gc, GXor);
 		XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
 		XFillRectangles(display, window, gc, rp->Xpoints[0], nstars);
-		if (stype == DOUBLECLOUD) {
+		if (stype & DOUBLECLOUD) {
 			XFillRectangles(display, window, gc, rp->Xpoints[1], nstars);
 		}
 		XSetForeground(display, gc, MI_LEFT_COLOR(mi));
 		XFillRectangles(display, window, gc, rp->Xpointsleft[0], nstars);
-		if (stype == DOUBLECLOUD) {
+		if (stype & DOUBLECLOUD) {
 			XFillRectangles(display, window, gc, rp->Xpointsleft[1], nstars);
 		}
 		if (MI_WIN_IS_INSTALL(mi))
 			XSetFunction(display, gc, GXcopy);
 	} else {
 
-		XSetForeground(display, gc, rp->color[0]);
-		XFillRectangles(display, window, gc, rp->Xpoints[0], nstars);
-		if (stype == DOUBLECLOUD) {
+		if ((stype & COLORCLOUD) && (MI_NPIXELS(mi) > 2)) {
+			for (starn = 0; starn < nstars; starn++) {
+				XSetForeground(display, gc, rp->stars[starn].color);
+				XFillRectangle(display, window, gc,
+					       rp->Xpoints[0][starn].x, rp->Xpoints[0][starn].y,
+					       rp->Xpoints[0][starn].width, rp->Xpoints[0][starn].height);
+			}
+		} else {
+			XSetForeground(display, gc, rp->color[0]);
+			XFillRectangles(display, window, gc, rp->Xpoints[0], nstars);
+		}
+		if (stype & DOUBLECLOUD) {
 			XSetForeground(display, gc, rp->color[1]);
 			XFillRectangles(display, window, gc, rp->Xpoints[1], nstars);
 		}
@@ -492,7 +520,7 @@ init_pyro(ModeInfo * mi)
 	if (MI_NPIXELS(mi) > 3)
 		pp->rockpixel = MI_PIXEL(mi, ORANGE);
 	else
-		pp->rockpixel = MI_WIN_WHITE_PIXEL(mi);
+		pp->rockpixel = MI_WHITE_PIXEL(mi);
 
 /* Geometry-dependent physical data: */
 	pp->maxvelx = (float) (pp->width) * XVELFACTOR;
@@ -507,11 +535,10 @@ init_pyro(ModeInfo * mi)
 	pp->rockdecel = (float) (pp->height) * GRAVFACTOR;
 	pp->stardecel = pp->rockdecel * BOUYANCY;
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi)) {
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
-		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			       0, 0, pp->width, pp->height);
-	} else
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+		MI_CLEARWINDOWCOLOR(mi, MI_NONE_COLOR(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
+	}
 }
 
 /* ARGSUSED */
