@@ -157,7 +157,7 @@ static char nice_modes[][MAXMODECHARS] =
 	"daisy" "dclock", "demon" "deco",
 	"eyes",
 	"fadeplot", "flag", "flame",
-	"geometry", "grav",
+	"grav",
 	"helix", "hyper",
 	"ico", "image",
 	"kaleid",
@@ -210,7 +210,7 @@ static char fractal_modes[][MAXMODECHARS] =
 
 static char geometry_modes[][MAXMODECHARS] =
 {
-	"braid", "fadeplot", "geometry", "helix", "hyper", "ico", "kaleid",
+	"braid", "fadeplot", "helix", "hyper", "ico", "kaleid",
 	"laser", "lisa", "lissie", "penrose", "petal", "qix",
 	"shape", "sphere", "spiral", "spline"
 };
@@ -268,8 +268,7 @@ pickMode(void)
 	} else {
 		/* pick a random slot in the list, check for last */
 		if (sequential) {
-			i = (last_index + 1) % mode_count;
-			last_index = i;
+			last_index = i = (last_index + 1) % nmodes;
 		} else
 			while (mode_indexes[i = NRAND(mode_count)] == last_mode);
 	}
@@ -347,13 +346,12 @@ strpmtok(int *sign, char *str)
 static void
 parsemodelist(ModeInfo * mi)
 {
-	int         i, sign = 1;
-	unsigned int j;
+	int         i, sign = 1, j;
 	char       *p;
 
 	modes = (int *) calloc(numprocs - 1, sizeof (int));
 
-	p = strpmtok(&sign, modelist);
+	p = strpmtok(&sign, (modelist) ? modelist : "");
 
 	while (p) {
 		if (!strcmp(p, "all")) {
@@ -494,7 +492,7 @@ init_random(ModeInfo * mi)
 	}
 	if (rp->fix) {
 		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi), MI_NCOLORS(mi),
-			    MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
 			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
 		rp->fix = False;
 	}
@@ -526,7 +524,7 @@ draw_random(ModeInfo * mi)
 	}
 	if (rp->fix) {
 		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi), MI_NCOLORS(mi),
-			    MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
 			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
 		rp->fix = False;
 	}
