@@ -2,7 +2,7 @@
 /* slip --- lots of slipping blits */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)slip.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)slip.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -42,6 +42,15 @@ static const char sccsid[] = "@(#)slip.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt slip_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   slip_description =
+{"slip", "init_slip", "draw_slip", "release_slip",
+ "init_slip", "init_slip", NULL, &slip_opts,
+ 50000, 35, 50, 1, 64, 1.0, "",
+ "Shows slipping blits", 0, NULL};
+
+#endif
+
 typedef struct {
 	int         width, height;
 	int         nblits_remaining;
@@ -109,7 +118,7 @@ prepare_screen(ModeInfo * mi, slipstruct * s)
 	s->backwards = LRAND() & 1;	/* jwz: go the other way sometimes */
 
 	if (s->first_time || !halfrandom(5)) {
-		XClearWindow(display, MI_WINDOW(mi));
+		MI_CLEARWINDOW(mi);
 		n = 300;
 	} else {
 		if (halfrandom(5))
@@ -123,18 +132,19 @@ prepare_screen(ModeInfo * mi, slipstruct * s)
 	if (MI_NPIXELS(mi) > 2)
 		XSetForeground(display, gc, MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
 	else if (halfrandom(2))
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 	for (i = 0; i < n; i++) {
-		if (not_solid)
+		if (not_solid) {
 			if (MI_NPIXELS(mi) > 2)
 				XSetForeground(display, gc, MI_PIXEL(mi, halfrandom(MI_NPIXELS(mi))));
 			else if (halfrandom(2))
-				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+				XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 			else
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+		}
 		XFillRectangle(display, MI_WINDOW(mi), gc,
 			       halfrandom(s->width - w),
 			       halfrandom(s->height - w),
@@ -196,7 +206,7 @@ draw_slip(ModeInfo * mi)
 		double      x, y, dx = 0, dy = 0, t, s1, s2;
 
 		if (0 == s->nblits_remaining--) {
-			static      lut[] =
+			static int  lut[] =
 			{0, 0, 0, 1, 1, 1, 2};
 
 			prepare_screen(mi, s);
