@@ -2,7 +2,7 @@
 /* spline --- spline fun */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)spline.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)spline.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -86,7 +86,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt spline_opts =
-{2, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   spline_description =
+{"spline", "init_spline", "draw_spline", "release_spline",
+ "refresh_spline", "init_spline", NULL, &spline_opts,
+ 30000, -6, 2048, 1, 64, 0.4, "",
+ "Shows colorful moving splines", 0, NULL};
+
+#endif
 
 #define MINPOINTS 3
 
@@ -134,9 +143,10 @@ init_spline(ModeInfo * mi)
 	}
 	sp = &splines[MI_SCREEN(mi)];
 
-	sp->erase = erase;
 	if (MI_WIN_IS_FULLRANDOM(mi))
 		sp->erase = (Bool) (LRAND() & 1);
+	else
+		sp->erase = erase;
 
 	sp->width = MI_WIN_WIDTH(mi);
 	sp->height = MI_WIN_HEIGHT(mi);
@@ -177,7 +187,7 @@ init_spline(ModeInfo * mi)
 		sp->nsplines = 0;
 	}
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	/* Initialize points. */
 	for (i = 0; i < sp->points; ++i) {
@@ -229,7 +239,7 @@ draw_spline(ModeInfo * mi)
 	}
 
 	if (sp->erase) {
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 		/* Erase first figure. */
 		px = zx = (sp->splineq[sp->first][0].x +
@@ -257,7 +267,7 @@ draw_spline(ModeInfo * mi)
 		if (++sp->color >= MI_NPIXELS(mi))
 			sp->color = 0;
 	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
 	/* Draw the figure. */
 	px = zx = (sp->pt[0].pos.x + sp->pt[sp->points - 1].pos.x) / 2;
@@ -407,6 +417,7 @@ refresh_spline(ModeInfo * mi)
 	if (sp->erase) {
 		sp->redrawing = 1;
 		sp->redrawpos = 1;
-	} else
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
+	}
 }
