@@ -2,7 +2,7 @@
 /* star --- flying through an asteroid field */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)star.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)star.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -72,7 +72,7 @@ static const char sccsid[] = "@(#)star.c	4.04 97/07/28 xlockmore";
 
 #endif /* STANDALONE */
 
-#define DEF_TREK  "0"
+#define DEF_TREK  "50"
 #define DEF_ROCK  "False"
 #define DEF_STRAIGHT  "False"
 
@@ -104,7 +104,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt star_opts =
-{5, opts, 3, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   star_description =
+{"star", "init_star", "draw_star", "release_star",
+ "refresh_star", "init_star", NULL, &star_opts,
+ 40000, 100, 1, 100, 64, 0.2, "",
+ "Shows a star field with a twist", 0, NULL};
+
+#endif
 
 typedef struct _BitmapType {
 	int         direction, width, height;
@@ -178,8 +187,14 @@ typedef struct {
 } trekstruct;
 
 typedef struct {
+	int         current_delta;	/* observer Z rotation */
+	int         new_delta;
+	int         dchange_tick;
 	int         width, height;
 	int         midx, midy;
+	int         current_dep[2];
+	int         speed_dep[2];
+	short       direction[2];
 	int         rotate_p, speed, nstars;
 	float       max_dep;
 	int         move_p;
@@ -195,13 +210,11 @@ typedef struct {
 } starstruct;
 
 static float cos_array[RESOLUTION], sin_array[RESOLUTION];
-static double depths[(MAX_DEPTH + 1) * DEPTH_SCALE];
+static float depths[(MAX_DEPTH + 1) * DEPTH_SCALE];
 
 static starstruct *stars = NULL;
 
 static void star_draw(ModeInfo * mi, astar * astars, int draw_p);
-static void init_pixmaps(ModeInfo * mi);
-static void tick_stars(ModeInfo * mi, int d);
 static int  compute_move(starstruct * sp, int axe);
 
 static void
@@ -237,7 +250,7 @@ star_reset(ModeInfo * mi, astar * astars)
 	if (MI_NPIXELS(mi) > 2)
 		astars->color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	else
-		astars->color = MI_WIN_WHITE_PIXEL(mi);
+		astars->color = MI_WHITE_PIXEL(mi);
 	star_compute(mi, astars);
 	star_draw(mi, astars, True);
 }
@@ -328,12 +341,13 @@ draw_trek(ModeInfo * mi)
 		    (sp->trek.loc.y > sp->height) ||
 		    (sp->trek.loc.x > sp->width)) {
 			sp->current_trek = TREKIES;
-			XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 			XFillRectangle(display, window, gc,
-				       sp->trek.loc.x - sp->trek.delta.x, sp->trek.loc.y - sp->trek.delta.y,
+				       sp->trek.loc.x - sp->trek.delta.x,
+				       sp->trek.loc.y - sp->trek.delta.y,
 				       sp->trek.size.x, sp->trek.size.y);
 		} else {
-			XSetForeground(display, sp->stippledGC, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, sp->stippledGC, MI_WHITE_PIXEL(mi));
 			XSetTSOrigin(display, sp->stippledGC, sp->trek.loc.x, sp->trek.loc.y);
 			XSetStipple(display, sp->stippledGC, sp->trekPixmaps[sp->current_trek]);
 			XSetFillStyle(display, sp->stippledGC, FillOpaqueStippled);
@@ -342,7 +356,7 @@ draw_trek(ModeInfo * mi)
 				       sp->trek.size.x, sp->trek.size.y);
 
 			if (!new_one) {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				ERASE_IMAGE(display, window, gc,
 					    sp->trek.loc.x, sp->trek.loc.y,
 					 (sp->trek.loc.x - sp->trek.delta.x),
@@ -365,16 +379,16 @@ star_draw(ModeInfo * mi, astar * astars, int draw_p)
 	if (draw_p) {
 		if (MI_WIN_IS_USE3D(mi)) {
 			if (MI_WIN_IS_INSTALL(mi))
-				XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
+				XSetForeground(MI_DISPLAY(mi), gc, MI_NONE_COLOR(mi));
 			else
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		} else
 			XSetForeground(display, gc, astars->color);
 	} else
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	if (astars->x <= 0 || astars->y <= 0 ||
 	    astars->x >= sp->width || astars->y >= sp->height) {
-		/* this means that if a star were to go off the screen at 12:00, but
+		/* This means that if a star were to go off the screen at 12:00, but
 		   would have been visible at 3:00, it won't come back once the observer
 		   rotates around so that the star would have been visible again.
 		   Oh well.
@@ -406,12 +420,14 @@ star_draw(ModeInfo * mi, astar * astars, int draw_p)
 				XSetForeground(display, gc, MI_LEFT_COLOR(mi));
 			}
 			XFillRectangle(display, window, gc,
-				       astars->x - astars->size / 2 - astars->diff, astars->y - astars->size / 2,
+				 astars->x - astars->size / 2 - astars->diff,
+				       astars->y - astars->size / 2,
 				       astars->size, astars->size);
 			if (draw_p)
 				XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
 			XFillRectangle(display, window, gc,
-				       astars->x - astars->size / 2 + astars->diff, astars->y - astars->size / 2,
+				 astars->x - astars->size / 2 + astars->diff,
+				       astars->y - astars->size / 2,
 				       astars->size, astars->size);
 			if (draw_p && MI_WIN_IS_INSTALL(mi))
 				XSetFunction(display, gc, GXcopy);
@@ -426,12 +442,14 @@ star_draw(ModeInfo * mi, astar * astars, int draw_p)
 			XSetForeground(display, gc, MI_LEFT_COLOR(mi));
 			XCopyPlane(display, sp->pixmaps[astars->size - MINSIZE], window, gc,
 				   0, 0, astars->size, astars->size,
-				   astars->x - astars->size / 2 - astars->diff, astars->y - astars->size / 2,
+				 astars->x - astars->size / 2 - astars->diff,
+				   astars->y - astars->size / 2,
 				   1L);
 			XSetForeground(display, gc, MI_RIGHT_COLOR(mi));
 			XCopyPlane(display, sp->pixmaps[astars->size - MINSIZE], window, gc,
 				   0, 0, astars->size, astars->size,
-				   astars->x - astars->size / 2 + astars->diff, astars->y - astars->size / 2,
+				 astars->x - astars->size / 2 + astars->diff,
+				   astars->y - astars->size / 2,
 				   1L);
 			if (MI_WIN_IS_INSTALL(mi))
 				XSetFunction(display, gc, GXcopy);
@@ -504,14 +522,15 @@ init_pixmaps(ModeInfo * mi)
 			points[6].x = (int) ((double) h * 0.45);
 			points[6].y = (int) ((double) h * 1.00);
 			XFillPolygon(display, p, fg_gc, points, 7, Nonconvex, CoordModeOrigin);
-		} else
+		} else {
 			XFillArc(display, p, fg_gc, 0, 0, h, h, 0, 23040);
+		}
 	}
 	XFreeGC(display, fg_gc);
 	XFreeGC(display, bg_gc);
 	if (!sp->stippledGC) {
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((sp->stippledGC = XCreateGC(display, window,
 				 GCForeground | GCBackground, &gcv)) == None)
 			return;
@@ -545,52 +564,45 @@ static int
 compute_move(starstruct * sp, int axe)
 				/* 0 for x, 1 for y */
 {
-	static int  current_dep[2] =
-	{0, 0};
-	static int  speed[2] =
-	{0, 0};
-	static short direction[2] =
-	{0, 0};
-	static int  limit[2] =
-	{0, 0};
+	int         limit[2];
 	int         change = 0;
 
 	limit[0] = sp->midx;
 	limit[1] = sp->midy;
 
-	current_dep[axe] += speed[axe];		/* We adjust the displacement */
+	sp->current_dep[axe] += sp->speed_dep[axe];	/* We adjust the displacement */
 
-	if (current_dep[axe] > (int) (limit[axe] * sp->max_dep)) {
-		if (current_dep[axe] > limit[axe])
-			current_dep[axe] = limit[axe];
-		direction[axe] = -1;
+	if (sp->current_dep[axe] > (int) (limit[axe] * sp->max_dep)) {
+		if (sp->current_dep[axe] > limit[axe])
+			sp->current_dep[axe] = limit[axe];
+		sp->direction[axe] = -1;
 	}			/* This is when we reach the upper screen limit */
-	if (current_dep[axe] < (int) (-limit[axe] * sp->max_dep)) {
-		if (current_dep[axe] < -limit[axe])
-			current_dep[axe] = -limit[axe];
-		direction[axe] = 1;
+	if (sp->current_dep[axe] < (int) (-limit[axe] * sp->max_dep)) {
+		if (sp->current_dep[axe] < -limit[axe])
+			sp->current_dep[axe] = -limit[axe];
+		sp->direction[axe] = 1;
 	}			/* This is when we reach the lower screen limit */
-	if (direction[axe] == 1)	/* We adjust the speed */
-		speed[axe] += 1;
-	else if (direction[axe] == -1)
-		speed[axe] -= 1;
+	if (sp->direction[axe] == 1)	/* We adjust the sp->speed_dep */
+		sp->speed_dep[axe] += 1;
+	else if (sp->direction[axe] == -1)
+		sp->speed_dep[axe] -= 1;
 
-	if (speed[axe] > MAX_DEP_SPEED)
-		speed[axe] = MAX_DEP_SPEED;
-	else if (speed[axe] < -MAX_DEP_SPEED)
-		speed[axe] = -MAX_DEP_SPEED;
+	if (sp->speed_dep[axe] > MAX_DEP_SPEED)
+		sp->speed_dep[axe] = MAX_DEP_SPEED;
+	else if (sp->speed_dep[axe] < -MAX_DEP_SPEED)
+		sp->speed_dep[axe] = -MAX_DEP_SPEED;
 
 	if (!straight && !NRAND(DIRECTION_CHANGE_RATE)) {
 		/* We change direction */
 		change = LRAND() & 1;
 		if (change != 1) {
-			if (direction[axe] == 0)
-				direction[axe] = change - 1;	/* 0 becomes either 1 or -1 */
+			if (sp->direction[axe] == 0)
+				sp->direction[axe] = change - 1;	/* 0 becomes either 1 or -1 */
 			else
-				direction[axe] = 0;	/* -1 or 1 become 0 */
+				sp->direction[axe] = 0;		/* -1 or 1 become 0 */
 		}
 	}
-	return (current_dep[axe]);
+	return (sp->current_dep[axe]);
 }
 
 void
@@ -638,9 +650,9 @@ init_star(ModeInfo * mi)
 			sin_array[i] = SINF((((float) i) / (RESOLUTION / 2.0)) * M_PI);
 			cos_array[i] = COSF((((float) i) / (RESOLUTION / 2.0)) * M_PI);
 		}
-		/* we actually only need i/speed of these, but wtf */
+		/* We actually only need i/speed of these. Oh well. */
 		for (i = 1; i < (int) (sizeof (depths) / sizeof (depths[0])); i++)
-			depths[i] = atan(((double) 0.5) / (((double) i) / DEPTH_SCALE));
+			depths[i] = (float) atan(((double) 0.5) / (((double) i) / DEPTH_SCALE));
 		depths[0] = M_PI_2;	/* avoid division by 0 */
 	}
 	if (!sp->astars)
@@ -654,36 +666,32 @@ init_star(ModeInfo * mi)
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
 		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
 			       0, 0, sp->width, sp->height);
-	} else
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
+	}
 }
 
 void
 draw_star(ModeInfo * mi)
 {
-	static int  current_delta = 0;	/* observer Z rotation */
-	static int  window_tick = 50;
-	static int  new_delta = 0;
-	static int  dchange_tick = 0;
+	starstruct *sp = &stars[MI_SCREEN(mi)];
 
-	if (window_tick++ == 50)
-		window_tick = 0;
-	if (current_delta != new_delta) {
-		if (dchange_tick++ == 5) {
-			dchange_tick = 0;
-			if (current_delta < new_delta)
-				current_delta++;
+	if (sp->current_delta != sp->new_delta) {
+		if (sp->dchange_tick++ == 5) {
+			sp->dchange_tick = 0;
+			if (sp->current_delta < sp->new_delta)
+				sp->current_delta++;
 			else
-				current_delta--;
+				sp->current_delta--;
 		}
 	} else {
 		if (!NRAND(50)) {
-			new_delta = (NRAND(11) - 5);
+			sp->new_delta = (NRAND(11) - 5);
 			if (!NRAND(10))
-				new_delta *= 5;
+				sp->new_delta *= 5;
 		}
 	}
-	tick_stars(mi, current_delta);
+	tick_stars(mi, sp->current_delta);
 	draw_trek(mi);
 }
 
