@@ -2,7 +2,7 @@
 /* vines --- vine fractals */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)vines.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)vines.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -60,6 +60,15 @@ static const char sccsid[] = "@(#)vines.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt vines_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   vines_description =
+{"vines", "init_vines", "draw_vines", "release_vines",
+ "refresh_vines", "init_vines", NULL, &vines_opts,
+ 200000, 0, 1, 1, 64, 1.0, "",
+ "Shows fractals", 0, NULL};
+
+#endif
+
 typedef struct {
 	int         a;
 	int         x1;
@@ -85,11 +94,11 @@ refresh_vines(ModeInfo * mi)
 void
 init_vines(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
 	vinestruct *fp;
 
 	if (vines == NULL) {
-		if ((vines = (vinestruct *) calloc(MI_NUM_SCREENS(mi), sizeof (vinestruct))) == NULL) {
+		if ((vines = (vinestruct *) calloc(MI_NUM_SCREENS(mi),
+					     sizeof (vinestruct))) == NULL) {
 			return;
 		}
 	}
@@ -99,7 +108,7 @@ init_vines(ModeInfo * mi)
 	fp->length = 0;
 	fp->iterations = 30 + NRAND(100);
 
-	XClearWindow(display, MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }				/* init_vines */
 
 void
@@ -131,7 +140,7 @@ draw_vines(ModeInfo * mi)
 		if (MI_NPIXELS(mi) > 2)
 			XSetForeground(display, gc, MI_PIXEL(mi, NRAND(MI_NPIXELS(mi))));
 		else
-			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	}
 	count = fp->i + MI_BATCHCOUNT(mi);
 	if ((count <= fp->i) || (count > fp->length))
