@@ -2,7 +2,7 @@
 /* world --- world spinner */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)world.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)world.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -46,6 +46,15 @@ static const char sccsid[] = "@(#)world.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt world_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   world_description =
+{"world", "init_world", "draw_world", "release_world",
+ "refresh_world", "init_world", NULL, &world_opts,
+ 100000, -16, 1, 1, 64, 0.3, "",
+ "Shows spinning Earths", 0, NULL};
+
+#endif
+
 #include "bitmaps/terra-00.xbm"
 #include "bitmaps/terra-01.xbm"
 #include "bitmaps/terra-02.xbm"
@@ -181,7 +190,7 @@ init_world(ModeInfo * mi)
 	for (i = 0; i < wp->nplanets; i++)
 		wp->planets[i].x = wp->planets[i].y = -1;
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -201,7 +210,7 @@ draw_world(ModeInfo * mi)
 #endif
 	if (wp->frame_num == NUM_EARTHS * NUM_REV) {
 		wp->frame_num = 0;
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		for (i = 0; i < wp->nplanets; i++) {
 			if ((wp->ncols > wp->nplanets || wp->nrows < 2) &&
 			    wp->planets[i].x >= 0 && wp->planets[i].y >= 0)
@@ -225,7 +234,7 @@ draw_world(ModeInfo * mi)
 			if (MI_NPIXELS(mi) > 2)
 				wp->planets[i].color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 			else
-				wp->planets[i].color = MI_WIN_WHITE_PIXEL(mi);
+				wp->planets[i].color = MI_WHITE_PIXEL(mi);
 		}
 	}
 #ifndef NOFLASH
@@ -239,11 +248,11 @@ draw_world(ModeInfo * mi)
 			if ((wp->planets[i].frame < 0) && wp->planets[i].direction == 0)
 				wp->planets[i].frame = NUM_EARTHS - 1;
 		}
-		XPutImage(display, MI_WINDOW(mi), gc, (Earths + wp->planets[i].frame),
-			  0, 0,
-			  wp->xb + SIZE_X * wp->planets[i].x,
-			  wp->yb + SIZE_Y * wp->planets[i].y,
-			  SIZE_X, SIZE_Y);
+		(void) XPutImage(display, MI_WINDOW(mi), gc,
+				 (Earths + wp->planets[i].frame), 0, 0,
+				 wp->xb + SIZE_X * wp->planets[i].x,
+				 wp->yb + SIZE_Y * wp->planets[i].y,
+				 SIZE_X, SIZE_Y);
 
 		(wp->planets[i].direction) ? wp->planets[i].frame++ : wp->planets[i].frame--;
 	}
