@@ -135,18 +135,17 @@ worm_doit(ModeInfo * mi, int which, unsigned long color)
 		diff = ws->diffcirc[ws->tail];
 		if (MI_WIN_IS_INSTALL(mi)) {
 			XSetForeground(display, gc, MI_NONE_COLOR(mi));
-			XFillRectangle(display, window, gc, x - diff, y,
-				       wp->circsize, wp->circsize);
-			XFillRectangle(display, window, gc, x + diff, y,
-				       wp->circsize, wp->circsize);
 		} else {
-			XClearArea(display, window, x - diff, y,
-				   wp->circsize, wp->circsize, False);
-			XClearArea(display, window, x + diff, y,
-				   wp->circsize, wp->circsize, False);
+			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		}
-	} else
-		XClearArea(display, window, x, y, wp->circsize, wp->circsize, False);
+		XFillRectangle(display, window, gc, x - diff, y,
+			       wp->circsize, wp->circsize);
+		XFillRectangle(display, window, gc, x + diff, y,
+			       wp->circsize, wp->circsize);
+	} else {
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+		XFillRectangle(display, window, gc, x, y, wp->circsize, wp->circsize);
+	}
 
 	if (LRAND() & 1)
 		ws->dir = (ws->dir + 1) % SEGMENTS;
