@@ -74,7 +74,7 @@ static char namegeom[1024];
 static Widget ListMode;
 static Widget Drawxlock;	/* the widget where is xlock show */
 
-static int  numberprocess = -1;	/* PID of xlock process */
+static pid_t numberprocess = -1;	/* PID of xlock process */
 
 #include "modes.h"
 
@@ -294,18 +294,19 @@ execxlock(Widget w, char *str, int index, void *data)
 		(void) wait(&n);
 	}
 #ifdef VMS
-	if ((numberprocess = vfork()) == -1)
-		/* This may also be a goodie for older Unix systems. J.J. */
+#define FORK vfork
 #else
-	if ((numberprocess = fork()) == -1)
+#define FORK fork
 #endif
+	if ((numberprocess = FORK()) == -1)
 		(void) fprintf(stderr, "Fork error\n");
 	else if (numberprocess == 0) {
 		(void) execlp(XLOCK, XLOCK, "-parent", numberwidget,
-		    "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
-		       "-nolock", "-inwindow", 0);
+			      "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
+			      "-nolock", "-inwindow", 0);
 	}
 }
+
 /* This function sets up the display.  For any kind of a real program, 
  * you'll probably want to save the values returned by the MakeXXX calls
  * so that you have a way to refer to the display objects you have 
@@ -379,8 +380,8 @@ main(int argc, char **argv)
 
 	MainLoop();		/* go right into the main loop */
 #ifdef VMS
-  return 1;
+	return 1;
 #else
-  return 0;
+	return 0;
 #endif
 }
