@@ -123,21 +123,21 @@ allocPixel(Display * display, Colormap cmap, char *name, char *def)
 }
 
 void
-monoColormap(Screen *scr, ScreenInfo *si, char *foreground, char *background)
+monoColormap(Screen * scr, ScreenInfo * si, char *foreground, char *background)
 {
-		si->black_pixel = BlackPixelOfScreen(scr);
-		si->white_pixel = WhitePixelOfScreen(scr);
-    if (strcmp(foreground, "White") == 0 || strcmp(foreground, "white") == 0 ||
-        strcmp(background, "Black") == 0 || strcmp(background, "black") == 0) {
-		  si->fg_pixel = WhitePixelOfScreen(scr);
-		  si->bg_pixel = BlackPixelOfScreen(scr);
-     } else {   
+	si->black_pixel = BlackPixelOfScreen(scr);
+	si->white_pixel = WhitePixelOfScreen(scr);
+	if (strcmp(foreground, "White") == 0 || strcmp(foreground, "white") == 0 ||
+	    strcmp(background, "Black") == 0 || strcmp(background, "black") == 0) {
+		si->fg_pixel = WhitePixelOfScreen(scr);
+		si->bg_pixel = BlackPixelOfScreen(scr);
+	} else {
 		si->fg_pixel = BlackPixelOfScreen(scr);
 		si->bg_pixel = WhitePixelOfScreen(scr);
-    }
-		si->pixels[0] = WhitePixelOfScreen(scr);
-		si->pixels[1] = BlackPixelOfScreen(scr);
-		si->npixels = 2;
+	}
+	si->pixels[0] = WhitePixelOfScreen(scr);
+	si->pixels[1] = BlackPixelOfScreen(scr);
+	si->npixels = 2;
 }
 
 void
@@ -150,17 +150,18 @@ fixColormap(Display * display, Window window,
 	Colormap    cmap = Scr[screen].colormap;
 	XColor      xcolor;
 	unsigned char *red, *green, *blue;
-	int         colorcount, i;
+	int         colorcount, i, fixed, visualclass;
 	extern char *foreground;
 	extern char *background;
 
 
 	if (mono || CellsOfScreen(scr) <= 2) {
 		if (Scr[screen].pixels)
-      return;
+			return;
 		Scr[screen].pixels =
 			(unsigned long *) calloc(2, sizeof (unsigned long));
-    monoColormap(scr, &(Scr[screen]), foreground, background);
+
+		monoColormap(scr, &(Scr[screen]), foreground, background);
 		return;
 	}
 	colorcount = ncolors;
@@ -168,7 +169,13 @@ fixColormap(Display * display, Window window,
 	green = (unsigned char *) calloc(ncolors, sizeof (unsigned char));
 	blue = (unsigned char *) calloc(ncolors, sizeof (unsigned char));
 
-	if (cmap != None && cmap != DefaultColormapOfScreen(scr) && Scr[screen].pixels) {
+  visualclass = Scr[screen].visual->CLASS;
+  fixed = (visualclass == StaticGray) || (visualclass == StaticColor) ||
+      (visualclass == TrueColor);
+	if ((!fixed && (!install || inroot)) || cmap == None) {
+		cmap = DefaultColormapOfScreen(scr);
+	}
+	if (cmap != DefaultColormapOfScreen(scr) && Scr[screen].pixels) {
 		XFreeColors(display, cmap, Scr[screen].pixels, Scr[screen].npixels, 0);
 		XFreeColors(display, cmap, &(Scr[screen].black_pixel), 1, 0);
 		XFreeColors(display, cmap, &(Scr[screen].white_pixel), 1, 0);
@@ -224,7 +231,7 @@ fixColormap(Display * display, Window window,
 		XFreeColors(display, cmap, &(Scr[screen].white_pixel), 1, 0);
 		XFreeColors(display, cmap, &(Scr[screen].bg_pixel), 1, 0);
 		XFreeColors(display, cmap, &(Scr[screen].fg_pixel), 1, 0);
-    monoColormap(scr, &(Scr[screen]), foreground, background);
+		monoColormap(scr, &(Scr[screen]), foreground, background);
 		Scr[screen].colormap = cmap = DefaultColormapOfScreen(scr);
 		return;
 	}
@@ -235,12 +242,15 @@ fixColormap(Display * display, Window window,
 		if (cmap != xgwa.colormap)
 #endif
 			setColormap(display, window, cmap, inwindow);
-	} else {
-		Scr[screen].colormap = cmap = DefaultColormapOfScreen(scr);
+	}
+#if 0
+	else {
+		/* white and black colors may not be right for GL modes so lets set them */
 		Scr[screen].black_pixel = BlackPixelOfScreen(scr);
 		Scr[screen].white_pixel = WhitePixelOfScreen(scr);
-    /* foregorund and background colors may not be right.... */
-  }
+		/* foreground and background colors may not be right.... */
+	}
+#endif
 }
 
 void
