@@ -43,6 +43,7 @@ static const char sccsid[] = "@(#)logout.c	4.02 97/04/01 xlockmore";
 
 extern Bool inroot, inwindow, nolock, debug;
 extern char *ProgramName;
+extern char *logoutCmd;
 
 /*-
  * This file contains a function called logoutUser() that, when called,
@@ -370,7 +371,18 @@ logoutUser(Display * display)
 	       ProgramName, getuid(), getenv("DISPLAY"));
 	syslogStop(XDisplayString(display));
 #endif
+	if (logoutCmd && *logoutCmd) {
+		int         cmd_pid;
 
+		if ((cmd_pid = FORK()) == -1) {
+			(void) fprintf(stderr, "Failed to launch \"%s\"\n", logoutCmd);
+			perror(ProgramName);
+			cmd_pid = 0;
+		} else if (!cmd_pid) {
+			(void) system(logoutCmd);
+			exit(0);
+		}
+	}
 #ifdef CLOSEDOWN_LOGOUT
 	(void) finish(display, False);
 #else
@@ -386,7 +398,7 @@ logoutUser(Display * display)
 #endif
 
 #ifdef CLOSEDOWN_LOGOUT
-  /* Do not want to kill other user's processes like if telnetted in */
+	/* Do not want to kill other user's processes like if telnetted in */
 	closedownLogout(display);
 	return;
 #endif
