@@ -81,7 +81,7 @@ LockStruct  LockProcs[] =
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	{"cartoon", init_cartoon, draw_cartoon, release_cartoon,
 	 NULL, init_cartoon, NULL, &cartoon_opts,
-	 10000, 1, 1, 1, 4, 1.0, "",
+	 10000, 1, 1, 1, 64, 1.0, "",
 	 "Shows bouncing cartoons", 0, NULL},
 #endif
 	{"clock", init_clock, draw_clock, release_clock,
@@ -98,7 +98,7 @@ LockStruct  LockProcs[] =
 	 "Shows polygons in 2D plane groups", 0, NULL},
 	{"daisy", init_daisy, draw_daisy, release_daisy,
 	 refresh_daisy, init_daisy, NULL, &daisy_opts,
-	 100000, 300, 350, 1, 6, 1.0, "",
+	 100000, 300, 350, 1, 64, 1.0, "",
 	 "Shows a meadow of daisies", 0, NULL},
 	{"dclock", init_dclock, draw_dclock, release_dclock,
 	 refresh_dclock, init_dclock, NULL, &dclock_opts,
@@ -150,10 +150,6 @@ LockStruct  LockProcs[] =
 	 1000, 1, 2, 1, 64, 1.0, "",
 	 "Shows GL's gears", 0, NULL},
 #endif
-	{"geometry", init_geometry, draw_geometry, release_geometry,
-	 refresh_geometry, init_geometry, NULL, &geometry_opts,
-	 10000, -10, 1, 1, 64, 1.0, "",
-	 "Shows morphing of a complete graph", 0, NULL},
 	{"grav", init_grav, draw_grav, release_grav,
 	 refresh_grav, init_grav, NULL, &grav_opts,
 	 10000, -12, 1, 1, 64, 1.0, "",
@@ -220,7 +216,7 @@ LockStruct  LockProcs[] =
 	 "Shows lissajous worms", 0, NULL},
 	{"loop", init_loop, draw_loop, release_loop,
 	 refresh_loop, init_loop, NULL, &loop_opts,
-	 100000, 1, 1600, -12, 15, 1.0, "",
+	 100000, 1, 1600, -12, 64, 1.0, "",
 	 "Shows Langton's self-producing loops", 0, NULL},
 	{"mandelbrot", init_mandelbrot, draw_mandelbrot, release_mandelbrot,
 	 NULL, init_mandelbrot, NULL, &mandelbrot_opts,
@@ -258,7 +254,7 @@ LockStruct  LockProcs[] =
     "Shows a man with a big nose runs around spewing out messages", 0, NULL},
 	{"pacman", init_pacman, draw_pacman, release_pacman,
 	 refresh_pacman, init_pacman, NULL, &pacman_opts,
-	 100000, 10, 1, 0, 6, 1.0, "",
+	 100000, 10, 1, 0, 64, 1.0, "",
 	 "Shows Pacman(tm)", 0, NULL},
 	{"penrose", init_penrose, draw_penrose, release_penrose,
 	 init_penrose, init_penrose, NULL, &penrose_opts,
@@ -289,7 +285,7 @@ LockStruct  LockProcs[] =
 	 "Shows fireworks", 0, NULL},
 	{"qix", init_qix, draw_qix, release_qix,
 	 refresh_qix, init_qix, NULL, &qix_opts,
-	 30000, 1, 64, 1, 64, 1.0, "",
+	 30000, -5, 32, 1, 64, 1.0, "",
 	 "Shows spinning lines a la Qix(tm)", 0, NULL},
 	{"roll", init_roll, draw_roll, release_roll,
 	 refresh_roll, init_roll, NULL, &roll_opts,
@@ -341,11 +337,11 @@ LockStruct  LockProcs[] =
 #endif
 	{"star", init_star, draw_star, release_star,
 	 refresh_star, init_star, NULL, &star_opts,
-	 75000, 100, 1, 100, 4, 0.2, "",
+	 75000, 100, 1, 100, 64, 0.2, "",
 	 "Shows a star field with a twist", 0, NULL},
 	{"strange", init_strange, draw_strange, release_strange,
 	 init_strange, init_strange, NULL, &strange_opts,
-	 1000, 1, 1, 1, 4, 1.0, "",
+	 1000, 1, 1, 1, 64, 1.0, "",
 	 "Shows strange attractors", 0, NULL},
 #ifdef USE_GL
 	{"superquadrics", init_superquadrics, draw_superquadrics, release_superquadrics,
@@ -401,7 +397,7 @@ LockStruct  LockProcs[] =
 /* SPECIAL MODES */
 	{"blank", init_blank, draw_blank, release_blank,
 	 refresh_blank, init_blank, NULL, &blank_opts,
-	 3000000, 1, 1, 1, 4, 1.0, "",
+	 3000000, 1, 1, 1, 64, 1.0, "",
 	 "Shows nothing but a black screen", 0, NULL},
 #ifdef USE_BOMB
 	{"bomb", init_bomb, draw_bomb, release_bomb,
