@@ -216,7 +216,16 @@ typedef struct ModeInfo_s {
 #define MI_USERDATA(mi)		((mi)->lockstruct->userdata)
 
 #ifdef SOLARIS2
+/*-
+ * It was acting weird with the first rectangle drawn
+ * on a Ultra2 with TrueColor Solaris2.x in deco mode.
+ * Not sure if other modes are affected... now handled in deco mode.
+ * If it does happen it is probably not as noticable to worry about.
+ */
+#endif
+#if 0
 #define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
+{ \
  XSetForeground(MI_DISPLAY(mi), gc, pixel); \
  if ((MI_WIN_WIDTH(mi) >= 2) || (MI_WIN_HEIGHT(mi) >= 2)) { \
  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 1, 1, \
@@ -225,16 +234,15 @@ typedef struct ModeInfo_s {
   (unsigned int) MI_WIN_WIDTH(mi) - 1, (unsigned int) MI_WIN_HEIGHT(mi) - 1); \
  } else \
  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 0, 0, \
-  (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi))
+  (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi)); \
+}
 #else
-/*-
- * It was acting weird with the first rectangle drawn after this
- * on a Ultra2 with TrueColor Solaris2.6 (deco for example)
- */
 #define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
+{ \
  XSetForeground(MI_DISPLAY(mi), gc, pixel); \
  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, \
-   0, 0, (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi))
+   0, 0, (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi)); \
+}
 #endif
 #define MI_CLEARWINDOWCOLOR(mi, pixel) \
  MI_CLEARWINDOWCOLORMAP(mi, MI_GC(mi), pixel)
@@ -436,12 +444,6 @@ extern ModeSpecOpt gears_opts;
 
 #endif
 
-extern ModeHook init_geometry;
-extern ModeHook draw_geometry;
-extern ModeHook release_geometry;
-extern ModeHook refresh_geometry;
-extern ModeSpecOpt geometry_opts;
-
 extern ModeHook init_grav;
 extern ModeHook draw_grav;
 extern ModeHook release_grav;
