@@ -11,6 +11,10 @@ static const char sccsid[] = "@(#)passwd.c	4.02 97/04/01 xlockmore";
  * Revision History:
  *
  * Changes maintained by David Bagley <bagleyd@bigfoot.com>
+ * 24-Jan-98: Updated PAM support and made it configure-able.
+ *            Marc Ewing <marc@redhat.com>  Original PAM support from
+ *            25-Jul-96 Michael K. Johnson <johnsonm@redhat.com>
+ * 18-Jan-98: Kerberos corrections by RLS <sinkr@cbl.umces.edu>
  * 25-May-96: When xlock is compiled with shadow passwords it will still
  *            work on non shadowed systems.  Marek Michalkiewicz
  *            <marekm@i17linuxb.ists.pwr.wroc.pl>
@@ -44,15 +48,21 @@ extern Bool inwindow;
 extern Bool grabmouse;
 extern Bool nolock;
 
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
+#include <sys/stat.h>		/* order of includes matters */
+#endif
+
 #if !defined( VMS ) && !defined( SUNOS_ADJUNCT_PASSWD )
 #include <pwd.h>
 #endif
 
-#ifdef __bsdi__
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )  || defined( __bsdi__ ) || defined( HAVE_KRB4 ) || defined( HAVE_KRB5 )
 #include <sys/param.h>
-#if _BSDI_VERSION >= 199608
-#define       BSD_AUTH
 #endif
+
+
+#if defined( __bsdi__ ) && _BSDI_VERSION >= 199608
+#define       BSD_AUTH
 #endif
 
 #ifdef        BSD_AUTH
@@ -66,16 +76,17 @@ static login_cap_t *rlc = NULL;
 #include <syslog.h>
 #endif
 
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
 #include <fcntl.h>
 #include <errno.h>
 extern int  errno;
 
 void        get_multiple(struct passwd *);
-void        set_multiple();
+void        set_multiple(int uid);
 
 #define BUFMAX 1024		/* Maximum size of pipe buffer */
 
+
 /* Linked list to keep track of everyone that's authorized * to unlock the
    screen. */
 struct pwln {
@@ -130,18 +141,21 @@ gid_t       rgid, egid;
 #endif
 
 #if defined( __linux__ ) && defined( __ELF__ ) && !defined( HAVE_SHADOW )
-/* Linux may or may not have shadow passwords, so it is best to make the same
-   binary work with both shadow and non-shadow passwords. It's easy with the
-   ELF libc since it has getspnam() and there is no need for any additional
-   libraries like libshadow.a.  This is a quick hack; it probably should be
-   done properly in the Imakefile.  */
+/*-
+ * Linux may or may not have shadow passwords, so it is best to make the same
+ * binary work with both shadow and non-shadow passwords. It's easy with the
+ * ELF libc since it has getspnam() and there is no need for any additional
+ * libraries like libshadow.a.
+ */
 #define HAVE_SHADOW
 #endif
 
 #if defined( __linux__ ) && defined( HAVE_SHADOW ) && defined( HAVE_PW_ENCRYPT )
-/* Deprecated - long passwords have known weaknesses.  Also, pw_encrypt is
-   non-standard (requires libshadow.a) while everything else you need to
-   support shadow passwords is in the standard (ELF) libc.  */
+/*-
+ * Deprecated - long passwords have known weaknesses.  Also, pw_encrypt is
+ *  non-standard (requires libshadow.a) while everything else you need to
+ *  support shadow passwords is in the standard (ELF) libc.
+ */
 #define crypt pw_encrypt
 #endif
 
@@ -228,7 +242,76 @@ struct itmlst {
 
 char        user[PASSLENGTH];
 
-#ifndef ultrix
+#ifdef PAM
+#include <security/pam_appl.h>
+/* used to pass the password to the conversation function */
+static char *PAM_password;
+
+/*-
+ * PAM conversation function
+ * Here we assume (for now, at least) that echo on means login name, and
+ * echo off means password.
+ */
+static int
+PAM_conv(int num_msg,
+	 /* const */ struct pam_message **msg,
+	 struct pam_response **resp,
+	 void *appdata_ptr)
+{
+	int         replies = 0;
+	struct pam_response *reply = NULL;
+
+
+#define COPY_STRING(s) (s) ? strdup(s) : NULL
+
+
+	reply = (struct pam_response *) malloc(sizeof (struct pam_response) *
+					       num_msg);
+
+	if (!reply)
+		return PAM_CONV_ERR;
+
+
+	for (replies = 0; replies < num_msg; replies++) {
+		switch (msg[replies]->msg_style) {
+			case PAM_PROMPT_ECHO_ON:
+				reply[replies].resp_retcode = PAM_SUCCESS;
+				reply[replies].resp = COPY_STRING(user);
+				/* PAM frees resp */
+				break;
+			case PAM_PROMPT_ECHO_OFF:
+				reply[replies].resp_retcode = PAM_SUCCESS;
+				reply[replies].resp = COPY_STRING(PAM_password);
+				/* PAM frees resp */
+				break;
+			case PAM_TEXT_INFO:
+				/* ignore it... */
+				reply[replies].resp_retcode = PAM_SUCCESS;
+				reply[replies].resp = NULL;
+				break;
+			case PAM_ERROR_MSG:
+				/* ignore it... */
+				reply[replies].resp_retcode = PAM_SUCCESS;
+				reply[replies].resp = NULL;
+				break;
+			default:
+				/* Must be an error of some sort... */
+				(void) free((void *) reply);
+				return PAM_CONV_ERR;
+		}
+	}
+	*resp = reply;
+	return PAM_SUCCESS;
+}
+static struct pam_conv PAM_conversation =
+{
+	&PAM_conv,
+	NULL
+};
+
+#endif /* PAM */
+
+#if !defined( ultrix ) && !defined( PAM )
 static char userpass[PASSLENGTH];
 static char rootpass[PASSLENGTH];
 
@@ -250,7 +333,6 @@ static int  check_dce_net_passwd(char *, char *);
 #else /* HAVE_KRB5 */
 #include <krb5.h>
 #endif
-#include <sys/param.h>
 static int  krb_check_password(struct passwd *, char *);
 
 #endif
@@ -265,7 +347,7 @@ extern char *crypt(const char *, const char *);
 #endif
 #endif
 
-#if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) &&  !defined( HP_PASSWDETC ) && !defined( VMS )))
+#if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) && !defined( HP_PASSWDETC ) && !defined( VMS )))
 static struct passwd *
 my_passwd_entry(void)
 {
@@ -309,6 +391,34 @@ my_passwd_entry(void)
 }
 #endif
 
+#ifdef SAFEWORD
+int
+checkDynamic()
+{
+	struct passwd *pw;
+	char       *buf;
+
+#ifndef SAFELOG_SWCHECK
+#define SAFELOG_SWCHECK "/progs/safelog/swcheck"
+#endif
+#ifndef SAFELOG_IDENT
+#define SAFELOG_IDENT "/progs/safelog/ident"
+#endif
+
+
+	if (!(pw = my_passwd_entry())) {
+		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
+		buf = (char *) malloc(strlen(ProgramName) + 80);
+		(void) sprintf(buf,
+			     "%s: could not get user shell.\n", ProgramName);
+		error(buf);
+		(void) free((void *) buf);	/* Should never get here */
+	}
+	return (!(strcmp(pw->pw_shell, SAFELOG_SWCHECK) &&
+		  strcmp(pw->pw_shell, SAFELOG_IDENT)));
+}
+#endif
+
 static void
 getUserName(void)
 {
@@ -336,10 +446,10 @@ getUserName(void)
 	pipe(pipefd);
 
 	if ((pid = fork()) == 0) {
-		close(pipefd[0]);
+		(void) close(pipefd[0]);
 		pw = getpwuid(getuid());
 		write(pipefd[1], pw->pw_name, strlen(pw->pw_name));
-		close(pipefd[1]);
+		(void) close(pipefd[1]);
 		_exit(0);
 	}
 	if (pid < 0) {
@@ -349,7 +459,7 @@ getUserName(void)
 		error(buf);
 		(void) free((void *) buf);	/* Should never get here */
 	}
-	close(pipefd[1]);
+	(void) close(pipefd[1]);
 
 	while ((n = read(pipefd[0], &(user[total]), 50)) > 0)
 		total += n;
@@ -402,7 +512,7 @@ getUserName(void)
 	}
 	(void) strcpy(user, pw->pw_name);
 
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
 	get_multiple(pw);
 #endif
 
@@ -411,7 +521,110 @@ getUserName(void)
 #endif /* !VMS */
 }
 
-#if defined( HAVE_SHADOW ) && !defined( USE_XLOCKRC )
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
+
+#define CPASSLENGTH 14
+#define CPASSCHARS 64
+static void
+gpasskey(char *pass)
+{
+	static char saltchars[CPASSCHARS + 1] =
+	"abcdefghijklmnopwrstuvwxyzABCDEFGHIJKLMNOPWRSTUVWXYZ1234567890./";
+	char        pw[9], salt[2], *pw2;
+
+#if 0
+	char       *getpass(const char *);
+
+#endif
+
+	(void) strncpy(pw, (char *) getpass("Key: "), sizeof (pw));
+	pw2 = (char *) getpass("Again:");
+	if (strcmp(pw, pw2) != 0) {
+		(void) fprintf(stderr, "%s: Mismatch, try again\n", ProgramName);
+		exit(1);
+	}
+	/* grab a random printable character that is not a colon */
+	salt[0] = saltchars[LRAND() & (CPASSCHARS - 1)];
+	salt[1] = saltchars[LRAND() & (CPASSCHARS - 1)];
+	(void) strncpy(pass, (char *) crypt(pw, salt), CPASSLENGTH);
+}
+
+static void
+gpass()
+{
+	FILE       *fp;
+	extern char *cpasswd;
+	char        xlockrc[MAXPATHLEN], *home;
+
+	if (!cpasswd || !*cpasswd) {
+		/*
+		 * No password given on command line or from database, get from
+		 * $HOME/.xlockrc instead.
+		 */
+		if ((home = getenv("HOME")) == 0) {
+			/* struct passwd *p = getpwuid(getuid()); */
+			struct passwd *p = my_passwd_entry();
+
+			if (p == 0) {
+				char       *buf = (char *) malloc(strlen(ProgramName) + 80);
+
+				(void) sprintf(buf,
+					  "%s: Who are you?\n", ProgramName);
+				error(buf);
+				(void) free((void *) buf);	/* Should never get here */
+			}
+			home = p->pw_dir;
+		}
+		(void) strncpy(xlockrc, home, MAXPATHLEN);
+		(void) strncat(xlockrc, "/.xlockrc", MAXPATHLEN);
+		if ((fp = my_fopen(xlockrc, "r")) == NULL) {
+			if ((fp = my_fopen(xlockrc, "w")) != NULL)
+				(void) fchmod(fileno(fp), 0600);
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+			(void) strcpy(userpass, "*");
+#else
+			gpasskey(userpass);
+#endif /* KRB4 || KRB5 */
+			if (fp)
+				(void) fprintf(fp, "%s\n", userpass);
+		} else {
+			char        buf[BUFSIZ];
+
+			(void) fchmod(fileno(fp), 0600);
+			buf[0] = '\0';
+			if ((fgets(buf, sizeof buf, fp) == NULL) ||
+			    (!(strlen(buf) == CPASSLENGTH - 1 ||
+			       (strlen(buf) == CPASSLENGTH && buf[CPASSLENGTH - 1] == '\n')))) {
+				(void) fprintf(stderr, "%s: %s crypted password %s\n", xlockrc,
+				       buf[0] == '\0' ? "null" : "bad", buf);
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+				gpasskey(buf);
+#else
+				exit(1);
+#endif
+			}
+			buf[CPASSLENGTH - 1] = '\0';
+			(void) strncpy(userpass, buf, CPASSLENGTH);
+		}
+		if (fp)
+			(void) fclose(fp);
+	} else {
+
+		if (strlen(cpasswd) != CPASSLENGTH - 1) {
+			(void) fprintf(stderr, "%s: bad crypted password %s\n",
+				       ProgramName, cpasswd);
+			exit(1);
+		} else
+			(void) strncpy(userpass, cpasswd, CPASSLENGTH);
+	}
+}
+
+#endif /* USE_XLOCKRC || FALLBACK_XLOCKRC */
+
+#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )  && !defined ( PAM )
+#ifndef USE_XLOCKRC
+
+#if defined( HAVE_SHADOW ) && !defined( AFS )
 static int
 passwd_invalid(char *passwd)
 {
@@ -421,9 +634,6 @@ passwd_invalid(char *passwd)
 }
 #endif
 
-#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )
-#ifndef USE_XLOCKRC
-/* This routine is not needed if HAVE_FCNTL_H and USE_MULTIPLE_ROOT */
 static void
 getCryptedUserPasswd(void)
 {
@@ -468,10 +678,10 @@ getCryptedUserPasswd(void)
 	pipe(pipefd);
 
 	if ((pid = fork()) == 0) {
-		close(pipefd[0]);
+		(void) close(pipefd[0]);
 		pw = getpwuid(getuid());
 		write(pipefd[1], pw->pw_passwd, strlen(pw->pw_passwd));
-		close(pipefd[1]);
+		(void) close(pipefd[1]);
 		_exit(0);
 	}
 	if (pid < 0) {
@@ -481,7 +691,7 @@ getCryptedUserPasswd(void)
 		error(buf);
 		(void) free((void *) buf);	/* Should never get here */
 	}
-	close(pipefd[1]);
+	(void) close(pipefd[1]);
 
 	while ((n = read(pipefd[0], &(userpass[total]), 50)) > 0)
 		total += n;
@@ -534,21 +744,29 @@ getCryptedUserPasswd(void)
 	/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 	/* Check if there is any chance of unlocking the display later...  */
 	/* Program probably needs to be setuid to root.  */
-	/* If this is not possible, compile with -DUSE_XLOCKRC */
+	(void) strcpy(userpass, pw->pw_passwd);
 #ifdef HAVE_SHADOW
 #ifndef AFS
 	if (passwd_invalid(pw->pw_passwd)) {
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: it looks like you have shadow passwording.\nContact your administrator to setgid or setuid %s for /etc/shadow read access.\n",
-			       ProgramName, ProgramName);
-		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
-	}
-#endif /* AFS */
-#endif /* HAVE_SHADOW */
-	(void) strcpy(userpass, pw->pw_passwd);
+		extern Bool verbose, debug;
 
+		buf = (char *) malloc(strlen(ProgramName) + 80);
+		if (verbose || debug) {
+			(void) fprintf(stderr,
+			  "%s: it looks like you have shadow passwording.\n",
+				       ProgramName);
+			(void) fprintf(stderr, "Contact your administrator to setgid or\n");
+			(void) fprintf(stderr, "setuid %s for /etc/shadow read access.\n",
+				       ProgramName);
+			(void) fprintf(stderr, "Falling back on $HOME/.xlockrc password.\n");
+		}
+		gpass();
+	}
+#endif /* !AFS */
+#endif /* HAVE_SHADOW */
+#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_USER ))
+	set_multiple(getuid());
+#endif /* HAVE_FCNTL_H && MULTIPLE_USER */
 #endif /* !OSF1_ENH_SEC */
 #endif /* !HP_PASSWDETC */
 #endif /* !VMS */
@@ -600,10 +818,10 @@ getCryptedRootPasswd(void)
 	pipe(pipefd);
 
 	if ((pid = fork()) == 0) {
-		close(pipefd[0]);
+		(void) close(pipefd[0]);
 		pw = getpwnam(ROOT);
 		write(pipefd[1], pw->pw_passwd, strlen(pw->pw_passwd));
-		close(pipefd[1]);
+		(void) close(pipefd[1]);
 		_exit(0);
 	}
 	if (pid < 0) {
@@ -613,7 +831,7 @@ getCryptedRootPasswd(void)
 		error(buf);
 		(void) free((void *) buf);	/* Should never get here */
 	}
-	close(pipefd[1]);
+	(void) close(pipefd[1]);
 
 	while ((n = read(pipefd[0], &(rootpass[total]), 50)) > 0)
 		total += n;
@@ -681,7 +899,7 @@ getCryptedRootPasswd(void)
 	(void) strcpy(rootpass, pw->pw_passwd);
 
 #if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
-	set_multiple();
+	set_multiple(0);
 #endif /* HAVE_FCNTL_H && MULTIPLE_ROOT */
 
 #endif /* !OSF1_ENH_SEC */
@@ -689,18 +907,18 @@ getCryptedRootPasswd(void)
 #endif /* !VMS */
 }
 
-#endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH */
+#endif /* !ultrix && !DCE_PASSWD && !BSD_AUTH && !PAM */
 
 
 /*-
- * we don't allow for root to have no password, but we handle the case
+ * We do not allow for root to have no password, but we handle the case
  * where the user has no password correctly; they have to hit return
- * only
+ * only.
  */
 int
 checkPasswd(char *buffer)
 {
-	int         done;
+	int         done = False;
 
 #ifdef VMS
 	struct ascid password;
@@ -735,9 +953,9 @@ checkPasswd(char *buffer)
 		done = !strcmp(userpass, crypt(buffer, userpass));
 
 	if (done)
-		return 1;
+		return True;
 	if (!allowroot)
-		return 0;
+		return False;
 
 	if (rootnet)
 		done = check_dce_net_passwd(ROOT, buffer);
@@ -745,6 +963,42 @@ checkPasswd(char *buffer)
 		done = !strcmp(rootpass, crypt(buffer, rootpass));
 #else /* !DCE_PASSWD */
 
+#ifdef PAM
+/*-
+ * Use PAM to do authentication.  No session logging, only authentication.
+ * Bail out if there are any errors.
+ * For now, don't try to display strings explaining errors.
+ * Later, we could integrate PAM more by posting errors to the
+ * user.
+ * Query: should we be using PAM_SILENT to shut PAM up?
+ */
+	pam_handle_t *pamh;
+	int         pam_error;
+
+#define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
+    pam_end(pamh, 0); return 0; \
+}
+	PAM_password = buffer;
+	pam_error = pam_start("xlock", user, &PAM_conversation, &pamh);
+	PAM_BAIL;
+	pam_error = pam_authenticate(pamh, 0);
+	if (pam_error != PAM_SUCCESS) {
+		/* Try as root; bail if no success there either */
+		pam_error = pam_set_item(pamh, PAM_USER, ROOT);
+		PAM_BAIL;
+		pam_error = pam_authenticate(pamh, 0);
+		PAM_BAIL;
+	}
+	/* Don't do account management or credentials; credentials
+	 * aren't needed and account management would just lock up
+	 * a computer and require root to come and unlock it.  Blech.
+	 */
+	pam_end(pamh, PAM_SUCCESS);
+	/* If this point is reached, the user has been authenticated. */
+	done = True;
+
+#else /* !PAM */
+
 #ifdef ultrix
 
 #ifdef HAVE_SETEUID
@@ -773,8 +1027,7 @@ checkPasswd(char *buffer)
 	char       *style;
 	char       *name;
 
-	done = 0;
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER ))
 	/* Scan through the linked list until you match a password.  Print
 	 * message to log if password match doesn't equal the user.
 	 *
@@ -792,69 +1045,89 @@ checkPasswd(char *buffer)
 		style = login_getstyle(lc, buffer, "auth-xlock");
 		if (auth_response(name, lc->lc_class, style,
 				  "response", NULL, "", pass) > 0)
-			done = 1;
+			done = True;
 		else if (rlc != NULL) {
 			style = login_getstyle(rlc, buffer, "auth-xlock");
 			if (auth_response(ROOT, rlc->lc_class, style,
 					  "response", NULL, "", pass) > 0)
-				done = 1;
+				done = True;
 		}
 		pass[-1] = ':';
 	}
-	if (done == 0) {
+	if (!done) {
 		style = login_getstyle(lc, NULL, "auth-xlock");
 		if (auth_response(name, lc->lc_class, style,
 				  "response", NULL, "", buffer) > 0)
-			done = 1;
+			done = True;
 		else if (rlc != NULL) {
 			style = login_getstyle(rlc, NULL, "auth-xlock");
 			if (auth_response(ROOT, rlc->lc_class, style,
 					  "response", NULL, "", buffer) > 0)
-				done = 1;
+				done = True;
 		}
 	}
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER ))
 }
 #endif
 
 #else /* !BSD_AUTH */
 
 #ifdef AFS
-	char       *reason;
+	{
+		char       *reason;
 
-	/* check afs passwd first, then local, then root */
-	done = !ka_UserAuthenticate(user, "", 0, buffer, 0, &reason);
-	if (!done)
+		/* check afs passwd first, then local, then root */
+		if (!ka_UserAuthenticate(user, "", 0, buffer, 0, &reason))
+			return True;
+
+	}
 #endif /* !AFS */
 #if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+	{
+		/* Somehow, buffer gets 'erased' after either doing a krb pw check
+		   or a reg pw check */
+		char        buffer2[1024];
+
+		(void) strncpy(buffer2, buffer, sizeof (buffer2));
+#if 0
 		if (!strcmp(userpass, "*"))
-			done = (krb_check_password((struct passwd *) getpwuid(getuid()), buffer) ||
-				(allowroot && !strcmp((char *) crypt(buffer, rootpass), rootpass)));
-		else
-#endif /* !HAVE_KRB4 && !HAVE_KRB5 */
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
-			/* Scan through the linked list until you match a password.  Print
-			 * message to log if password match doesn't equal the user.
-			 *
-			 * This should be changed to allow the user name to be typed in also
-			 * to make this more secure.
-			 */
-			done = 0;
-	for (pwll = pwllh; pwll->next; pwll = pwll->next)
-		if (!strcmp((char *) crypt(buffer, pwll->pw_passwd), pwll->pw_passwd)) {
-#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
-			if (strcmp(user, pwll->pw_name) != 0)
-				syslog(SYSLOG_NOTICE, "%s: %s unlocked screen",
-				       ProgramName, pwll->pw_name);
 #endif
-			done = 1;
-			break;
-		}
-#else
-			done = (!strcmp((char *) crypt(buffer, userpass), userpass));
+			if (krb_check_password((struct passwd *) getpwuid(getuid()), buffer2))
+				return True;
+	}
+#endif /* !HAVE_KRB4 && !HAVE_KRB5 */
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
+	if (!done) {
+		/* Scan through the linked list until you match a password.  Print
+		 * message to log if password match does not equal the user.
+		 *
+		 * This should be changed to allow the user name to be typed in also
+		 * to make this more secure.
+		 */
+		for (pwll = pwllh; pwll->next; pwll = pwll->next)
+			if (!strcmp((char *) crypt(buffer, pwll->pw_passwd), pwll->pw_passwd)) {
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+				if (strcmp(user, pwll->pw_name) != 0)
+					syslog(SYSLOG_NOTICE, "%s: %s unlocked screen",
+					       ProgramName, pwll->pw_name);
+#endif
+				return True;
+			}
+	}
+#endif
+	if (!done) {
+		done = (!strcmp((char *) crypt(buffer, userpass), userpass));
+		/* userpass is used */
+		if (!*userpass && *buffer)
+			/*
+			 * the user has no password, but something was typed anyway.
+			 * sounds fishy: don't let him in...
+			 */
+			done = False;
+	}
 #if 0
 	(void) printf("buffer=%s, encrypt=%s, userpass=%s\n", buffer,
-		      crypt(buffer, userpass), userpass);
+		      (char *) crypt(buffer, userpass), userpass);
 #endif
 	if (!done) {
 		done = (allowroot &&
@@ -864,17 +1137,9 @@ checkPasswd(char *buffer)
 			syslog(SYSLOG_NOTICE, "%s: %s unlocked screen", ProgramName, ROOT);
 #endif
 	}
-#endif
-	/* userpass is used */
-	if (!*userpass && *buffer)
-		/*
-		 * the user has no password, but something was typed anyway.
-		 * sounds fishy: don't let him in...
-		 */
-		done = False;
-
 #endif /* !BSD_AUTH */
 #endif /* !ultrix */
+#endif /* !PAM */
 #endif /* !DCE_PASSWD */
 #endif /* !VMS */
 
@@ -1070,7 +1335,7 @@ check_dce_net_passwd(char *usr, char *pass)
 			(void) fprintf(stderr, "sec_login_setup_identity() failed: %s\n",
 				       error_string(error_status));
 			pthread_lock_global_np();
-			return 0;
+			return False;
 		}
 	} else
 		current_context = 1;
@@ -1079,7 +1344,7 @@ check_dce_net_passwd(char *usr, char *pass)
 
 	/* make a copy of pass, because sec_login_validate_identity() will
 	   clobber the plaintext password passed to it */
-	passcpy = strdup(pass);
+	passcpy = (char *) strdup(pass);
 
 	password.key.key_type = sec_passwd_plain;
 	password.key.tagged_union.plain = (unsigned char *) passcpy;
@@ -1105,7 +1370,7 @@ check_dce_net_passwd(char *usr, char *pass)
 			(void) fprintf(stderr, "sec_login_refresh_identity failed: %s\n",
 				       error_string(error_status));
 		} else {
-			passcpy = strdup(pass);
+			passcpy = (char *) strdup(pass);
 
 			password.key.key_type = sec_passwd_plain;
 			password.key.tagged_union.plain = (unsigned char *) passcpy;
@@ -1150,9 +1415,12 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 	/* find local realm */
 	if (krb_get_lrealm(realm, 1) != KSUCCESS)
-		/*(void) strncpy(realm, KRB_REALM, sizeof (realm)); */
+#ifdef KRB_REALM
+		/* krb_get_default_realm() may not work well on Solaris */
+		(void) strncpy(realm, KRB_REALM, sizeof (realm));
+#else
 		(void) strncpy(realm, krb_get_default_realm(), sizeof (realm));
-
+#endif
 	/* Construct a ticket file */
 	(void) sprintf(tkfile, "/tmp/xlock_tkt_%d", getpid());
 
@@ -1164,10 +1432,10 @@ krb_check_password(struct passwd *pwd, char *pass)
 			      "krbtgt", realm,
 			      DEFAULT_TKT_LIFE,
 			      pass) == INTK_OK) {
-		unlink(tkfile);
-		return 1;
+		(void) unlink(tkfile);
+		return True;
 	}
-	return 0;
+	return False;
 }
 #endif /* HAVE_KERB4 */
 
@@ -1208,23 +1476,23 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 	if ((code = krb5_timeofday(kcontext, &now))) {
 		com_err(ProgramName, code, "while getting time of day");
-		return 0;	/* seems better to deny access, than just exit, which
+		return False;	/* seems better to deny access, than just exit, which
 				   was what happened in kinit  */
 	}
 	if ((code = krb5_cc_default(kcontext, &ccache))) {
 		com_err(ProgramName, code, "while getting default ccache");
-		return 0;
+		return False;
 	}
 	code = krb5_cc_get_principal(kcontext, ccache, &me);
 	if (code) {
 		if ((code = krb5_parse_name(kcontext, pwd->pw_name, &me))) {
 			com_err(ProgramName, code, "when parsing name %s", pwd->pw_name);
-			return 0;
+			return False;
 		}
 	}
 	if ((code = krb5_unparse_name(kcontext, me, &client_name))) {
 		com_err(ProgramName, code, "when unparsing name");
-		return 0;
+		return False;
 	}
 	(void) memset((char *) &my_creds, 0, sizeof (my_creds));
 
@@ -1238,7 +1506,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 					krb5_princ_realm(kcontext, me)->data,
 					     0))) {
 		com_err(ProgramName, code, "while building server name");
-		return 0;
+		return False;
 	}
 	my_creds.server = server;
 
@@ -1247,7 +1515,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 	my_creds.times.renew_till = 0;
 
 	if (strlen(pass) == 0)
-		strcpy(pass, "*");
+		(void) strcpy(pass, "*");
 	/* if pass is NULL, krb5_get_in_tkt_with_password will prompt with
 	   krb5_default_pwd_prompt1 for password, you don't want that in
 	   this application, most likely the user won't have '*' for a
@@ -1261,27 +1529,27 @@ krb_check_password(struct passwd *pwd, char *pass)
 
 	if (code) {
 		if (code == KRB5KRB_AP_ERR_BAD_INTEGRITY)
-			return 0;	/* bad password entered */
+			return False;	/* bad password entered */
 		else {
 			com_err(ProgramName, code, "while getting initial credentials");
-			return 0;
+			return False;
 		}
 	}
 	code = krb5_cc_initialize(kcontext, ccache, me);
 	if (code != 0) {
 		com_err(ProgramName, code, "when initializing cache");
-		return 0;
+		return False;
 	}
 	code = krb5_cc_store_cred(kcontext, ccache, &my_creds);
 	if (code) {
 		com_err(ProgramName, code, "while storing credentials");
-		return 0;
+		return False;
 	}
 	krb5_free_principal(kcontext, server);
 
 	krb5_free_context(kcontext);
 
-	return 1;		/* success */
+	return True;		/* success */
 }
 #endif /* HAVE_KRB5 */
 
@@ -1294,7 +1562,7 @@ krb_check_password(struct passwd *pwd, char *pass)
 #endif
 #endif
 
-#if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_ROOT ))
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
 
 void
 get_multiple(struct passwd *pw)
@@ -1311,14 +1579,14 @@ get_multiple(struct passwd *pw)
 		}
 		pwllh = pwll;
 	}
-	if ((pwll->pw_name = strdup(pw->pw_name)) == NULL) {
+	if ((pwll->pw_name = (char *) strdup(pw->pw_name)) == NULL) {
 		perror("new");
 		exit(1);
 	}
 #ifdef        BSD_AUTH
 	pwll->pw_lc = login_getclass(pw->pw_class);
 #else
-	if ((pwll->pw_passwd = strdup(pw->pw_passwd)) == NULL) {
+	if ((pwll->pw_passwd = (char *) strdup(pw->pw_passwd)) == NULL) {
 		perror("new");
 		exit(1);
 	}
@@ -1330,7 +1598,7 @@ get_multiple(struct passwd *pw)
 }
 
 void
-set_multiple(void)
+set_multiple(int uid)
 {
 #ifdef BSD_AUTH
 	struct passwd *pw;
@@ -1346,9 +1614,9 @@ set_multiple(void)
 	for (pwll = pwllh; pwll->next; pwll = pwll->next);
 
 	while ((pw = getpwent()) != (struct passwd *) NULL) {
-		if (pw->pw_uid)
+		if (pw->pw_uid != uid)
 			continue;
-		if ((pwll->pw_name = strdup(pw->pw_name)) == NULL) {
+		if ((pwll->pw_name = (char *) strdup(pw->pw_name)) == NULL) {
 			perror("new");
 			exit(1);
 		}
@@ -1363,7 +1631,7 @@ set_multiple(void)
 	if (pwll->next = new_pwlnode())
 		pwll = pwll->next;
 #else /* !BSD_AUTH */
-	/* If you thought the above was sick, then you'll think this is
+	/* If you thought the above was sick, then you will think this is
 	 * downright horrific.  This is set up so that a child process
 	 * is created to read in the password entries using getpwent(3C).
 	 * In the man pages on the HPs, getpwent(3C) has in it the fact
@@ -1395,14 +1663,14 @@ set_multiple(void)
 		exit(1);
 	} else if (cid == 0) {
 		/* child process.  Used to read in password file.  Also checks to
-		 * see if the uid is zero.  If so, then writes that to the pipe.
+		 * see if the uid is one we want.  If so, then writes that to the pipe.
 		 */
 		register int sbuf = 0;
 		char       *cbuf, *pbuf;
 
-		close(pipefd[0]);
+		(void) close(pipefd[0]);
 		while ((pw = getpwent()) != (struct passwd *) NULL) {
-			if (pw->pw_uid != 0)
+			if (pw->pw_uid != uid)
 				continue;
 #ifdef HAVE_SHADOW
 			if ((spw = getspnam(pw->pw_name)) != NULL) {
@@ -1448,7 +1716,7 @@ set_multiple(void)
 		if (write(pipefd[1], buf, sbuf) != sbuf)
 			perror("write");
 
-		close(pipefd[1]);
+		(void) close(pipefd[1]);
 		exit(0);
 	} else {
 		/* parent process.  Does the actual creation of the linked list.
@@ -1460,7 +1728,7 @@ set_multiple(void)
 		char       *cbuf, *pbuf;
 		pwlptr      pwll;
 
-		close(pipefd[1]);
+		(void) close(pipefd[1]);
 
 		if (pwllh == (pwlptr) NULL) {
 			if ((pwll = new_pwlnode()) == (pwlptr) ENOMEM) {
@@ -1532,110 +1800,20 @@ set_multiple(void)
 				break;
 			pwll = pwll->next;
 		}
-		close(pipefd[0]);
+		(void) close(pipefd[0]);
 	}
 #endif /* !BSD_AUTH */
 }
-#endif
-
-#ifdef USE_XLOCKRC
-
-static void
-gpass(void)
-{
-#include <sys/param.h>
-#include <sys/stat.h>
-#define CPASSLENGTH 14
-#define CPASSCHARS 64
-	FILE       *fp;
-	extern char *cpasswd;
-	static char saltchars[CPASSCHARS + 1] =
-	"abcdefghijklmnopwrstuvwxyzABCDEFGHIJKLMNOPWRSTUVWXYZ1234567890./";
-	char        xlockrc[MAXPATHLEN], *home;
-
-#if 0
-	char       *getpass(const char *);
 
 #endif
 
-	if (cpasswd[0] == '\0') {
-		/*
-		 * No password given on command line or from database, get from
-		 * $HOME/.xlockrc instead.
-		 */
-		if ((home = getenv("HOME")) == 0) {
-			/* struct passwd *p = getpwuid(getuid()); */
-			struct passwd *p = my_passwd_entry();
-
-			if (p == 0) {
-				char       *buf = (char *) malloc(strlen(ProgramName) + 80);
-
-				(void) sprintf(buf,
-					  "%s: Who are you?\n", ProgramName);
-				error(buf);
-				(void) free((void *) buf);	/* Should never get here */
-			}
-			home = p->pw_dir;
-		}
-		(void) strncpy(xlockrc, home, MAXPATHLEN);
-		(void) strncat(xlockrc, "/.xlockrc", MAXPATHLEN);
-		if ((fp = my_fopen(xlockrc, "r")) == NULL) {
-			char        pw[9], salt[2], *pw2;
-
-			if ((fp = my_fopen(xlockrc, "w")) != NULL)
-				(void) fchmod(fileno(fp), 0600);
-			while (1) {
-				(void) strncpy(pw, getpass("Key: "), sizeof pw);
-				pw2 = getpass("Again:");
-				if (strcmp(pw, pw2) != 0) {
-					(void) fprintf(stderr, "%s: Mismatch, try again\n", ProgramName);
-					exit(1);
-				} else
-					break;
-			}
-			/* grab a random printable character that isn't a colon */
-			salt[0] = saltchars[LRAND() & (CPASSCHARS - 1)];
-			salt[1] = saltchars[LRAND() & (CPASSCHARS - 1)];
-			(void) strncpy(userpass, (char *) crypt(pw, salt), CPASSLENGTH);
-			if (fp)
-				(void) fprintf(fp, "%s\n", userpass);
-		} else {
-			char        buf[BUFSIZ];
-
-			(void) fchmod(fileno(fp), 0600);
-			buf[0] = '\0';
-			if ((fgets(buf, sizeof buf, fp) == NULL) ||
-			    (!(strlen(buf) == CPASSLENGTH - 1 ||
-			       (strlen(buf) == CPASSLENGTH && buf[CPASSLENGTH - 1] == '\n')))) {
-				(void) fprintf(stderr, "%s: %s crypted password %s\n", xlockrc,
-				       buf[0] == '\0' ? "null" : "bad", buf);
-				exit(1);
-			}
-			buf[CPASSLENGTH - 1] = '\0';
-			(void) strncpy(userpass, buf, CPASSLENGTH);
-		}
-		if (fp)
-			(void) fclose(fp);
-	} else {
-
-		if (strlen(cpasswd) != CPASSLENGTH - 1) {
-			(void) fprintf(stderr, "%s: bad crypted password %s\n",
-				       ProgramName, cpasswd);
-			exit(1);
-		} else
-			(void) strncpy(userpass, cpasswd, CPASSLENGTH);
-	}
-}
-
-#endif /* USE_XLOCKRC */
-
 void
 initPasswd(void)
 {
 	getUserName();
-#if !defined( ultrix ) && !defined( DCE_PASSWD )
+#if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( PAM )
 	if (!nolock && !inroot && !inwindow && grabmouse) {
-#ifdef        BSD_AUTH
+#ifdef BSD_AUTH
 		struct passwd *pwd = getpwnam(user);
 
 		lc = login_getclass(pwd->pw_class);
@@ -1651,7 +1829,7 @@ initPasswd(void)
 			getCryptedRootPasswd();
 #endif /* !BSD_AUTH */
 	}
-#endif /* !ultrix && !DCE_PASSWD */
+#endif /* !ultrix && !DCE_PASSWD && !PAM */
 #ifdef DCE_PASSWD
 	initDCE();
 #endif
