@@ -153,7 +153,7 @@ static int  descr();
 #endif
 #define DEF_MSGFONT     "-adobe-courier-medium-r-*-*-14-*-*-*-m-*-iso8859-1"
 #ifdef USE_MB
-# define DEF_FONTSET	DEF_FONT ## ",-*-24-*"
+#define DEF_FONTSET	DEF_FONT ## ",-*-24-*"
 #endif
 #define DEF_BG		"White"
 #define DEF_FG		"Black"
@@ -395,7 +395,9 @@ static XrmOptionDescRec genTable[] =
 	{"-validsound", ".validsound", XrmoptionSepArg, (caddr_t) NULL},
 	{"-invalidsound", ".invalidsound", XrmoptionSepArg, (caddr_t) NULL},
 #endif
-
+	{"-startCmd", ".startCmd", XrmoptionSepArg, (caddr_t) NULL},
+	{"-endCmd", ".endCmd", XrmoptionSepArg, (caddr_t) NULL},
+	{"-logoutCmd", ".logoutCmd", XrmoptionSepArg, (caddr_t) NULL},
 };
 
 #define genEntries (sizeof genTable / sizeof genTable[0])
@@ -556,6 +558,9 @@ static OptionStruct opDesc[] =
 	{"-validsound string", "sound to use when password is valid"},
 	{"-invalidsound string", "sound to use when password is invalid"},
 #endif
+	{"-startCmd string", "command to run at locktime"},
+	{"-endCmd string", "command to run when unlocking"},
+      {"-logoutCmd string", "command to run when automatically logging out"},
 };
 
 #define opDescEntries (sizeof opDesc / sizeof opDesc[0])
@@ -610,8 +615,10 @@ int         timeout;
 
 char       *fontname;
 char       *messagefont;
+
 #ifdef USE_MB
-char	   *fontsetname;
+char       *fontsetname;
+
 #endif
 char       *background;
 char       *foreground;
@@ -673,6 +680,10 @@ char       *invalidsound;
 
 #endif
 
+char       *startCmd;
+char       *endCmd;
+char       *logoutCmd;
+
 static argtype genvars[] =
 {
     {(caddr_t *) & allowaccess, "allowaccess", "AllowAccess", "off", t_Bool},
@@ -701,7 +712,7 @@ static argtype genvars[] =
 	{(caddr_t *) & fontname, "font", "Font", DEF_FONT, t_String},
     {(caddr_t *) & messagefont, "msgfont", "MsgFont", DEF_MSGFONT, t_String},
 #ifdef USE_MB
-	{(caddr_t *) & fontsetname, "fontset", "FontSet", DEF_FONTSET, t_String},
+    {(caddr_t *) & fontsetname, "fontset", "FontSet", DEF_FONTSET, t_String},
 #endif
     {(caddr_t *) & background, "background", "Background", DEF_BG, t_String},
     {(caddr_t *) & foreground, "foreground", "Foreground", DEF_FG, t_String},
@@ -758,6 +769,11 @@ static argtype genvars[] =
 	{(caddr_t *) & validsound, "validsound", "ValidSound", DEF_VALIDSOUND, t_String},
 	{(caddr_t *) & invalidsound, "invalidsound", "InvalidSound", DEF_INVALIDSOUND, t_String},
 #endif
+	{(caddr_t *) & startCmd, "startCmd", "StartCmd", "", t_String},
+	{(caddr_t *) & endCmd, "endCmd", "EndCmd", "", t_String},
+	{(caddr_t *) & logoutCmd, "logoutCmd", "LogoutCmd", "", t_String},
+
+
 #if 0
     /* These resources require special handling.  They must be examined
      * before the display is opened.  They are evaluated by individual
@@ -917,7 +933,8 @@ DumpResources(void)
 		(void) printf("%s.%s.%s: %g\n", classname, LockProcs[i].cmdline_arg,
 			      "saturation", LockProcs[i].def_saturation);
 		(void) printf("%s.%s.%s: %s\n", classname, LockProcs[i].cmdline_arg,
-			      "imagefile", LockProcs[i].def_imagefile);
+			      "imagefile",
+			      (LockProcs[i].def_imagefile) ? LockProcs[i].def_imagefile : "");
 		for (j = 0; j < LockProcs[i].msopt->numvarsdesc; j++)
 			(void) printf("%s.%s.%s: %s\n", classname, LockProcs[i].cmdline_arg,
 				      LockProcs[i].msopt->vars[j].name,
@@ -948,7 +965,7 @@ GetResource(XrmDatabase database, char *parentname, char *parentclassname,
 	char        buffer[1024];
 	char       *fullname;
 	char       *fullclassname;
-	int         len;
+	int         len, temp;
 
 	fullname = (char *) malloc(strlen(parentname) + strlen(name) + 2);
 	fullclassname = (char *) malloc(strlen(parentclassname) +
@@ -956,19 +973,21 @@ GetResource(XrmDatabase database, char *parentname, char *parentclassname,
 
 	(void) sprintf(fullname, "%s.%s", parentname, name);
 	(void) sprintf(fullclassname, "%s.%s", parentclassname, classname);
-	if (XrmGetResource(database, fullname, fullclassname, &type, &value)) {
+	temp = XrmGetResource(database, fullname, fullclassname, &type, &value);
+	(void) free((void *) fullname);
+	(void) free((void *) fullclassname);
+
+	if (temp) {
 		string = value.addr;
 		len = value.size - 1;
 	} else {
 		string = def;
-		if (string == NULL) {
+		if (!string || !*string) {
 			*valuep = NULL;
 			return;
 		}
 		len = strlen(string);
 	}
-	(void) free((void *) fullname);
-	(void) free((void *) fullclassname);
 
 	(void) strncpy(buffer, string, sizeof (buffer));
 	buffer[sizeof (buffer) - 1] = '\0';
@@ -1208,10 +1227,12 @@ open_display(void)
 #endif
 
 
-	if (!(dsp = XOpenDisplay(displayname))) {
-		buf = (char *) malloc(strlen(ProgramName) + strlen(displayname) + 80);
+	if (!(dsp = XOpenDisplay((displayname) ? displayname : ""))) {
+		buf = (char *) malloc(strlen(ProgramName) +
+			     ((displayname) ? strlen(displayname) : 0) + 80);
 		(void) sprintf(buf,
-		"%s: unable to open display %s.\n", ProgramName, displayname);
+			     "%s: unable to open display %s.\n", ProgramName,
+			       ((displayname) ? displayname : ""));
 		error(buf);
 		(void) free((void *) buf);	/* Should never get here */
 	}
@@ -1368,6 +1389,19 @@ getResources(int argc, char **argv)
 
 	XrmInitialize();
 
+#ifndef USE_MODULES
+	/*
+	 * Moved back because its annoying if you need -remote and
+	 * you do not know it.
+	 */
+	for (i = 0; i < argc; i++) {
+		if (!strncmp(argv[i], "-help", strlen(argv[i])))
+			Help();
+		if (!strncmp(argv[i], "-version", strlen(argv[i])))
+			Version();
+	}
+#endif
+
 	/*
 	 * get -name arg from command line so you can have different resource
 	 * files for different configurations/machines etc...
@@ -1432,7 +1466,7 @@ getResources(int argc, char **argv)
 		    "", &displayname);
 	GetResource(RDB, ProgramName, classname, "parent", "Parent", t_String,
 		    "", &parentname);
-	if (*parentname) {
+	if (parentname && *parentname) {
 		if (sscanf(parentname, "%ld", &parent))
 			parentSet = True;
 	}
@@ -1487,10 +1521,9 @@ getResources(int argc, char **argv)
 	/* read modules from modules directory */
 	atexit(UnloadModules);	/* make sure modules get unloaded */
 	LoadModules(modulepath);
-#endif
 
 	/*
-	 * I moved the search for help to here because now the modules
+	 * Moved the search for help to here because now the modules
 	 * have been loaded so they can be listed by help.
 	 */
 
@@ -1500,6 +1533,7 @@ getResources(int argc, char **argv)
 		if (!strncmp(argv[i], "-version", strlen(argv[i])))
 			Version();
 	}
+#endif
 
 	XrmParseCommand(&generalDB, genTable, genEntries, ProgramName, &argc, argv);
 /* PURIFY 4.0.1 on Solaris 2 reports an uninitialized memory read on the next
@@ -1664,7 +1698,7 @@ getResources(int argc, char **argv)
 		for (i = 0; i < (int) NMODEARGS; i++)
 			printvar(modename, modevars[i]);
 	}
-	if (!strcmp(visualname, "default") || (*visualname == '\0')) {
+	if (!visualname || !*visualname || !strcmp(visualname, "default")) {
 		VisualClassWanted = -1;
 		if (verbose)
 			(void) fprintf(stderr, "Using default visual class\n");
@@ -1705,25 +1739,16 @@ getResources(int argc, char **argv)
 void
 checkResources(void)
 {
+	extern char old_default_mode[];
 	int         i;
 
-	/* batchcount and size we allow negative
-	   to mean randomize up to that number */
-	if (delay < 0)
-		Syntax("-delay argument must not be negative.");
-	if (cycles < 0)
-		Syntax("-cycles argument must not be negative.");
-	if (ncolors < 2 || ncolors > 200)
-		Syntax("-ncolors argument must be between 2 and 200.");
-	if (saturation < 0.0 || saturation > 1.0)
-		Syntax("-saturation argument must be between 0.0 and 1.0.");
-	if (delta3d < 0.0 || delta3d > 20.0)
-		Syntax("-delta3d argument must be between 0.0 and 20.0.");
-
 	/* in case they have a 'xlock*mode: ' empty resource */
 	if (!mode || *mode == '\0')
 		mode = DEF_MODE;
 
+	if (!strcmp(old_default_mode, "")) {
+		(void) strcpy(old_default_mode, mode);
+	}
 	for (i = 0; i < numprocs; i++) {
 		if (!strcmp(LockProcs[i].cmdline_arg, mode)) {
 			set_default_mode(&LockProcs[i]);
@@ -1733,5 +1758,19 @@ checkResources(void)
 	if (i == numprocs) {
 		(void) fprintf(stderr, "Unknown mode: ");
 		Syntax(mode);
+	} else {
+
+		/* batchcount and size we allow negative
+		   to mean randomize up to that number */
+		if (delay < 0)
+			Syntax("-delay argument must not be negative.");
+		if (cycles < 0)
+			Syntax("-cycles argument must not be negative.");
+		if (ncolors < 2 || ncolors > 200)
+			Syntax("-ncolors argument must be between 2 and 200.");
+		if (saturation < 0.0 || saturation > 1.0)
+			Syntax("-saturation argument must be between 0.0 and 1.0.");
+		if (delta3d < 0.0 || delta3d > 20.0)
+			Syntax("-delta3d argument must be between 0.0 and 20.0.");
 	}
 }
