@@ -272,11 +272,11 @@ getWords(int screen, int screens)
 	}
 	p = &buf[screen * BUFSIZ];
 	*p = '\0';
-	if (strlen(message))
+	if (message && *message)
 		getwordsfrom = FROM_RESRC;
-	else if (strlen(messagefile)) {
+	else if (messagefile && *messagefile) {
 		getwordsfrom = FROM_FILE;
-	} else if (strlen(messagesfile)) {
+	} else if (messagesfile && *messagesfile) {
 		getwordsfrom = FROM_FORMATTEDFILE;
 	} else {
 		getwordsfrom = FROM_PROGRAM;
@@ -565,6 +565,23 @@ get_dir(char *fullpath, char *dir, char *filename)
 	filename[MAXNAMLEN - 1] = '\0';
 #ifdef VMS
 	upcase(filename);	/* VMS knows uppercase filenames only */
+#else
+	{
+		DIR        *dirp;
+
+		/* This finds a directory if there is no final slash */
+		if (filename[0] && (dirp = opendir(fullpath)) != NULL) {
+			(void) closedir(dirp);
+			(void) strncpy(dir, fullpath, DIRBUF - 1);
+			ip = strlen(fullpath);
+			if (ip < DIRBUF - 2) {
+				dir[ip] = '/';
+				dir[ip + 1] = '\0';
+			}
+			dir[DIRBUF - 1] = '\0';
+			filename[0] = '\0';
+		}
+	}
 #endif
 	if (debug)
 		(void) printf("get_dir %s %s %s\n", fullpath, dir, filename);
@@ -625,7 +642,7 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 	num_list_tmp = 0;
 	while ((new_entry = readdir(dirp)) != NULL) {
 #ifndef VMS
-		if (!strcmp(new_entry->d_name, ".") && !strcmp(new_entry->d_name, ".."))
+		if (!strcmp(new_entry->d_name, ".") || !strcmp(new_entry->d_name, ".."))
 			continue;
 #endif
 		if (select != NULL && !(*select) (new_entry))
