@@ -15,12 +15,6 @@ static const char sccsid[] = "@(#)visual.c	4.07 97/01/30 xlockmore";
 
 #include "xlock.h"
 
-#if defined(__cplusplus) || defined(c_plusplus)
-#define CLASS c_class
-#else
-#define CLASS class
-#endif
-
 extern Bool mono;
 extern Bool verbose;
 extern Bool debug;
@@ -31,7 +25,7 @@ showVisualInfo(XVisualInfo * Vis)
 	(void) fprintf(stderr, "Visual info: ");
 	(void) fprintf(stderr, "screen %d, ", Vis->screen);
 	(void) fprintf(stderr, "class %s, ", nameOfVisualClass(Vis->CLASS));
-	/* (void) fprintf(stderr, "visual %ld, ", Vis->visual);*/
+	/* (void) fprintf(stderr, "visual %ld, ", Vis->visual); */
 	(void) fprintf(stderr, "depth %d\n", Vis->depth);
 }
 
@@ -40,14 +34,14 @@ extern ScreenInfo *Scr;
 Bool
 fixedColors(ModeInfo * mi)
 {
-  Bool temp;
+	Bool        temp;
 
 #ifdef FORCEFIXEDCOLORS
 	/* pretending a fixed colourmap */
 	return TRUE;
 #else
 	/* get information about the default visual */
-	temp =  (!((MI_NPIXELS(mi) > 2) &&
+	temp = (!((MI_NPIXELS(mi) > 2) &&
 		  (MI_VISUALCLASS(mi) != StaticGray) &&
 		  (MI_VISUALCLASS(mi) != StaticColor) &&
 		  (MI_VISUALCLASS(mi) != TrueColor) &&
@@ -60,11 +54,11 @@ fixedColors(ModeInfo * mi)
 		  !MI_WIN_IS_INROOT(mi) &&
 		  MI_WIN_IS_INSTALL(mi)));
 #endif
-  if (debug) {
-    (void) printf("%s colors on screen %d\n", (temp) ? "fixed" : "writeable",
-       MI_SCREEN(mi));
-  }
-  return temp;
+	if (debug) {
+		(void) printf("%s colors on screen %d\n", (temp) ? "fixed" : "writeable",
+			      MI_SCREEN(mi));
+	}
+	return temp;
 }
 
 static struct visual_class_name {
@@ -193,9 +187,9 @@ getGLVisual(Display * display, int screen, XVisualInfo * wantVis, int mono)
 		if ((depthBits > 0) && doubleBuffer) {
 			return wantVis;
 		} else {
-	    XFree((char *) wantVis);	/* Free it up since its useless now. */
-      wantVis = NULL;
-    }
+			XFree((char *) wantVis);	/* Free it up since its useless now. */
+			wantVis = NULL;
+		}
 	}
 	/* If wantVis is useless, try glXChooseVisual() */
 	if (mono) {
@@ -402,22 +396,22 @@ defaultVisualInfo(Display * display, int screen)
 					/* Wanted visual not found so use default */
 					vTemplate.CLASS = DefaultVisual(display, screen)->CLASS;
 					if (info_list)
-					  XFree((char *) info_list);
+						XFree((char *) info_list);
 					info_list = XGetVisualInfo(display,
 								   VisualScreenMask | VisualDepthMask | VisualClassMask,
 							     &vTemplate, &n);
-          if (n == 0) {
+					if (n == 0) {
 						info_list = XGetVisualInfo(display,
-								   VisualScreenMask | VisualClassMask,
+									   VisualScreenMask | VisualClassMask,
 							     &vTemplate, &n);
 					}
 				}
 				if ((info_list == NULL) || (n == 0)) {
 					if (info_list)
-					  XFree((char *) info_list);
+						XFree((char *) info_list);
 					if (verbose)
 						(void) fprintf(stderr,
-							 "Could get visual for screen %d.\n", screen);
+							       "Could get visual for screen %d.\n", screen);
 					return;
 				}
 				if (VisualClassWanted == -1) {
@@ -428,18 +422,18 @@ defaultVisualInfo(Display * display, int screen)
 						    info->screen == screen)
 							break;
 				} else {
-						info = info_list;
-						for (i = 0; i < n; i++, info++) {
-							if (info->screen == screen &&
-							    info->CLASS == VisualClassWanted) {
-								break;
-							}
+					info = info_list;
+					for (i = 0; i < n; i++, info++) {
+						if (info->screen == screen &&
+						    info->CLASS == VisualClassWanted) {
+							break;
 						}
+					}
 					if (i == n) {
 						info = info_list;
 						for (i = 0; i < n; i++, info++) {
 							if (info->screen == screen &&
-						      info->visual == DefaultVisual(display, screen))
+							    info->visual == DefaultVisual(display, screen))
 								break;
 						}
 					}
@@ -462,5 +456,5 @@ defaultVisualInfo(Display * display, int screen)
 	Scr[screen].red_mask = info->red_mask;
 	Scr[screen].green_mask = info->green_mask;
 	Scr[screen].blue_mask = info->blue_mask;
- 	XFree((char *) info_list);	/* NOT info ... its modified */
+	XFree((char *) info_list);	/* NOT info ... its modified */
 }
