@@ -295,10 +295,12 @@ extern Bool use3d;
 
 extern char *fontname;
 extern char *messagefont;
+
 #ifdef USE_MB
-XFontSet fontset;
+XFontSet    fontset;
 extern char *fontsetname;
-XRectangle mbRect;
+XRectangle  mbRect;
+
 #endif
 extern char *background;
 extern char *foreground;
@@ -328,6 +330,11 @@ extern void play_sound(char *string);
 extern Bool sound;
 
 #endif
+
+extern char *startCmd;
+extern char *endCmd;
+extern char *logoutCmd;
+
 #ifdef USE_AUTO_LOGOUT
 extern int  logoutAuto;
 
@@ -388,6 +395,24 @@ static int  got_invalid = 0;
 
 #endif
 
+#if (defined( SYSV ) || defined( SVR4 )) && defined( SOLARIS2 ) && !defined( LESS_THAN_SOLARIS2_5 )
+#if !defined( __cplusplus ) && !defined( c_plusplus ) && !defined ( _SIGSET_T )
+#define _SIGSET_T
+typedef struct {		/* signal set type */
+	unsigned long __sigbits[4];
+} sigset_t;
+
+ /* This is good for gcc compiled on Solaris-2.5 but used on Solaris-2.6 */
+
+#endif
+#endif
+
+int         signalUSR1 = 0;
+int         signalUSR2 = 0;
+char        old_default_mode[20] = "";
+void        SigUsr1(int sig);
+void        SigUsr2(int sig);
+
 #define AllPointerEventMask \
 	(ButtonPressMask | ButtonReleaseMask | \
 	EnterWindowMask | LeaveWindowMask | \
@@ -698,7 +723,7 @@ mode_info(Display * display, int scrn, Window window, int iconic)
 void
 finish(Display * display, Bool closeDisplay)
 {
-  int screen;
+	int         screen;
 
 	for (screen = 0; screen < screens; screen++) {
 		if (Scr[screen].window != None) {
@@ -753,7 +778,7 @@ xio_error(Display * d)
 /* Convenience function for drawing text */
 static void
 putText(Display * display, Window window, GC gc,
-      char *string, int bold, int left, int *px, int *py)
+	char *string, int bold, int left, int *px, int *py)
 				/* which window */
 				/* gc */
 				/* text to write */
@@ -1030,6 +1055,38 @@ runMainLoop(int maxtime, int iconscreen)
 	started = seconds();
 
 	for (;;) {
+		if (signalUSR1 || signalUSR2) {
+			int         i;
+
+			if (signalUSR1) {
+#ifdef DEBUG
+				(void) printf("switch to mode %s\n", "blank");
+#endif
+				for (i = 0; i < numprocs; i++) {
+					if (!strcmp(LockProcs[i].cmdline_arg, "blank")) {
+						set_default_mode(&LockProcs[i]);
+						break;
+					}
+				}
+			} else {
+#ifdef DEBUG
+				(void) printf("switch to mode %s\n", old_default_mode);
+#endif
+				for (i = 0; i < numprocs; i++) {
+					if (!strcmp(LockProcs[i].cmdline_arg, old_default_mode)) {
+						set_default_mode(&LockProcs[i]);
+						break;
+					}
+				}
+			}
+
+			for (screen = 0; screen < screens; screen++) {
+				call_change_hook((LockStruct *) NULL,
+						 mode_info(dsp, screen, Scr[screen].window, False));
+			}
+
+			signalUSR1 = signalUSR2 = 0;
+		}
 		if (delay != lastdelay || maxtime != lastmaxtime) {
 			if (!delay || (maxtime && delay / 1000000 > maxtime)) {
 				repeat.tv_sec = maxtime;
@@ -1146,10 +1203,10 @@ ReadXString(char *s, int slen)
 	for (screen = 0; screen < screens; screen++)
 		if (thisscreen == screen) {
 			call_init_hook((LockStruct *) NULL,
-				  mode_info(dsp, screen, Scr[screen].icon, True));
+			     mode_info(dsp, screen, Scr[screen].icon, True));
 		} else {
 			call_init_hook((LockStruct *) NULL,
-			       mode_info(dsp, screen, Scr[screen].window, False));
+			  mode_info(dsp, screen, Scr[screen].window, False));
 		}
 	statusUpdate(True, thisscreen);
 	bp = 0;
@@ -1380,7 +1437,7 @@ getPassword(void)
 	(void) XGetWindowAttributes(dsp, Scr[screen].window, &xgwa);
 
 	ChangeGrabbedCursor(dsp, Scr[screen].window,
-       XCreateFontCursor(dsp, XC_left_ptr));
+			    XCreateFontCursor(dsp, XC_left_ptr));
 
 	XSetForeground(dsp, Scr[screen].gc, Scr[screen].bg_pixel);
 	XFillRectangle(dsp, Scr[screen].window, Scr[screen].gc,
@@ -1679,7 +1736,7 @@ justDisplay(Display * display)
 
 	for (screen = 0; screen < screens; screen++) {
 		call_init_hook((LockStruct *) NULL,
-       mode_info(display, screen, Scr[screen].window, False));
+		      mode_info(display, screen, Scr[screen].window, False));
 	}
 
 	while (not_done) {
@@ -1688,7 +1745,7 @@ justDisplay(Display * display)
 			(void) usleep(delay);
 			for (screen = 0; screen < screens; screen++)
 				call_callback_hook((LockStruct *) NULL,
-				mode_info(display, screen, Scr[screen].window, False));
+						   mode_info(display, screen, Scr[screen].window, False));
 #ifdef USE_AUTO_LOGOUT
 			checkLogout(display);
 #endif
@@ -1810,14 +1867,7 @@ lockDisplay(Display * display, Bool do_display)
 		play_sound(locksound);
 #endif
 	if (!allowaccess) {
-#if defined( SYSV ) || defined( SVR4 )
-#if !defined( __cplusplus ) && !defined( c_plusplus ) && defined( SOLARIS2 ) && !defined ( _SIGSET_T )	/* gcc does not like this but suncc -Xc does */
-#define _SIGSET_T
-		typedef struct {	/* signal set type */
-			unsigned long __sigbits[4];
-		} sigset_t;
-
-#endif
+#if defined( SYSV ) || defined( SVR4 ) || ( __VMS_VER >= 70000000 )
 		sigset_t    oldsigmask;
 		sigset_t    newsigmask;
 
@@ -1861,7 +1911,7 @@ lockDisplay(Display * display, Bool do_display)
 			XGrabServer(display);
 		XGrabHosts(display);
 
-#if defined( SYSV ) || defined( SVR4 )
+#if defined( SYSV ) || defined( SVR4 ) || ( __VMS_VER >= 70000000 )
 		(void) sigprocmask(SIG_SETMASK, &oldsigmask, &oldsigmask);
 #else
 		(void) sigsetmask(oldsigmask);
@@ -1988,12 +2038,13 @@ read_message()
 }
 
 #ifdef USE_MB
-static XFontSet
-createFontSet(Display *display, char *name)
+static      XFontSet
+createFontSet(Display * display, char *name)
 {
-	XFontSet xfs;
-	char     *def, **miss;
-	int      miss_count;
+	XFontSet    xfs;
+	char       *def, **miss;
+	int         miss_count;
+
 #define DEF_FONTSET2 "fixed,-*-14-*"
 
 	if ((xfs = XCreateFontSet(display, name, &miss, &miss_count, &def)) == NULL) {
@@ -2001,7 +2052,6 @@ createFontSet(Display *display, char *name)
 		if ((xfs = XCreateFontSet(display, DEF_FONTSET2, &miss, &miss_count, &def)) == NULL)
 			(void) fprintf(stderr, "Could not create FontSet %s\n", DEF_FONTSET2);
 	}
-
 	return xfs;
 }
 #endif
@@ -2016,6 +2066,15 @@ main(int argc, char **argv)
 	char       *buf;
 	int         tmp;
 	uid_t       ruid;
+	pid_t       cmd_pid = 0;
+
+#if defined( SYSV ) || defined( SVR4 ) || ( __VMS_VER >= 70000000 )
+	static sigset_t old_sigmask;
+
+#else
+	static int  old_sigmask;
+
+#endif
 
 #if ultrix
 	extern gid_t rgid;
@@ -2037,9 +2096,12 @@ main(int argc, char **argv)
 #endif
 
 #ifdef USE_MB
-        setlocale(LC_ALL, "");
+	setlocale(LC_ALL, "");
 #endif
 
+	(void) signal(SIGUSR1, SigUsr1);
+	(void) signal(SIGUSR2, SigUsr2);
+
 #ifdef OSF1_ENH_SEC
 	set_auth_parameters(argc, argv);
 #endif
@@ -2131,6 +2193,12 @@ main(int argc, char **argv)
 #endif
 	(void) setuid(ruid);
 
+#if 0
+	/* synchronize -- so I'm aware of errors immediately */
+	/* Too slow only for debugging */
+	XSynchronize(dsp, True);
+#endif
+
 #ifdef USE_MB
 	fontset = createFontSet(dsp, fontsetname);
 	XmbTextExtents(fontset, "A", 1, NULL, &mbRect);
@@ -2155,7 +2223,7 @@ main(int argc, char **argv)
 		unsigned int w, h;
 
 #ifdef FX
-		if (*glgeometry == '\0') {
+		if (!glgeometry || !*glgeometry) {
 			glwidth = DEF_GLW;
 			glheight = DEF_GLH;
 		} else {
@@ -2168,7 +2236,7 @@ main(int argc, char **argv)
 				glheight = MINICONH;
 		}
 #endif
-		if (*icongeometry == '\0') {
+		if (!icongeometry || !*icongeometry) {
 			iconwidth = DEF_ICONW;
 			iconheight = DEF_ICONH;
 		} else {
@@ -2184,7 +2252,7 @@ main(int argc, char **argv)
 			else if (iconheight > MAXICONH)
 				iconheight = MAXICONH;
 		}
-		if (*geometry == '\0') {
+		if (!geometry || !*geometry) {
 			fullscreen = True;
 		} else {
 			flags = XParseGeometry(geometry, &x, &y, &w, &h);
@@ -2337,7 +2405,7 @@ main(int argc, char **argv)
 					(void) XParseColor(dsp, cmap, DEF_BOTH3D, &C);
 #endif
 				Scr[screen].both_pixel = C.pixel =
-            pixels[0] | planemasks[0] | planemasks[1];
+					pixels[0] | planemasks[0] | planemasks[1];
 				XStoreColor(dsp, cmap, &C);
 			}
 
@@ -2356,7 +2424,7 @@ main(int argc, char **argv)
 			XChangeWindowAttributes(dsp, Scr[screen].window, CWBackPixel, &xswa);
 			/* this gives us these events from the root window */
 			XSelectInput(dsp, Scr[screen].window,
-         VisibilityChangeMask | ExposureMask);
+				     VisibilityChangeMask | ExposureMask);
 		} else
 #endif
 #ifdef USE_DTSAVER
@@ -2398,7 +2466,6 @@ main(int argc, char **argv)
 			       XA_WM_HINTS, XA_WM_HINTS, 32, PropModeReplace,
 					(unsigned char *) &xwmh, sizeof (xwmh) / sizeof (int));
 		}
-
 		fixColormap(dsp, Scr[screen].window, screen, ncolors, saturation,
 			    mono, install, inroot, inwindow, verbose);
 
@@ -2445,26 +2512,27 @@ main(int argc, char **argv)
 #define CIMASK CWBorderPixel | CWBackPixel | CWEventMask | CWColormap
 		if (nolock)
 			Scr[screen].icon = None;
-		else
+		else {
 			Scr[screen].icon = XCreateWindow(dsp, Scr[screen].window,
 				Scr[screen].iconpos.x, Scr[screen].iconpos.y,
 				    iconwidth, iconheight, 1, CopyFromParent,
 						 InputOutput, CopyFromParent,
 							 CIMASK, &xswa);
 #ifdef USE_BUTTON_LOGOUT
-		{
-			char        buf[1024];
-			int         w, h;
+			{
+				char        buf[1024];
+				int         w, h;
 
-			(void) sprintf(buf, " %s ", logoutButtonLabel);
-			w = XTextWidth(font, buf, strlen(buf));
-			h = font->ascent + font->descent + 2;
-			Scr[screen].button = XCreateWindow(dsp, Scr[screen].window,
+				(void) sprintf(buf, " %s ", logoutButtonLabel);
+				w = XTextWidth(font, buf, strlen(buf));
+				h = font->ascent + font->descent + 2;
+				Scr[screen].button = XCreateWindow(dsp, Scr[screen].window,
 					       0, 0, w, h, 1, CopyFromParent,
 						 InputOutput, CopyFromParent,
-							   CIMASK, &xswa);
-		}
+							      CIMASK, &xswa);
+			}
 #endif
+		}
 		XMapWindow(dsp, Scr[screen].window);
 		XRaiseWindow(dsp, Scr[screen].window);
 
@@ -2500,7 +2568,6 @@ main(int argc, char **argv)
 	} else if (!debug) {
 		GrabKeyboardAndMouse(dsp, Scr[0].window);
 	}
-
 	if (!nolock) {
 		XGetScreenSaver(dsp, &sstimeout, &ssinterval,
 				&ssblanking, &ssexposures);
@@ -2511,6 +2578,34 @@ main(int argc, char **argv)
 	(void) nice(nicelevel);
 
 	(void) XSetIOErrorHandler(xio_error);
+
+	/* start the command startcmd */
+	if (startCmd && *startCmd) {
+
+		if ((cmd_pid = FORK()) == -1) {
+			fprintf(stderr, "Failed to launch \"%s\"\n", startCmd);
+			perror(ProgramName);
+			cmd_pid = 0;
+		} else if (!cmd_pid) {
+#ifndef VMS
+			setpgid(0, 0);
+#endif
+#if 0
+#if (defined(sun) && defined(__svr4__)) || defined(sgi) || defined(__hpux) || defined(linux)
+			setsid();
+#else
+			setpgrp(getpid(), getpid());
+#endif
+#endif
+#if defined( SYSV ) || defined( SVR4 ) || ( __VMS_VER >= 70000000 )
+			(void) sigprocmask(SIG_SETMASK, &old_sigmask, &old_sigmask);
+#else
+			(void) sigsetmask(old_sigmask);
+#endif
+			system(startCmd);
+			exit(0);
+		}
+	}
 	if (nolock) {
 		(void) justDisplay(dsp);
 	} else if (lockdelay) {
@@ -2529,9 +2624,51 @@ main(int argc, char **argv)
 #endif
 	finish(dsp, True);
 
+	if (cmd_pid)
+#if defined(NO_KILLPG) || defined(__svr4__) || defined(sgi) || defined(__hpux) || defined(VMS)
+		kill(-cmd_pid, SIGTERM);
+#else
+		(void) killpg(cmd_pid, SIGTERM);
+#endif
+	if (endCmd && *endCmd) {
+		if ((cmd_pid = FORK()) == -1) {
+			fprintf(stderr, "Failed to launch \"%s\"\n", endCmd);
+			perror(ProgramName);
+			cmd_pid = 0;
+		} else if (!cmd_pid) {
+			system(endCmd);
+			exit(0);
+		}
+	}
 #ifdef VMS
 	return 1;
 #else
 	return 0;
 #endif
 }
+
+void
+SigUsr1(int sig)
+{
+#ifdef DEBUG
+	(void) printf("Signal %d received\n", sig);
+#endif
+	signalUSR1 = 1;
+	signalUSR2 = 0;
+
+	(void) signal(SIGUSR1, SigUsr1);
+	(void) signal(SIGUSR2, SigUsr2);
+}
+
+void
+SigUsr2(int sig)
+{
+#ifdef DEBUG
+	(void) printf("Signal %d received\n", sig);
+#endif
+	signalUSR1 = 0;
+	signalUSR2 = 1;
+
+	(void) signal(SIGUSR1, SigUsr1);
+	(void) signal(SIGUSR2, SigUsr2);
+}
