@@ -40,10 +40,10 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #ifdef USE_MB
-# ifdef __linux__
-#  define X_LOCALE
-# endif
-# include <X11/Xlocale.h>
+#ifdef __linux__
+#define X_LOCALE
+#endif
+#include <X11/Xlocale.h>
 #endif
 #else /* HAVE_CONFIG_H */
 
@@ -61,6 +61,7 @@
 #define HAVE_LIMITS_H 1
 #define HAVE_SYSLOG_H 1
 #define HAVE_GETHOSTNAME 1
+#define HAVE_UNISTD_H 1
 #ifdef VMS
 #if ( __VMS_VER < 70000000 )
 #ifdef USE_XVMSUTILS
@@ -83,7 +84,6 @@
 #endif /* __VMS_VER >= 70000000 */
 #else /* !VMS */
 #define HAVE_DIRENT_H 1
-#define HAVE_UNISTD_H 1
 #define HAVE_MEMORY_H 1
 #define HAVE_GETTIMEOFDAY 1
 #define GETTIMEOFDAY_TWO_ARGS 1
@@ -193,7 +193,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#ifdef SOLARIS2
+#if (defined( SYSV ) || defined( SVR4 )) && defined( SOLARIS2 ) && !defined( LESS_THAN_SOLARIS2_5 )
 struct hostent {
 	char       *h_name;	/* official name of host */
 	char      **h_aliases;	/* alias list */
@@ -253,6 +253,12 @@ struct hostent {
 #define DIRBUF          512	/* buffer size for fs-indep. dirs */
 #endif
 
+#if defined(__cplusplus) || defined(c_plusplus)
+#define CLASS c_class
+#else
+#define CLASS class
+#endif
+
 #if (!defined( AFS ) && defined( HAVE_SHADOW ) && !defined( OSF1_ENH_SEC ) && !defined( HP_PASSWDETC ) && !defined( VMS ) && !defined( PAM ))
 #define FALLBACK_XLOCKRC
 #endif
@@ -342,7 +348,7 @@ extern void setColormap(Display * display, Window window, Colormap map,
 extern void fixColormap(Display * display, Window window,
 			int screen, int ncolors, float saturation,
 	  Bool mono, Bool install, Bool inroot, Bool inwindow, Bool verbose);
-extern int  preserveColors(unsigned long black, unsigned long white, 
+extern int  preserveColors(unsigned long black, unsigned long white,
 			   unsigned long bg, unsigned long fg);
 extern Bool setupColormap(ModeInfo * mi, int *colors, Bool * truecolor,
   unsigned long *redmask, unsigned long *bluemask, unsigned long *greenmask);
@@ -352,7 +358,7 @@ extern char *nameOfVisualClass(int visualclass);
 extern Bool fixedColors(ModeInfo * mi);
 
 extern void getImage(ModeInfo * mi, XImage ** logo,
-		 int default_width, int default_height, unsigned char *default_bits,
+	  int default_width, int default_height, unsigned char *default_bits,
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 		     int default_xpm, char **name,
 #endif
@@ -360,9 +366,9 @@ extern void getImage(ModeInfo * mi, XImage ** logo,
 		     unsigned long *black);
 extern void destroyImage(XImage ** logo, int *graphics_format);
 extern void getPixmap(Display * display, Drawable drawable,
-    int default_width, int default_height, unsigned char *default_bits,
-    int *width, int *height, Pixmap *pixmap,
-    int *graphics_format);
+	  int default_width, int default_height, unsigned char *default_bits,
+		      int *width, int *height, Pixmap * pixmap,
+		      int *graphics_format);
 
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 extern void reserveColors(ModeInfo * mi, Colormap cmap,
@@ -402,6 +408,12 @@ extern char *strdup(char *);
 
 #endif
 
+#ifdef VMS
+#define FORK vfork
+#else
+#define FORK fork
+#endif
+
 #ifdef LESS_THAN_AIX3_2
 #undef NULL
 #define NULL 0
@@ -473,11 +485,12 @@ extern long LongRNG(void);
 
 #ifdef USE_MB
 extern XFontSet fontset;
-# define XTextWidth(font,string,length) \
+
+#define XTextWidth(font,string,length) \
 		XmbTextEscapement(fontset,string,length)
-# define XDrawString(display,window,gc,x,y,string,length) \
+#define XDrawString(display,window,gc,x,y,string,length) \
 		XmbDrawString(display,window,fontset,gc,x,y,string,length)
-# define XDrawImageString(display,window,gc,x,y,string,length) \
+#define XDrawImageString(display,window,gc,x,y,string,length) \
 		XmbDrawImageString(display,window,fontset,gc,x,y,string,length)
 #endif
 #endif /* __XLOCK_XLOCK_H__ */
