@@ -67,8 +67,6 @@ static LockStruct LockProcs[] =
 	{"gears",
 	 "Shows GL's gears"},
 #endif
-	{"geometry",
-	 "Shows morphing of a complete graph"},
 	{"grav",
 	 "Shows orbiting planets"},
 	{"helix",
