@@ -74,13 +74,13 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 extern char *c_Options[OPTIONS];
 extern Widget Menuoption;
 
-static int         numberprocess = -1;	/* PID of xlock */
+static pid_t  numberprocess = -1;	/* PID of xlock */
 
 extern void Setup_Option(Widget MenuBar);
 
 /* Widget */
 Widget      toplevel;
-static Widget      Labelxlock;
+static Widget Labelxlock;
 
 static Widget ScrolledListModes, PushButtons[PUSHBUTTONS], Toggles[TOGGLES];
 
@@ -205,16 +205,16 @@ f_ScrolledListModes(Widget w, XtPointer client_data, XtPointer call_data)
 		(void) wait(&n);
 	}
 #ifdef VMS
-	if ((numberprocess = vfork()) == -1)
-		/* This may also be a goodie for older Unix systems. J.J. */
+#define FORK vfork
 #else
-	if ((numberprocess = fork()) == -1)
+#define FORK fork
 #endif
+	if ((numberprocess = FORK()) == -1)
 		(void) fprintf(stderr, "Fork error\n");
 	else if (numberprocess == 0) {
 		(void) execlp(XLOCK, XLOCK, "-parent", numberwidget,
-		       "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
-           "-nolock", "-inwindow", 0);
+			      "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
+			      "-nolock", "-inwindow", 0);
 	}
 }
 
